/***************************************************************************
 *                                                                         *
 *  Filename : Arithmetic.c                                                *
 *                                                                         *
 *  Description : Routines for vector and matrix arithmetics.              *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *                         Modification History                            *
 *                                                                         *
 *  Date      Author       Comments                                        *
 * ----------------------------------------------------------------------  *
 *  6.8.93    R.Boedi      Created.                                        *
 * 30.3.94    MJ/RB        Added function VMult().                         *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 * Copyright ) 1993 Michael Joswig, Richard Boedi.  All rights reserved.   *
 *                                                                         *
 ***************************************************************************/

/*----------------------------- INCLUDES ----------------------------------*/

#include "IrrReps.h"
#include "Protos.h"

/*------------------------------- CODE ------------------------------------*/

/***************************************************************************
 *                                                                         *
 *  Function name : VectorNull ()                                          *
 *                                                                         *
 *  Description : Checks if a vector is the zero vector.                   *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : VectorNull (*Vector)                                        *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct Vector) *Vector:                                             *
 *       Pointer to a Vector structure.                                    *
 *                                                                         *
 *  Return value :                                                         *
 *    (BOOL)                                                               *
 *       TRUE, if the given vector is the zero vector,                     *
 *      FALSE, if not.                                                     *
 *                                                                         *
 ***************************************************************************/

BOOL  VectorNull (struct Vector *Vector)

{
unsigned long  i;

for (i = 0; i < Vector->length; i++)
   if (Vector->elements[i] != 0) return (FALSE);

return (TRUE);
}


/***************************************************************************
 *                                                                         *
 *  Function name : VectorCmp ()                                           *
 *                                                                         *
 *  Description : Checks if two vectors are equal.                         *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : VectorCmp (*Vector1, *Vector2)                              *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct Vector) *Vector1, *Vector2:                                  *
 *       Pointers to the vectors that should be compared.                  *
 *                                                                         *
 *  Return value :                                                         *
 *    (BOOL)                                                               *
 *       TRUE, if both vectors are the same.                               *
 *      FALSE, if not (includes the case where the vectors have different  *
 *                     lengths).                                           *
 *                                                                         *
 ***************************************************************************/

BOOL  VectorCmp (struct Vector *Vector1, struct Vector *Vector2)

{
unsigned long  i;

if (Vector1->length != Vector2->length) return (FALSE);
else
   for (i = 0; i < Vector1->length; i++)
      if (Vector1->elements[i] != Vector2->elements[i]) return (FALSE);

return (TRUE);
}


/***************************************************************************
 *                                                                         *
 *  Function name : VectorAdd ()                                           *
 *                                                                         *
 *  Description : Adds two vectors.                                        *
 *                Note that both vectors have to be of the same length.    *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : VectorAdd (*Vector1, *Vector2)                              *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct Vector) *Vector1, *Vector2:                                  *
 *       Pointers to the vectors that should be added.                     *
 *                                                                         *
 *  Return value :                                                         *
 *    (struct Vector *)                                                    *
 *       Returns a pointer to a struct vector which contains the sum of    *
 *       the vectors Vector1 and Vector2.                                  *
 *       A NULL pointer is returned if the vectors have different sizes    *
 *       or if no memory was available.                                    *
 *                                                                         *
 ***************************************************************************/

struct Vector *VectorAdd (struct Vector *Vector1, struct Vector *Vector2)

{
unsigned long  i;
struct Vector *SumVector = NULL;

if (Vector1->length == Vector2->length)
   if (SumVector = CreateVector (Vector1->length))
      for (i = 0; i < SumVector->length; i++)
         SumVector->elements[i] = Vector1->elements[i] + Vector2->elements[i];

return (SumVector);
}


/***************************************************************************
 *                                                                         *
 *  Function name : VectorMult ()                                          *
 *                                                                         *
 *  Description : Multiplies a vector waith a scalar.                      *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : VectorMult (Scalar, *Vector)                                *
 *                                                                         *
 *  Parameters :                                                           *
 *    (long) Scalar:                                                       *
 *       Scalar factor.                                                    *
 *    (struct Vector) *Vector:                                             *
 *       Pointer to a vector.                                              *
 *                                                                         *
 *  Return value :                                                         *
 *    (struct Vector *)                                                    *
 *       Returns a pointer to a struct vector which contains the scalar    *
 *       multiplication of Vector with Scalar.                             *
 *       A NULL pointer is returned if no memory was available.            *
 *                                                                         *
 ***************************************************************************/

struct Vector *VectorMult (long Scalar, struct Vector *Vector)

{
unsigned long  i;
struct Vector *MultVector = NULL;

if (MultVector = CreateVector (Vector->length))
      for (i = 0; i < MultVector->length; i++)
         MultVector->elements[i] = Scalar * Vector->elements[i];

return (MultVector);
}


/***************************************************************************
 *                                                                         *
 *  Function name : VectorRemainder ()                                     *
 *                                                                         *
 *  Description : Computes the remainders modulo Mod of all vector entries.*
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : VectorRemainder (*Vector, Mod)                              *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct Vector) *Vector:                                             *
 *       Pointer to a vector.                                              *
 *    (unsigned long) Mod:                                                 *
 *       The modulus.                                                      *
 *                                                                         *
 *  Return value :                                                         *
 *    (struct Vector *)                                                    *
 *       Returns a pointer to a struct vector which contains the remainders*
 *       of Vector modulo Mod.                                             *
 *       A NULL pointer is returned if no memory was available.            *
 *                                                                         *
 ***************************************************************************/

struct Vector *VectorRemainder (struct Vector *Vector, unsigned long Mod)

{
unsigned long  i;
struct Vector *RemainderVector = NULL;

if (RemainderVector = CreateVector (Vector->length))
      for (i = 0; i < RemainderVector->length; i++)
         RemainderVector->elements[i] = Vector->elements[i] % Mod;

return (RemainderVector);
}


/***************************************************************************
 *                                                                         *
 *  Function name : VMult ()                                               *
 *                                                                         *
 *  Description : Multiplies two unsigned longs and *detects* overflow.    *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : VMult (x, y, *Result)                                       *
 *                                                                         *
 *  Parameters :                                                           *
 *    (unsigned long) x,y:                                                 *
 *       Input parameters.                                                 *
 *    (unsigned long *) Result:                                            *
 *       Result = x * y.                                                   *
 *                                                                         *
 *  Return value :                                                         *
 *    (enum ErrorType)                                                     *
 *       NO_ERROR, OVERFLOW_ERROR.                                         *
 *                                                                         *
 ***************************************************************************/

enum ErrorType    VMult (unsigned long x, unsigned long y, unsigned long *Result)

{
int   bits = 0;

*Result = x * y;
for (; x != 0; bits++, x = x >> 1);
for (; y != 0; bits++, y = y >> 1);
if (bits > (sizeof(long) * 8)) return (OVERFLOW_ERROR);
else                           return (NO_ERROR);
}


/*--------------------------- END OF FILE ---------------------------------*/
