/***************************************************************************
 *                                                                         *
 *  Filename : Center.c                                                    *
 *                                                                         *
 *  Description : Gives the center of the simply connected group of a      *
 *                simple real Lie algebra.                                 *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *                         Modification History                            *
 *                                                                         *
 *  Date      Author       Comments                                        *
 * ----------------------------------------------------------------------  *
 *  5.8.93    M. Joswig    Created.                                        *
 *  6.8.93    M. Joswig    Changed order of generators.                    *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : CenterUC (*Group)                                           *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *                                                                         *
 *  Return value :                                                         *
 *    (struct Vector*) :                                                   *
 *       The result is a triple of integers (e,e',e'') that describes the  *
 *		 	center Z of the universal covering of *Group.                     *
 *			The abelian group Z has a generating system of cardinality at     *
 *			most 3.                                                           *
 *			Let Z = < z,z',z'' | z^e = {z'}^{e'} = {z''}^{e''} = 0 > be a     *
 *			finite (abelian) presentation of Z.                               *
 *       The set {z,z',z''} will be called a canonical generating system   *
 *			for Z.                                                            *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 * Copyright ) 1993  Richard Bdi, Michael Joswig.  All rights reserved.   *
 *                                                                         *
 ***************************************************************************/

/*----------------------------- INCLUDES ----------------------------------*/

#include "IrrReps.h"
#include "Protos.h"

/*------------------------------- CODE ------------------------------------*/

struct Vector *CenterUC (struct LiegroupClass *Group)
{
struct Vector *Center;
unsigned long  g,n = Group->rank;
long				i = Group->index;

if (!(Center = CreateVector(3))) return((struct Vector *)0);
                                /* not enough memory; this means: good bye */

Center->elements[0] = Center->elements[1] = Center->elements[2] = 1;
                                /* trivial center is default */

switch (Group->type)
	{
   case 'A':
		switch (Group->field) {
			case COMPLEX:
				Center->elements[0] = n + 1;
				break;
			case R:
				if (n == 1)           Center->elements[0] = 0;
				else
					switch (n % 4) {
						case 0: case 2: Center->elements[0] = 2; break;
						case 1:         Center->elements[0] = 4; break;
						case 3:         Center->elements[0] = Center->elements[1] = 2; break;
						}
				break;
			case C:
				if (i == 0)
					Center->elements[0] = n + 1;
				else /* i > 0 */ {
					g=gcd(n+1,i);
					if (g == 1) Center->elements[0] = 0;
					else {
						Center->elements[0] = g; Center->elements[1] = 0;
						}
					}
				break;
			case H:
				/* n is odd */
				Center->elements[0] = 2;
				break;
			}
      break;

   case 'B':
		switch (Group->field) {
			case COMPLEX:
				Center->elements[0] = 2;
				break;
			case R:
				switch (i) {
					case 0: case 1:
						Center->elements[0] = 2;
						break;
					case 2:
						Center->elements[0] = 2; Center->elements[1] = 0;
						break;
					default:
						Center->elements[0] = Center->elements[1] = 2;
						break;
					}
			case C: case H: /* do not exist */ break;
			}
   	break;
		
   case 'C':
		switch (Group->field) {
			case COMPLEX: case H:
				Center->elements[0] = 2;
				break;
			case R:
				if ((n % 2) == 0) {Center->elements[0] = 2; Center->elements[1] = 0;}
				else Center->elements[0] = 0;
				break;
			case C: /* does not exist */ break;
			}
      break;

   case 'D':
      if ((n % 2) == 0) {
			switch (Group->field) {
				case COMPLEX:
					Center->elements[0] = Center->elements[1] = 2;
					break;
				case R:
					switch (i) {
						case 0:
							Center->elements[0] = Center->elements[1] = 2;
							break;
						case 1:
							Center->elements[0] = 2;
							break;
						case 2:
							Center->elements[0] = 2; Center->elements[1] = 0;
							break;
						default:
							switch (i % 4) {
								case 0:
									Center->elements[0] = Center->elements[1] = Center->elements[2] = 2;
									break;
								case 2:
									Center->elements[0] = 2; Center->elements[1] = 4;
									break;
								case 1: case 3:
									Center->elements[0] = Center->elements[1] = 2;
									break;
								}
							break;
						}
					break;
				case C: /* does not exist */ break;
				case H:
					if ((n % 2) == 0) {Center->elements[0] = 2; Center->elements[1] = 0;}
					else               Center->elements[0] = 0;
					break;
				} /* switch (Group->field) */
			}
		else /* n is odd */ {
			switch (Group->field) {
				case COMPLEX:
					Center->elements[0] = 4;
					break;
				case R:
					switch (i) {
						case 0:
							Center->elements[0] = 4;
							break;
						case 1:
							Center->elements[0] = 2;
							break;
						case 2:
							Center->elements[0] = 2; Center->elements[1] = 0;
							break;
						default:
							switch (i % 4) {
								case 0: case 2:
									Center->elements[0] = 2; Center->elements[1] = 4;
									break;
								case 1: case 3:
									Center->elements[0] = Center->elements[1] = 2;
									break;
								}
							break;
						}
					break;
				case C: /* does not exist */ break;
				case H:
					Center->elements[0] = 0;
					break;
				}
			}
      break;

   case 'E':
		switch (n) {
			case 6:
				switch (Group->field) {
					case COMPLEX:
						Center->elements[0] = 3;
						break;
					case R:
						switch (i) {
							case -78:
								Center->elements[0] = 3;
								break;
							case -14:
								Center->elements[0] = 0;
								break;
							case 2:
								Center->elements[0] = 6;
								break;
							case 6:
								Center->elements[0] = 2;
								break;
							}
						break;
					case C: case H: /* do not exist */ break;
					}
				break;
			case 7:
				switch (Group->field) {
					case COMPLEX:
						Center->elements[0] = 2;
						break;
					case R:
						switch (i) {
							case -133: 
								Center->elements[0] = 2;
								break;
							case -25:
								Center->elements[0] = 0;
								break;
							case -5:
								Center->elements[0] = Center->elements[1] = 2;
								break;
							case 7:
								Center->elements[0] = 4;
								break;
							}
						break;
					case C: case H: /* do not exist */ break;
					}
				break;
			case 8:
				if ((Group->field == R) && (((i == -24) || (i == 8))))
					Center->elements[0] = 2;
				break;
			}
      break;

   case 'F':
		if ((Group->field == R) && (i == 4))
			Center->elements[0] = 2;
		break;

   case 'G':
		if ((Group->field == R) && (i == 2))
			Center->elements[0] = 2;
		break;
   }

return (Center);
}

/*--------------------------- END OF FILE ---------------------------------*/
