/***************************************************************************
 *                                                                         *
 *  Filename : CheckType.c                                                 *
 *                                                                         *
 *  Description : Checks if a Liegroup corresponding to the data stored    *
 *                in Group exists and if this group is of real or of       *
 *                complex type.                                            *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *                         Modification History                            *
 *                                                                         *
 *  Date      Author       Comments                                        *
 * ----------------------------------------------------------------------  *
 *  9.4.93    R.Bdi       Created.                                        *
 * 20.8.93    M.Joswig     Added handling of wildcards.                    *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : CheckType (*Group)                                          *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *                                                                         *
 *  Return value :                                                         *
 *    (enum AlgebraType)                                                   *
 *       DONT_EXIST   : No Liegroup of the given type exists.              *
 *       REAL_ALG     : A real Liegroup of the given type exists.          *
 *       COMPLEX_ALG  : A complex Liegroup of the given type exists.       *
 *       SET_OF_TYPES : There is at least one wildcard.                    *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 * Copyright ) 1993 Michael Joswig, Richard Bdi.  All rights reserved.    *
 *                                                                         *
 ***************************************************************************/

/*----------------------------- INCLUDES ----------------------------------*/

#include "IrrReps.h"
#include "Protos.h"

/*------------------------------- CODE ------------------------------------*/

enum AlgebraType  CheckType (struct LiegroupClass *Group)

{
enum AlgebraType  Algebra = DONT_EXIST;

/* check for wildcards */
if ((Group->type == WildCard) || (Group->rank == 0) || (Group->field == UNSPECIFIED)
			|| (Group->index == NO_INDEX)) return(SET_OF_TYPES);

/* completely specified type */
switch (Group->type)
   {
   case 'A':
      switch (Group->field)
         {
         case COMPLEX:
            Algebra = (Group->index == 0) ? COMPLEX_ALG : DONT_EXIST;
            break;
         case R:
            Algebra = (Group->index == 0) ? REAL_ALG : DONT_EXIST;
            break;
         case C:
            Algebra = ((Group->index >= 0) && (Group->index <= ((Group->rank + 1) / 2))) ? REAL_ALG : DONT_EXIST;
            if ((Group->rank == 1) && (Group->index == 1)) Algebra = DONT_EXIST;
            break;
         case H:
            Algebra = ((Group->rank % 2) && (Group->index == 0)) ? REAL_ALG : DONT_EXIST;
            if (Group->rank == 1) Algebra = DONT_EXIST;
            break;
         }

      if (Group->rank == 0) Algebra = DONT_EXIST;
      break;

   case 'B':
      switch (Group->field)
         {
         case COMPLEX:
            Algebra = (Group->index == 0) ? COMPLEX_ALG : DONT_EXIST;
            break;
         case R:
            Algebra = ((Group->index >= 0) && (Group->index <= Group->rank)) ? REAL_ALG : DONT_EXIST;
            break;
         case C:
            break;
         case H:
            break;
         }

      if (Group->rank < 2) Algebra = DONT_EXIST;
      break;

   case 'C':
      switch (Group->field)
         {
         case COMPLEX:
            Algebra = (Group->index == 0) ? COMPLEX_ALG : DONT_EXIST;
            break;
         case R:
            Algebra = (Group->index == 0) ? REAL_ALG : DONT_EXIST;
            break;
         case C:
            break;
         case H:
            Algebra = ((Group->index >= 0) && (Group->index <= (Group->rank / 2))) ? REAL_ALG : DONT_EXIST;
            break;
         }

      if (Group->rank < 3) Algebra = DONT_EXIST;
      break;

   case 'D':
      switch (Group->field)
         {
         case COMPLEX:
            Algebra = (Group->index == 0) ? COMPLEX_ALG : DONT_EXIST;
            break;
         case R:
            Algebra = ((Group->index >= 0) && (Group->index <= Group->rank)) ? REAL_ALG : DONT_EXIST;
            break;
         case C:
            break;
         case H:
            Algebra = ((Group->rank > 4) && (Group->index == (Group->rank / 2))) ? REAL_ALG : DONT_EXIST;
            break;
         }

      if (Group->rank < 4) Algebra = DONT_EXIST;
      break;

   case 'E':
      switch (Group->field)
         {
         case COMPLEX:
            Algebra = ((Group->rank > 5) && (Group->rank < 9) && (Group->index == 0)) ? COMPLEX_ALG : DONT_EXIST;
            break;
         case R:
            switch (Group->rank)
               {
               case 6:
                  Algebra = ((Group->index == 6) || (Group->index == 2) || (Group->index == -14) || (Group->index == -26) || (Group->index == -78)) ? REAL_ALG : DONT_EXIST;
                  break;
               case 7:
                  Algebra = ((Group->index == 7) || (Group->index == -5) || (Group->index == -25) || (Group->index == -133)) ? REAL_ALG : DONT_EXIST;
                  break;
               case 8:
                  Algebra = ((Group->index == 8) || (Group->index == -24) || (Group->index == -248)) ? REAL_ALG : DONT_EXIST;
                  break;
               }
            break;
         case C:
            break;
         case H:
            break;
         }
      break;

   case 'F':
      if (Group->rank == 4)
         switch (Group->field)
            {
            case COMPLEX:
               Algebra = (Group->index == 0) ? COMPLEX_ALG : DONT_EXIST;
               break;
            case R:
               Algebra = ((Group->index == 4) || (Group->index == -20) || (Group->index == -52)) ? REAL_ALG : DONT_EXIST;
               break;
            case C:
               break;
            case H:
               break;
            }
      break;

   case 'G':
      if (Group->rank == 2)
         switch (Group->field)
            {
            case COMPLEX:
               Algebra = (Group->index == 0) ? COMPLEX_ALG : DONT_EXIST;
               break;
            case R:
               Algebra = ((Group->index == 2) || (Group->index == -14)) ? REAL_ALG : DONT_EXIST;
               break;
            case C:
               break;
            case H:
               break;
            }
      break;
	
   }

return (Algebra);
}

/*--------------------------- END OF FILE ---------------------------------*/
