/***************************************************************************
 *                                                                         *
 *  Filename : Conversions.c                                               *
 *                                                                         *
 *  Description : Routines for converting structures.                      *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *                         Modification History                            *
 *                                                                         *
 *  Date      Author       Comments                                        *
 * ----------------------------------------------------------------------  *
 * 10.4.93    R.Boedi       Created.                                       *
 * 10.8.93    R.Boedi       Added Type parameter in VectorToString().      *
 * 21.3.94    R.Boedi       Replaced all occurances of atol() by strtol()  *
 *                          in order to get information about the success  *
 *                          of the conversion.                             *
 * 28.3.94    RB/MJ         Changed some SYNTAX_ERRORs to TYPE_ERRORs.     *
 * 28.3.94    R.Boedi       Added SQR_ULONG_MAX for range checking in      *
 *                          StringToLGClass ().                            *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 * Copyright ) 1993 Michael Joswig, Richard Boedi.  All rights reserved.   *
 *                                                                         *
 ***************************************************************************/

/*----------------------------- INCLUDES ----------------------------------*/

#include "IrrReps.h"
#include "Protos.h"

/*------------------------------- CODE ------------------------------------*/

/***************************************************************************
 *                                                                         *
 *  Function name : StringToVector ()                                      *
 *                                                                         *
 *  Description : Converts the input string (which is of the form          *
 *                (n1,n2,.....,nk), where ni is an integer) to a Vector    *
 *                structure.                                               *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : StringToVector (*trg, *Vector)                              *
 *                                                                         *
 *  Parameters :                                                           *
 *    (char) *Strg:                                                        *
 *       Pointer to the input string.                                      *
 *    (struct Vector) *Vector:    (OUTPUT)                                 *
 *       Pointer to a Vector structure which will be filled according to   *
 *       the contents of the input string.                                 *
 *                                                                         *
 *  Return value :                                                         *
 *    (enum ErrorType)                                                     *
 *         NO_ERROR, if conversion was successful                          *
 *     SYNTAX_ERROR, if the given vector doesn't match the correct syntax. *
 *     WRONG_LENGTH, if the number of entries in the input                 *
 *                   string doesn't match the length of the vector.        *
 *                                                                         *
 ***************************************************************************/

enum ErrorType    StringToVector (char *Strg, struct Vector *Vector)

{
unsigned long     i, index = 0;
char             *token, delimptr[2];
enum ErrorType    Input = NO_ERROR;


delimptr[0] = Delim; delimptr[1] = 0;

if ((Strg[0] != InnerLeft) || (Strg[strlen(Strg)-1] != InnerRight)) Input = SYNTAX_ERROR;
if (Vector->length == 0) Input = WRONG_LENGTH;

for (i = 1; i < (strlen(Strg) - 1); i++)
   if (!(isdigit(Strg[i])) && (Strg[i] != Delim)) Input = SYNTAX_ERROR;

if (Input == NO_ERROR)
   {
   token = strtok (Strg, delimptr) + 1;

   while (token)
      {
      *(Vector->elements + index) = strtol (token, NULL, 10);
      if ((*(Vector->elements + index) == LONG_MIN) || (*(Vector->elements + index) == LONG_MAX))
         Input = SYNTAX_ERROR;
      token = strtok (0, delimptr);
      if (index++ >= Vector->length)
         {
         Input = WRONG_LENGTH;
         break;
         }
      }
      if (index != Vector->length) Input = WRONG_LENGTH;
   }

return (Input);
}


/***************************************************************************
 *                                                                         *
 *  Function name : VectorToString ()                                      *
 *                                                                         *
 *  Description : Converts a vector to a string in the format x,....,x,x   *
 *                No line feed is added.                                   *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : VectorToString (*Vector, *String, *Spaces, Type)            *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct Vector) *Vector:                                             *
 *       Pointer to a Vector structure which will be converted.            *
 *    (char) *String:                                                      *
 *       Pointer to a string which will be filled up.                      *
 *       Be sure that the string is long enough to carry the output!       *
 *    (unsigned short) *Spaces:                                            *
 *       Specifies the minimal lengths for each entry.                     *
 *       This tag is omitted if a NULL pointer is supported.               *
 *    (enum ElementType) Type:                                             *
 *       If set to SIGNED, the elements of the vector are printed as       *
 *          signed values.                                                 *
 *       If set to UNSIGNED, the elements of the vector are printed as     *
 *          unsigned values.                                               *
 *                                                                         *
 *  Return value :                                                         *
 *    (char *)  Same as the string pointer above.                          *
 *                                                                         *
 ***************************************************************************/

char *VectorToString (struct Vector *Vector, char *String, unsigned short *Spaces, enum ElementType Type)

{
unsigned long     i;
char              Temp[COMPSIZE];

String[0] = 0;

if (Vector->length > 1)
   {
   for (i = 0; i < Vector->length - 1; i++)
      {
      if (Spaces)
         {
         if (Type == SIGNED)
            sprintf (Temp, "%*ld%c", Spaces[i], Vector->elements[i], Delim);
         else
            sprintf (Temp, "%*lu%c", Spaces[i], Vector->elements[i], Delim);
         }
      else
         {
         if (Type == SIGNED)
            sprintf (Temp, "%ld%c", Vector->elements[i], Delim);
         else
            sprintf (Temp, "%lu%c", Vector->elements[i], Delim);
         }
      strcat (String, Temp);
      }
   }

if (Spaces)
   {
   if (Type == SIGNED)
      sprintf (Temp, "%*ld", Spaces[Vector->length - 1], Vector->elements[Vector->length - 1]);
   else
      sprintf (Temp, "%*lu", Spaces[Vector->length - 1], Vector->elements[Vector->length - 1]);
   }
else
   {
   if (Type == SIGNED)
      sprintf (Temp, "%ld", Vector->elements[Vector->length - 1]);
   else
      sprintf (Temp, "%lu", Vector->elements[Vector->length - 1]);
   }

strcat (String, Temp);

return (String);
}


/***************************************************************************
 *                                                                         *
 *  Function name : StringToLGClass ()                                     *
 *                                                                         *
 *  Description : Converts the input string (which is of the form          *
 *                (t,r,f,i)) with                                          *
 *       t       : has to be one of types A, B, C, D, E, F, G              *
 *       r       : the rank of the Liegroup (rank > 0)                     *
 *       f       : the canonical field of the group (see Tits)             *
 *                   0 :   complex type                                    *
 *                   1 :   real numbers                                    *
 *                   2 :   complex numbers                                 *
 *                   4 :   quaternions                                     *
 *       i       : the index of the Liegroup                               *
 *                   (index >= 0, depends on type, field and rank)         *
 *                 No blanks in the vector are allowed.                    *
 *                into a struct LiegroupClass.                             *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : StringToLGClass (*LGClass, *Strg)                           *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LGClass) *LGClass:  (OUTPUT)                                 *
 *       Pointer to a LGClass structure which will be filled according to  *
 *       the contents of the input string.                                 *
 *    (char) *Strg:                                                        *
 *       Pointer to the input string.                                      *
 *                                                                         *
 *  Return value :                                                         *
 *    (enum ErrorType)                                                     *
 *         NO_ERROR, SYNTAX_ERROR, RANGE_ERROR, TYPE_ERROR.                *
 *                                                                         *
 ***************************************************************************/

enum ErrorType    StringToLGClass (struct LiegroupClass *LGClass, char *Strg)

{
char             *token, delimptr[2];
enum ErrorType    Input = NO_ERROR;
long					index;


delimptr[0] = Delim; delimptr[1] = 0;
if ((Strg[0] != InnerLeft) || (Strg[strlen(Strg)-1] != InnerRight)) Input = SYNTAX_ERROR;
Strg[strlen(Strg)-1] = 0;

if (Input == NO_ERROR)
   {
   token = strtok (Strg, delimptr) + 1;

   if (strlen (token) > 1) Input = SYNTAX_ERROR;
   else LGClass->type = token[0];
   if ((LGClass->type != WildCard) && ((LGClass->type < 'A') || (LGClass->type > 'G')))
      Input = TYPE_ERROR;

   token = strtok (0, delimptr);
   if (token[0] != WildCard)
      {
      LGClass->rank = (unsigned long)strtol (token, NULL, 10);
      if (LGClass->rank == 0)
         Input = TYPE_ERROR;
      }
   else LGClass->rank = 0;

   token = strtok (0, delimptr);
   if (token[0] != WildCard)
      {
      LGClass->field = (enum FieldType)strtol (token, NULL, 10);
      if ((LGClass->field != COMPLEX) && (LGClass->field != R) &&
          (LGClass->field != C) && (LGClass->field != H))
         Input = TYPE_ERROR;
      }
   else
      LGClass->field = UNSPECIFIED;

   token = strtok (0, delimptr);
   index = strtol (token, NULL, 10);
	if (index == NO_INDEX) Input = RANGE_ERROR;
   if (token[0] != WildCard) LGClass->index = index;
   else                      LGClass->index = NO_INDEX;
   if (strlen(token) == 0) Input = SYNTAX_ERROR;

   if (strtok (0, delimptr) != NULL) Input = SYNTAX_ERROR;
   }

return (Input);
}


/*--------------------------- END OF FILE ---------------------------------*/
