/***************************************************************************
 *                                                                         *
 *  Filename : Filter.c                                                    *
 *                                                                         *
 *  Description : Scans a table of representations and prints some of them *
 *                depending on numerous parameters.                        *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *                         Modification History                            *
 *                                                                         *
 *  Date      Author       Comments                                        *
 * ----------------------------------------------------------------------  *
 *  7.8.93    M.Joswig     Created.                                        *
 * 27.9.93    M.Joswig     Introduced -nolf switch.                        *
 * 31.3.94    R.Boedi      Suppressed filtering of empty tables.           *
 * 14.1.95    R.Boedi      Increased size of Header array in ScanAndPrint()*
 *                         from 256 to 4096. Little cosmetics.             *
 *                         Header and FormatString now global variables.   *
 * 27.1.95    RB/MJ        Added -v switch.                                *
 * 25.7.96    MJ           Corrected a few errors in places where          *
 *                         FormatStraing was built.                        *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : Filter                                                      *
 *              [-t <algebra>[ <algebra>[...]]] - specified algebras       *
 *             -[-g <group>[ <group>[...]]]     - specified groups         *
 *              [-dmin <minimum>]               - dimension >= <minimum>   *
 *              [-dmin <maximum>]               - dimension <= <maximum>   *
 *              [-c [R] [C] [H]]                - centralizer = R/C/H      *
 *             -[-w <weight>[ <weight>[...]]]   - specified weights        *
 *              [-old]                - convert from old input file format *
 *              [-nolf]                       - suppress LF within a table *
 *              [-v]                          - print version number       *
 *             The order of the parameters is not important.               *
 *                                                                         *
 *  Input :  (stdin)                                                       *
 *             Information about irreducible representations of simple     *
 *             real Lie algebras. Format described else where.             *
 *                                                                         *
 *  Output : (stdout)                                                      *
 *             A subset of the input according to specified parameters.    *
 *             If something goes wrong, then the output may be somewhat    *
 *             strange. The user is informed on stderr.                    *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 * Copyright (C) 1993 Richard Boedi, Michael Joswig. All rights reserved.  *
 *                                                                         *
 ***************************************************************************/

/*----------------------------- INCLUDES ----------------------------------*/

#include "IrrReps.h"
#include "Protos.h"
#include <string.h>

/*------------------------------- DATA ------------------------------------*/

unsigned long Limits[7][2]; /* 7 types, complex and real */

/*------------------------------- CODE ------------------------------------*/

enum ErrorType            	ScanAndPrint(void);
static unsigned long    	Minimum               	= 0,
									Maximum               	= ULONG_MAX;
static BOOL             	TypeRestricted        	= FALSE,
									GroupRestricted       	= FALSE,
									KernelRestricted      	= FALSE,
									DimensionRestricted   	= FALSE,
									CentralizerRestricted 	= FALSE,
									CentralizerAllowed[5],
									WeightRestricted      	= FALSE,
									OldInputFile			 	= FALSE,
                           NoLF							= FALSE;
static  LGList	     			TypeList						= NULL;

char                       FormatString[4096], Header[4096];


main (int argc, char *argv[])

{
unsigned long           i = 1;
short							j;
long							x;
enum ErrorType				TheInput = NO_ERROR, ExitCode;
struct LiegroupClass    Grp;

for (j = 0; j < 5; j++) CentralizerAllowed[j] = FALSE;

while ((i < argc) && (TheInput == NO_ERROR)) {

	if ((strcmp(argv[i],"-t") == 0) && (!TypeRestricted)) {
		TypeRestricted = TRUE; i++;
		while ((i < argc) && (argv[i][0] != '-') && (TheInput == NO_ERROR)) {
			if ((TheInput = StringToLGClass(&Grp,argv[i])) == NO_ERROR)
				TypeList = LGConstruct(Grp.type,Grp.rank,Grp.field,Grp.index,TypeList);
			i++;
			}
		continue;
		}

	if ((strcmp(argv[i],"-g") == 0) && (!GroupRestricted)) {
		GroupRestricted = TRUE; i++;
		while ((i < argc) && (argv[i][0] != '-') && (TheInput == NO_ERROR)) {
			i++;
			}
		fprintf(stderr,"Option -g not implemented.\n");
		continue;
		}

	if (strcmp(argv[i],"-dmin") == 0) {
		if ((i + 1 < argc)  && (Minimum == 0)) {
			DimensionRestricted = TRUE;
			x = atol(argv[i+1]);
			if ((x > 0) && (x <= Maximum)) Minimum = x;
			else TheInput = RANGE_ERROR;
			i += 2;
			}
		else TheInput = SYNTAX_ERROR;
		continue;
		}

	if (strcmp(argv[i],"-dmax") == 0) {
		if ((i + 1 < argc)  && (Maximum == ULONG_MAX)) {
			DimensionRestricted = TRUE;
			x = atol(argv[i+1]);
			if ((x > 0) && (x >= Minimum)) Maximum = x;
			else TheInput = RANGE_ERROR;
			i += 2;
			}
		else TheInput = SYNTAX_ERROR;
		continue;
		}
	
	if ((strcmp(argv[i],"-c") == 0) && (!CentralizerRestricted)) {
		CentralizerRestricted = TRUE; i++;
		while ((i < argc) && (argv[i][0] != '-')) {
			if (strcmp(argv[i],"R") == 0)       CentralizerAllowed[1] = TRUE;
			else {
				if (strcmp(argv[i],"C") == 0)    CentralizerAllowed[2] = TRUE;
				else {
					if (strcmp(argv[i],"H") == 0) CentralizerAllowed[4] = TRUE;
					else TheInput = SYNTAX_ERROR;
					}
				}
			i++;
			}
		continue;
		}
	
	if ((strcmp(argv[i],"-w") == 0) && (!WeightRestricted)) {
		WeightRestricted = TRUE; i++;
		while ((i < argc) && (argv[i][0] != '-') && (TheInput == NO_ERROR)) {
			i++;
			}
		fprintf(stderr,"Option -w not implemented.\n");
		continue;
		}

	if ((strcmp(argv[i],"-old") == 0) && (!OldInputFile)) {
		OldInputFile = TRUE;
		i++;
		continue;
		}
	
	if ((strcmp(argv[i],"-nolf") == 0) && (!NoLF)) {
		NoLF = TRUE;
		i++;
		continue;
		}
	
	if (strcmp(argv[i],"-v") == 0) {
		fprintf(stderr,"Version: %s [bug fix of 25-July-96]\n",VERSION);
		i++;
		continue;
		}

	TheInput = SYNTAX_ERROR; /* if you get here, then you skipped all the "continues" */
	}


switch (TheInput) {

	case NO_ERROR:
		/* If the centralizer is not restricted, then all possible centralizers are allowed. */
		if (!CentralizerRestricted)
			CentralizerAllowed[1] = CentralizerAllowed[2] = CentralizerAllowed[4] = TRUE;
		/* Now we are going to do something. */
		ExitCode = ScanAndPrint();
		switch (ExitCode) {
			case NO_ERROR:			break;
			case SYNTAX_ERROR: 	fprintf(stderr,"%s: Syntax error in input.\n",argv[0]); break;
			case MEM_ERROR:		fprintf(stderr,"%s aborted due to memory overflow.\n",argv[0]); break;
			default:				   fprintf(stderr,"WARNING: something strange happened (%d).\n",(int)ExitCode);
										ExitCode = NO_ERROR;
			}
		break;

	case SYNTAX_ERROR:
		fprintf(stderr,"Usage: %s [-t <algebra>[ <algebra>[...]]] [-g <group>[ <group>[...]]]\n",argv[0]);
		fprintf(stderr,"        [-dmin <minimum>] [-dmax <maximum>]\n");
		fprintf(stderr,"        [-c [R] [C] [H]] [-w <weight>[ <weight>[...]]]\n");
		fprintf(stderr,"        [-old] [-nolf] [-v]\n");
		ExitCode = SYNTAX_ERROR;
		break;
	
	case RANGE_ERROR:
		fprintf(stderr,"Usage: %s ... [-dmin <positive integer>] [-dmax <positive integer>] ... (dmin <= dmax)\n",argv[0]);
		ExitCode = RANGE_ERROR;
		break;
		
	case MEM_ERROR:
		fprintf(stderr,"%s aborted due to memory overflow.\n",argv[0]);
		ExitCode = MEM_ERROR;
		break;
	
	}

FreeLGList(TypeList);
exit((int)ExitCode);
}


enum ErrorType ScanAndPrint()
{
char  					bracket, HeaderPieces[256], WhitePlusChar[3],
							ActualOuterLeft = OuterLeft, ActualInnerLeft = InnerLeft,
							ActualInnerRight = InnerRight, ActualOuterRight = OuterRight;
unsigned long 			d_min, d_max, virtual_rank, i;
int  						cent;
long  					Eof;
unsigned long 			dim, f;
long  					center_uc[3], kernel[3][3];
short						sigma[3], no_of_generators, j;
unsigned long 		  *weight;
BOOL						ToBePrinted, HeaderPrinted, infinite_factor;
struct LiegroupClass	Grp;

/* We did not implement everything up to now. */
if (GroupRestricted)  fprintf(stderr,"WARNING: option -g ignored.\n");
if (WeightRestricted) fprintf(stderr,"WARNING: option -w ignored.\n");

/* We want to use this string, if we want to apply string functions to a single
   character. */
WhitePlusChar[0] = ' '; WhitePlusChar[2] = 0;

/* The old file format has always brackets as inner and outer parentheses. */
if (OldInputFile) {
	ActualOuterLeft = ActualInnerLeft = OuterLeft;
	ActualInnerRight = ActualOuterRight = OuterRight;
	}

/* This loops over all the algebras. */
while (scanf (" %c%lu %lu %ld %lu %lu",
				  &Grp.type, &Grp.rank, &f, &Grp.index, &d_min, &d_max) == 6) {
   HeaderPrinted = FALSE;
	Grp.field = (enum FieldType) f; /* do not know how to read an enumeration type */

	/* Skip the outer left parenthesis or bracket or whatelse. */
	WhitePlusChar[1] = ActualOuterLeft;
	scanf(WhitePlusChar);

	/* let us check whether we do like this particular algebra */
	if ((TypeRestricted) && (!LGinListNoCheck(&Grp,TypeList))) {
		/* We skip that thing completely. We just look for two brackets in sequence. */
		while (scanf(" %c",&bracket))
			if (bracket == ActualInnerRight)
				if ((scanf(" %c",&bracket)) && (bracket == ActualOuterRight)) break;
		}
	else {

		/* modify dimensions if necessary */
		if (d_min < Minimum) d_min = Minimum;
		if (d_max > Maximum) d_max = Maximum;

		/* print the algebra data */
		if (NoLF) sprintf (Header, "%c%011lu %011d %011ld ", Grp.type, Grp.rank, Grp.field, Grp.index);
		else sprintf (Header, "%c%lu %d %ld\n", Grp.type, Grp.rank, Grp.field, Grp.index);
		sprintf (HeaderPieces, "%lu %lu ", d_min, d_max);
		strcat (Header, HeaderPieces);
		if (!NoLF) strcat (Header, "\n");
		sprintf (HeaderPieces, "%c", ActualOuterLeft);
		strcat (Header, HeaderPieces);
		if (!NoLF) strcat(Header, "\n");

		/* if it is a complex algebra, we have pairs of weights */
		virtual_rank = (Grp.field == COMPLEX) ? Grp.rank * 2 : Grp.rank;
		/* get memory: if this fails, get out of here */
		if (!(weight = malloc (virtual_rank * sizeof(long)))) return (MEM_ERROR);

		/* first things first: read the head of the table */
		/* skip the first 2 columns (should contain 0) plus parentheses */
		/* and delimiters */
		sprintf (FormatString, " %c 0 %c 0 %c", ActualInnerLeft, Delim, Delim);
		scanf (FormatString);
      sprintf (HeaderPieces, "%c0%c 0%c ", InnerLeft, Delim, Delim);
		strcat (Header, HeaderPieces);
		/* skip the columns of the weight vectors */
		for (i = 0; i < virtual_rank; i++) {scanf (" 0 ,"); strcat (Header, "0,");}
		/* read the center Z of the universal covering */
		scanf (" %ld , %ld , %ld ,", &center_uc[0], &center_uc[1], &center_uc[2]);

		/* The old file format assumed a different order of the generators of Z. */
		if (OldInputFile) { /* compute the permutation sigma in Sym(3) */
			/* The center contains at most one infinite factor.
				If this is the case, then it is the first factor in the old format and
				it is represented by 0. */
			infinite_factor = (center_uc[0] == 0);
			/* How many (non-trivial) factors do we have? */
			no_of_generators =   (short)(center_uc[0] != 1) + (short)(center_uc[1] != 1)
									+ (short)(center_uc[2] != 1);
			/* now: rearrange */
			if (infinite_factor) {
				/* reorder the finite factors */
				for (j = 0; j < no_of_generators - 1; j++)
					sigma[j] = j + 4 - no_of_generators;
				/* the infinite factor will become the last non-trivial factor */
				sigma[no_of_generators-1] = 0;
				/* the trivial factors are now moved to the very end */
				for (j = no_of_generators; j < 3; j++)
					sigma[j] = j - no_of_generators + 1;
				}
			else {
				/* reorder the factors */
				for (j = 0; j < no_of_generators; j++)
					sigma[j] = j + 3 - no_of_generators;
				/* the trivial factors are now moved to the very end */
				for (j = no_of_generators; j < 3; j++)
					sigma[j] = j - no_of_generators;
				}
			}
		else { /* We do not want to change the order in this case. */
			sigma[0] = 0; sigma[1] = 1; sigma[2] = 2;}
		
		/* skip the rest of the line */
		for (Eof = scanf("%c",&bracket); Eof && (bracket != ActualInnerRight); Eof = scanf ("%c",&bracket));
		/* skip "," */
		scanf(" %c", &bracket);
		/* print three times this garbage */
		sprintf (HeaderPieces, " %ld%c%ld%c%ld%c ", center_uc[sigma[0]], Delim, center_uc[sigma[1]], Delim, center_uc[sigma[2]], Delim);
		strcat (Header, HeaderPieces);
		sprintf (HeaderPieces, "%ld%c%ld%c%ld%c ",  center_uc[sigma[0]], Delim, center_uc[sigma[1]], Delim, center_uc[sigma[2]], Delim);
		strcat (Header, HeaderPieces);
		sprintf (HeaderPieces, "%ld%c%ld%c%ld%c",  center_uc[sigma[0]], Delim, center_uc[sigma[1]], Delim, center_uc[sigma[2]], InnerRight);
		strcat (Header, HeaderPieces);

		/* first line successfully read */
		if (bracket == Delim) {

			sprintf (FormatString, " %c %%ld %c %%ld", ActualInnerLeft, Delim);

			/* now try an ordinary line */
			while (scanf(FormatString, &dim, &cent) == 2) {

				/* read it */
				for (i = 0; i < virtual_rank; i++) scanf (" , %ld", weight + i);
				for (i = 0; i < 3; i++)
					scanf(" , %ld , %ld , %ld", &kernel[i][0], &kernel[i][1], &kernel[i][2]);
				WhitePlusChar[1] = ActualInnerRight;
				scanf (WhitePlusChar);
				/* read "," or "]" */
				scanf (" %c", &bracket);

				/* are we going to print it? */
				ToBePrinted  = TRUE; /* we are optimistic (in this respect) */
				ToBePrinted &= ((Minimum <= dim) && (dim <= Maximum));
				ToBePrinted &= CentralizerAllowed[cent];

				/* finally, we print it */
				if (ToBePrinted) {

               /* print table header (only the first time) */
	   		   if (!HeaderPrinted) {
		   	      printf ("%s", Header);
                  HeaderPrinted = TRUE;
			         }

					printf(",");
					if (!NoLF) printf("\n");
					printf("%c%ld, %ld, ", InnerLeft, dim, cent);
					for (i = 0; i < virtual_rank; i++) printf("%ld,", weight[i]);
					printf(" %ld,%ld,%ld,",
						kernel[0][sigma[0]], kernel[0][sigma[1]], kernel[0][sigma[2]]);
					printf(" %ld,%ld,%ld,",
						kernel[1][sigma[0]], kernel[1][sigma[1]], kernel[1][sigma[2]]);
					printf(" %ld,%ld,%ld",
						kernel[2][sigma[0]], kernel[2][sigma[1]], kernel[2][sigma[2]]);
					printf("%c", InnerRight);
					}
				}
			}
		free(weight);

		if (HeaderPrinted)
		   {
   		if (!NoLF) printf("\n");
	   	printf("%c\n", OuterRight);
		   if (!NoLF) printf("\n");
         if (bracket != ActualOuterRight) return (SYNTAX_ERROR); /* this must be a syntax error in the input */
         }
   	} /* else */
	} /* this algebra is done */

return (NO_ERROR);
}

/*--------------------------- END OF FILE ---------------------------------*/
