/***************************************************************************
 *                                                                         *
 *  Filename : gcd.c                                                       *
 *                                                                         *
 *  Description : Routine for calculating the greatest common divisor      *
 *                of two long integer numbers including the cofactors.     *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *                         Modification History                            *
 *                                                                         *
 *  Date      Author       Comments                                        *
 * ----------------------------------------------------------------------  *
 * 10.4.93    R.Bdi       Created.                                        *
 * 24.8.93    R.Bdi       Added simple binary gcd algorithm.              *
 *            M.Joswig                                                     *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 * Copyright  1993 Michael Joswig, Richard Bdi.  All rights reserved.    *
 *                                                                         *
 ***************************************************************************/

/*------------------------------- CODE ------------------------------------*/

/***************************************************************************
 *                                                                         *
 *  Function name : gcd ()                                                 *
 *                                                                         *
 *  Description : Calculating the greatest common divisor d of two long    *
 *                integer numbers m,n.                                     *
 *                The binary algorithm is used (see Knuth, II).            *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : gcd (m,n)                                                   *
 *                                                                         *
 *  Parameters :                                                           *
 *    (unsigned long) m,n:                                                 *
 *       The two integers, both > 0                                        *
 *                                                                         *
 *  Return value :                                                         *
 *    (unsigned long)                                                      *
 *       The greatest common divisor of the numbers m and n.               *
 *       Returns 0 if m or n are 0.                                        *
 *                                                                         *
 ***************************************************************************/

unsigned long  gcd (unsigned long m, unsigned long n)

{
unsigned long  k;
long           t;

if ((m == 0) || (n == 0)) return (0);

for (k = 0; ((m % 2) == 0) && ((n % 2) == 0); k++)
   {
   m /= 2; n /= 2;
   }

if ((m % 2) != 0) t = -n;
else              t = m;

while (t != 0)
   {
   while (((t % 2) == 0) && (t != 0)) t /= 2;
   if (t > 0)  m = t;
   else        n = -t;
   t = m - n;
   }

return (m << k);
}



/***************************************************************************
 *                                                                         *
 *  Function name : gcd_cf ()                                              *
 *                                                                         *
 *  Description : Calculating the greatest common divisor d of two long    *
 *                integer numbers m,n including the cofactors u,v,         *
 *                i.e. we have d = u * m + v * n.                          *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : gcd_cf (m,n,*u,*v)                                          *
 *                                                                         *
 *  Parameters :                                                           *
 *    (unsigned long) m,n:                                                 *
 *       The two integers, both > 0                                        *
 *    (long) *u, *v:      (OUTPUT)                                         *
 *       Pointer to long integers which are filled up with the cofactors.  *
 *                                                                         *
 *  Return value :                                                         *
 *    (unsigned long)                                                      *
 *       The greatest common divisor of the numbers m and n.               *
 *       Returns 0 if m or n are 0. In this case the variables u and v are *
 *       unaffected.                                                       *
 *                                                                         *
 ***************************************************************************/

unsigned long  gcd_cf (unsigned long m, unsigned long n, long *u, long *v)

{
long           u1 = 1, v1 = 0, t = 0;
unsigned long  q = 0, r = 0;

if ((m * n) == 0) return (0);
else
   {
   *u = 0; *v = 1;
   q = m / n; r = m % n;

   while (r != 0)
      {
      m = n; n = r;
      t = u1; u1 = *u; *u = t - q * *u;
      t = v1; v1 = *v; *v = t - q * *v;
      q = m / n; r = m % n;
      }

   return (n);
   }
}


/*--------------------------- END OF FILE ---------------------------------*/
