/***************************************************************************
 *                                                                         *
 *  Filename : Image.c                                                     *
 *                                                                         *
 *  Description : Computes the kernel of an irreducible representation.    *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *                         Modification History                            *
 *                                                                         *
 *  Date      Author       Comments                                        *
 * ----------------------------------------------------------------------  *
 *  5.8.93    M. Joswig    Created.                                        *
 *  6.8.93    M. Joswig    Changed order of generators.                    *
 * 19.8.93    M. Joswig    Removed error in the An,C,i>0 case.             *
 * 27.8.93    R. Boedi     Removed error in the Dn,R,2 case.               *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : ImageFundRep (*Group,j)                                     *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *    (unsigned long) j:                                                   *
 *       Number of fundamental representation.                             *
 *                                                                         *
 *  Return value :                                                         *
 *    (struct Vector*) :                                                   *
 *       The result is a quadruple [d,d',d'',m].                           *
 *       If rho is the j-th fundamental representation (of the given       *
 *       algebra), and if z,z',z'' are the canonical generators of the     *
 *       center Z of the univeral covering, then the image of Z under rho  *
 *       is contained in a cyclic group C of order m. If xi is a generator *
 *       of C, then rho(z) = xi^d, rho(z') = xi^{d'}, rho(z'') = xi^{d''}. *
 *       If an error occurs, then a zero pointer is returned.              *
 *                                                                         *
 *                                                                         *
 *  Synopsis : ImageRep (*Group,*Weight)                                   *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *    (struct Vector) *Weight:                                             *
 *       Highest weight of some representation.                            *
 *       _NOTE_ that here *Weight is _NOT_ a pair of two weights in the    *
 *       complex case.                                                     *
 *                                                                         *
 *  Return value :                                                         *
 *    (struct Vector*) :                                                   *
 *       The result is a quadruple [m,d,d',d''].                           *
 *       If rho is the representation corresponding to *Weight (of the     *
 *			given algebra), and if z,z',z'' are the canonical generators of   *
 *			the center Z of the univeral covering, then the image of Z under  *
 *			rho is contained in a cyclic group C of order m.                  *
 *			If xi is a generator of C, then rho(z) = xi^d, rho(z') = xi^{d'}, *
 *			rho(z'') = xi^{d''}.                                              *
 *       If an error occurs, then a zero pointer is returned.              *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 * Copyright ) 1993  Richard Bdi, Michael Joswig.  All rights reserved.   *
 *                                                                         *
 ***************************************************************************/

/*----------------------------- INCLUDES ----------------------------------*/

#include "IrrReps.h"
#include "Protos.h"

/*------------------------------- CODE ------------------------------------*/

struct Vector *ImageFundRep (struct LiegroupClass *Group,unsigned long j)
{
struct Vector *Image;
long				a,b;
unsigned long  g,l,n = Group->rank;
long				i = Group->index;

if (!(Image=CreateVector(4))) return((struct Vector *)0);
                                /* not enough memory; this means: good bye */

/* _NOTE_ that Image's entries are initialized to 0;
   i.e. the default means that each generator is mapped to the neutral element. */
switch (Group->type)
	{
   case 'A':
      Image->elements[3] = n + 1;
		switch (Group->field) {
			case COMPLEX:
				Image->elements[0] = j;
				break;
			case R:
				if (n == 1) Image->elements[0] = j;
				else
					switch (n % 4) {
						case 0: case 2: break;
						case 1: l = n / 4; Image->elements[0] = j*(2*l+1); break;
						case 3: l = n / 4; Image->elements[0] = Image->elements[1] = j*(2*l+2);
						}
				break;
			case C:
				if (i == 0)
					Image->elements[0] = j;
				else /*i>0*/ {
					/* _NOTE_: this situation is difficult.
					   If the gcd equals 1, then the center is cyclic. */
					g=gcd_cf(n+1,i,&a,&b);
					if (g == 1)
						Image->elements[0] = j*b;
					else {
						Image->elements[0] = j*(n+1)/g; Image->elements[1] = j*b;
						}
					}
				break;
			case H:
				/* n is odd */
				l = n / 2; Image->elements[0] = j*(l+1);
				break;
			}
      break;

   case 'B':
      Image->elements[3] = 2;
		switch (Group->field) {
			case COMPLEX:
				if (j == n) Image->elements[0] = 1;
				break;
			case R:
				if     (i <= 1)
					{if (j == n) Image->elements[0] = 1;}
				else /* i >= 2 */
					{if (j == n) {Image->elements[0] = Image->elements[1] = 1;}}
				break;
			case C: case H: /* do not exist */ break;
			}
   	break;
		
   case 'C':
		Image->elements[3] = 2; 
		switch (Group->field) {
			case COMPLEX: case R: case H: Image->elements[0] = j; break;
			case C: /* does not exist */ break;
			}
      break;

   case 'D':
      if ((n % 2) == 0) {
      	Image->elements[3] = 2;
			switch (Group->field) {
				case COMPLEX:
					if (j < n - 1)     Image->elements[0] = Image->elements[1] = j;
					else {
						if (j == n - 1) Image->elements[1] = 1;
						else            Image->elements[0] = 1;
						}
					break;
				case R:
					switch (i) {
						case 0:
							if   (j <  n - 1)    		Image->elements[0] = Image->elements[1] = j;
							else {
								if   (j == n - 1) 		Image->elements[1] = 1;
								else /* j == n */ 		Image->elements[0] = 1;
								}
							break;
						case 1:
							if (j >= n - 1) 				Image->elements[0] = 1;
							break;
						case 2:
							if (j < n - 1) 				Image->elements[1] = j;
							else {
								if   (j == n - 1) 		Image->elements[0] = 1;
								else /* j == n */ 		Image->elements[0] = Image->elements[1] = 1;
								}
							break;
						default:
							switch (i % 4) {
								case 0:
									if (j <  n - 1)      Image->elements[2] = j;
									else {
										if   (j == n - 1) Image->elements[0] = Image->elements[1] = 1;
										else /* j == n */ Image->elements[0] = Image->elements[1] = Image->elements[2] = 1;
										}
									break;
								case 2:
									if (j <  n - 1)      Image->elements[1] = j;
									else {
										if   (j == n - 1) Image->elements[0] = 1;
										else /* j == n */ Image->elements[0] = Image->elements[1] = 1;
										}
									break;
								case 1: case 3:
									if (j >= n - 1) Image->elements[0] = Image->elements[1] = 1;
									break;
								}
							break;
						}
					break;
				case C: /* does not exist */ break;
				case H:
					if (j < n - 1)     Image->elements[0] = j;
					else {
						if (j == n - 1) Image->elements[1] = 1;
						else            Image->elements[0] = Image->elements[1] = 1;
						}
					break;
				} /* switch (Group->field) */
			}
		else /* n is odd */ {
      	Image->elements[3] = 4;
			switch (Group->field) {
				case COMPLEX:
					if (j < n - 1)     Image->elements[0] = 2*j;
					else {
						if (j == n - 1) Image->elements[0] = 1;
						else            Image->elements[0] = 3;
						}
					break;
				case R:
					switch (i) {
						case 0:
							if (j < n - 1)
								Image->elements[0] = 2*j;
							else {
								if   (j == n - 1) Image->elements[0] = 1;
								else /* j == n */ Image->elements[0] = 3;
								}
							break;
						case 1:
							if  (j >= n - 1) Image->elements[0] = 2;
							break;
						default:
							switch (i % 2) {
								case 0:
									if (j < n - 1)
										Image->elements[1] = 2*j;
									else {
										if   (j == n - 1) {Image->elements[0] = 2; Image->elements[1] = 1;}
										else /* j == n */ {Image->elements[0] = 2; Image->elements[1] = 3;}
										}
									break;
								case 1:
									if (j >= n - 1) Image->elements[0] = Image->elements[1] = 2;
									break;
								}
							break;
						}
					break;
				case C: /* does not exist */ break;
				case H:
					if        (j <  n - 1)  Image->elements[0] = j;
					else {
						if     (j == n - 1)  Image->elements[0] = 1;
						else /* j ==   n  */ Image->elements[0] = 3;
						}
					break;
				}
			}
      break;

   case 'E':
		switch (n) {
			case 6:
				Image->elements[3] = 3;
				switch (Group->field) {
					case COMPLEX:
						switch (j) {
							case 1: case 5: Image->elements[0] = 1; break;
							case 3: case 6: Image->elements[0] = 2; break;
							case 2: case 4: break;
							};
						break;
					case R:
						switch (i) {
							case -78: case -14:case 2: 
								switch (j) {
									case 1: case 5: Image->elements[0] = 1; break;
									case 3: case 6: Image->elements[0] = 2; break;
									case 2: case 4: break;
									};
								break;
							case -26: case 6:	break;
							}
						break;
					case C: case H: /* do not exist */ break;
					}
				break;
			case 7:
				Image->elements[3] = 2;
				switch (Group->field) {
					case COMPLEX:
						switch (j) {
							case 2: case 5: case 7:         Image->elements[0] = 1; break;
							case 1: case 3: case 4: case 6: break;
							};
						break;
					case R:
						switch (i) {
							case -133: case -25: case 7:
								switch (j) {
									case 2: case 5: case 7:         Image->elements[0] = 1; break;
									case 1: case 3: case 4: case 6: break;
									};
								break;
							case -5:
								switch (j) {
									case 2: case 5: case 7:         Image->elements[1] = 1; break;
									case 1: case 3: case 4: case 6: break;
									};
								break;
							}
						break;
					case C: case H: /* do not exist */ break;
					}
				break;
			case 8:
				Image->elements[3] = 1;
				break;
			}
      break;

   case 'F':
      Image->elements[3] = 1;
      break;

   case 'G':
      Image->elements[3] = 1;
      break;
   }

return (Image);
}


struct Vector *ImageRep (struct LiegroupClass *Group,struct Vector *Weight)
{
struct Vector *Image,*FundImage;
unsigned long  j,n = Group->rank;

if (!(Image = CreateVector(4))) return((struct Vector *)0);
                                /* not enough memory; this means: good bye */

/* _NOTE_ that Image is initialized to (0,0,...,0) */
for (j = 1; j <= n; j++) {
	FundImage = ImageFundRep(Group,j);
	Image->elements[0] += Weight->elements[j-1] * FundImage->elements[0];
	Image->elements[1] += Weight->elements[j-1] * FundImage->elements[1];
	Image->elements[2] += Weight->elements[j-1] * FundImage->elements[2];
	Image->elements[3]  = FundImage->elements[3]; /* this is not very efficient */
	FreeVector(FundImage);
	}

/* reduce mod order of group and make numbers non-negative */
Image->elements[0] %= Image->elements[3];
if (Image->elements[0] < 0) Image->elements[0] += Image->elements[3];
Image->elements[1] %= Image->elements[3];
if (Image->elements[1] < 0) Image->elements[1] += Image->elements[3];
Image->elements[2] %= Image->elements[3];
if (Image->elements[2] < 0) Image->elements[2] += Image->elements[3];

return(Image);
}

/*--------------------------- END OF FILE ---------------------------------*/
