/***************************************************************************
 *                                                                         *
 *  Filename : Index.c                                                     *
 *                                                                         *
 *  Description : Computes the index of an irreducible representation.     *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *                         Modification History                            *
 *                                                                         *
 *  Date      Author       Comments                                        *
 * ----------------------------------------------------------------------  *
 *  6.8.93    M. Joswig    Created.                                        *
 * 27.8.93    R. Boedi     Fixed case DnR.                                 *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : IndexFundRep (*Group,j)                                     *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *    (unsigned long) j:                                                   *
 *       Number of fundamental representation.                             *
 *                                                                         *
 *  Return value :                                                         *
 *    (unsigned int) :                                                     *
 *       The result is the index of the j-th fundamental representation    *
 *		 	of *Group. It is either 0 or 1.                                   *
 *                                                                         *
 *                                                                         *
 *  Synopsis : IndexRep (*Group,*Weight)                                   *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *    (struct Vector) *Weight:                                             *
 *       Highest weight of some representation.                            *
 *                                                                         *
 *  Return value :                                                         *
 *    (unsigned int) :                                                     *
 *       The result is the index of the irreducible representation of      *
 *		 	*Group which has *Weight as its highest weight.                   *
 *       It is either 0 or 1.                                              *
 *       _NOTE_ that this function does not make any sense for a complex   *
 *       *Group. Nevertheless 0 will be returned in this case.             *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 * Copyright ) 1993  Richard Bdi, Michael Joswig.  All rights reserved.   *
 *                                                                         *
 ***************************************************************************/

/*----------------------------- INCLUDES ----------------------------------*/

#include "IrrReps.h"
#include "Protos.h"

/*------------------------------- CODE ------------------------------------*/

unsigned int IndexFundRep (struct LiegroupClass *Group, unsigned long j)
{
unsigned int	alpha = 0; /* this will contain the index that is to be computed */
unsigned long  n = Group->rank;
long				i = Group->index;
						        /* do not mix things up: this index is something different */

/* We are just going to list the cases in which the index is not 0. */
switch (Group->type)
	{
   case 'A':
		switch (Group->field) {
			case R: break;
			case C: if ((((n+1) % 2) == 0) && (j == (n+1)/2)) alpha = ((n+1)/2 - i) % 2; break;
			case H: alpha = j % 2; break;
			}
      break;

   case 'B':
		switch (Group->field) {
			case R: if ((j == n) && ((((n - i) % 4) == 1) || (((n - i) % 4) == 2))) alpha = 1; break;
			case C: case H: /* do not exist */ break;
			}
   	break;

   case 'C':
		switch (Group->field) {
			case R: break;
			case C: /* does not exist */ break;
			case H: alpha = j % 2; break;
			}
      break;

   case 'D':
		switch (Group->field) {
			case R: if (((j == n) || (j == (n-1))) && (((n-i) % 4) != 0)) alpha = 1; break;
			case C: /* does not exist */ break;
			case H:
				if (j < n - 1) alpha = j % 2;
				else if ((j == n) || ((n % 2) != 0)) alpha = 1;
				break;
			}
      break;

   case 'E':
		switch (n) {
			case 6:
				switch (Group->field) {
					case R:
						switch (i) {
							case -78: case -14: case 2:
								if ((j == 1) || (j == 3) || (j == 5) || (j == 6)) alpha = 1;
								break;
							case -26: case 6:
								break;
							}
						break;
					case C: case H: /* do not exist */ break;
					}
				break;
			case 7:
				switch (Group->field) {
					case R:
						switch (i) {
							case -133:  case -5:
								if ((j == 2)  || (j == 5) || (j == 7)) alpha = 1;
								break;
							case -25: case 7:
								break;
							}
						break;
					case C: case H: /* do not exist */ break;
					}
				break;
			case 8:
				break;
			}
      break;

   case 'F':
      break;

   case 'G':
      break;
   }

return (alpha);
}

unsigned int IndexRep (struct LiegroupClass *Group,struct Vector *Weight)
{
unsigned int   alpha = 0;
unsigned long  j,n = Group->rank;

/* The index is multiplicative map into the cyclic group of order 2;
   this allows some (obvious) optimization. */
for (j = 1; j <= n; j++)
	switch (Weight->elements[j-1] % 2) {
		case 0: break;
		case 1: alpha += IndexFundRep(Group,j); break;
		}

/* reduce mod 2 */
return(alpha % 2);
}


/***************************************************************************
 *                                                                         *
 *  Function name : Centralizer ()                                         *
 *                                                                         *
 *  Description : Determines the centralizer of a representation.          *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : Centralizer (Group, Weight)                                 *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass *) Group:                                      *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *    (struct Vector *) Weight:                                            *
 *       The dominant weight of the representation.                        *
 *                                                                         *
 *  Return value :                                                         *
 *    (enum FieldType) :                                                   *
 *       R, C, or H.                                                       *
 *       If no memory was available, UNSPECIFIED is returned.              *
 *                                                                         *
 ***************************************************************************/

enum FieldType    Centralizer (struct LiegroupClass *Group, struct Vector *Weight)

{
struct Vector     weight1, weight2, *conjweight;
enum FieldType    centralizer;


if (Group->field == COMPLEX)
   {
   weight1.length = weight2.length = Weight->length / 2;
   weight1.elements = Weight->elements;
   weight2.elements = Weight->elements + weight1.length; 

   if (VectorCmp (&weight1, &weight2)) centralizer = R;
   else                                centralizer = C;
   }
else
   if (conjweight = WeightConj (Group, Weight))
      {
      if (!VectorCmp (Weight, conjweight)) centralizer = C;
      else
         {
         if (IndexRep (Group, Weight) == 0) centralizer = R;
         else                               centralizer = H;
         }
      FreeVector (conjweight);
      }
   else  centralizer = UNSPECIFIED;

return (centralizer);
}

/*--------------------------- END OF FILE ---------------------------------*/
