/***************************************************************************
 *                                                                         *
 *  Filename : Kernel.c                                                    *
 *                                                                         *
 *  Description : Routines for computing the kernel of a irreducible real  *
 *                representation of a quasi-simple Liegroup.               *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *                         Modification History                            *
 *                                                                         *
 *  Date      Author       Comments                                        *
 * ----------------------------------------------------------------------  *
 * 5.8.93    R.Boedi      Created.                                         *
 * 6.8.93    R.Boedi      Changed memory allocation for vectors and        *
 *                        matrices, see file Memory.c                      *
 * 23.9.93   R.Boedi      Fixed bug in function KernelComplex().           *
 *                        We didn't realize that we had to take the        *
 *                        contragradient representation in the second      *
 *                        component.                                       *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 * Copyright (C) 1993 Michael Joswig, Richard Boedi.  All rights reserved. *
 *                                                                         *
 ***************************************************************************/

/*----------------------------- INCLUDES ----------------------------------*/

#include "IrrReps.h"
#include "Protos.h"

/*----------------------------- GLOBALS -----------------------------------*/

/*------------------------------- CODE ------------------------------------*/

/***************************************************************************
 *                                                                         *
 *  Function name : Kernel ()                                              *
 *                                                                         *
 *  Description : Computes the kernel of a irreducible real                *
 *                representation of a quasi-simple Liegroup.               *
 *                Calls the routines KernelReal() and KernelComplex()      *
 *                according to the field type of *Group.                   *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : Kernel (*Group, *HWeight)                                   *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *    (struct Vector) *HWeight:                                            *
 *       Pointer to a Vector structure which contains the heighest weight  *
 *       vector of the irreducible representation.                         *
 *                                                                         *
 *  Return value :                                                         *
 *    (struct Matrix)                                                      *
 *       Pointer to a Matrix structure which will be filled with the       *
 *       kernel of the representation. The size of the matrix is 3x3.      *
 *       NULL is returned if no memory was available.                      *
 *                                                                         *
 ***************************************************************************/

struct Matrix  *Kernel (struct LiegroupClass *Group, struct Vector *HWeight)

{
if (Group->field == COMPLEX)
   return (KernelComplex (Group, HWeight));
else
   return (KernelReal (Group, HWeight));
}



/***************************************************************************
 *                                                                         *
 *  Function name : KernelReal ()                                          *
 *                                                                         *
 *  Description : Computes the kernel of a irreducible real                *
 *                representation of a quasi-simple Liegroup G,             *
 *                if G is a real form, i.e. the field tag in *Group        *
 *                is not set to COMPLEX.                                   *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : KernelReal (*Group, *HWeight)                               *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *    (struct Vector) *HWeight:                                            *
 *       Pointer to a Vector structure which contains the heighest weight  *
 *       vector of the irreducible representation.                         *
 *                                                                         *
 *  Return value :                                                         *
 *    (struct Matrix)                                                      *
 *       Pointer to a Matrix structure which will be filled with the       *
 *       kernel of the representation. The size of the matrix is 3x3.      *
 *       NULL is returned if no memory was available.                      *
 *                                                                         *
 ***************************************************************************/

struct Matrix  *KernelReal (struct LiegroupClass *Group, struct Vector *HWeight)

{
struct Vector *Image, *Center;
struct Matrix *Kernel;
long           Order, i, d, r, x0, y0, k1, k2, k3, Rows, Generators;


Kernel = CreateMatrix (3, 3);
Image  = ImageRep (Group, HWeight);
Center = CenterUC (Group);

if (Kernel && Image)
   {
   k1    = Image->elements[0];
   k2    = Image->elements[1];
   k3    = Image->elements[2];
   Order = Image->elements[3];

   for (i = 0, Generators = 0; i < Center->length; i++)
      if (Center->elements[i] != 1) Generators++;

   switch (Generators)
      {
      case 1:  /* center is a cyclic group */
         if (k1 == 0) Kernel->prow[0][0] = 1;
         else         Kernel->prow[0][0] = (Order / gcd (Order, k1));
         if (Center->elements[0] != 0) Kernel->prow[0][0] %= Center->elements[0];
         break;

      case 2:  /* center is a group with two generators */
         if ((k1 == 0) && (k2 == 0))
            Kernel->prow[0][0] = Kernel->prow[1][1] = 1;

         if ((k1 == 0) && (k2 != 0))
            {
            Kernel->prow[0][0] = 1;
            Kernel->prow[1][1] = Order / gcd (Order, k2);
            if (Center->elements[1] != 0) Kernel->prow[1][1] %= Center->elements[1];
            }

         if ((k1 != 0) && (k2 == 0))
            {
            Kernel->prow[0][0] = Order / gcd (Order, k1);
            if (Center->elements[0] != 0) Kernel->prow[0][0] %= Center->elements[0];
            if (Kernel->prow[0][0] == 0)  Kernel->prow[0][1] = 1;
            else                          Kernel->prow[1][1] = 1;
            }

         if ((k1 != 0) && (k2 != 0))
            {
            d  = gcd (k1, k2);
            x0 = Order * k1 / gcd (Order, k1);
            y0 = 0;

            /* collect the (maximal two) generators of the kernel */
            for (Rows = 0, r = 0; r < 2; r++)
               {
               Kernel->prow[Rows][0] = x0 - r * k2 / d;
               Kernel->prow[Rows][1] = y0 + r * k1 / d;
               /* reduce modulo orders of the groups */
               if (Center->elements[0] != 0) Kernel->prow[Rows][0] %= Center->elements[0];
               if (Center->elements[1] != 0) Kernel->prow[Rows][1] %= Center->elements[1];
               if (Kernel->prow[Rows][0] < 0) Kernel->prow[Rows][0] += Center->elements[0];
               if (Kernel->prow[Rows][1] < 0) Kernel->prow[Rows][1] += Center->elements[1];
               /* don't take zero vectors */
               if ((Kernel->prow[Rows][0] != 0) || (Kernel->prow[Rows][1] != 0)) Rows++;
               }

            /* throw away obselete generators */
            if ((Rows == 2) && (Center->elements[0] != 0) && (Center->elements[1] != 0))
               {
               d = (Kernel->prow[1][1] * k2) / gcd (Kernel->prow[1][1], k2);
               if (((d * Kernel->prow[1][0] - Kernel->prow[0][0]) % Center->elements[1]) == 0)
                  Kernel->prow[1][0] = Kernel->prow[1][1] = 0;
               }
            }
         break;

      case 3:     /* center is Z2 x Z2 x Z2 */
         d = 0;
         if (k1 != 0) d++;
         if (k2 != 0) d++;
         if (k3 != 0) d++;

         switch (d)
            {
            case 0:
               for (r = 0; r < Kernel->rows; r++) Kernel->prow[r][r] = 1;
               break;
            case 1:
               Kernel->prow[0][0] = 1 - k1; Kernel->prow[1][1] = 1 - k2; Kernel->prow[2][2] = 1 - k3;
               break;
            case 2:
               Kernel->prow[0][0] = k1;     Kernel->prow[0][1] = k2;     Kernel->prow[0][2] = k3;
               Kernel->prow[1][0] = 1 - k1; Kernel->prow[1][1] = 1 - k2; Kernel->prow[1][2] = 1 - k3;
               break;
            case 3:
               Kernel->prow[0][0] = Kernel->prow[0][1] = Kernel->prow[1][1] = Kernel->prow[1][2] = 1;
               break;
            }
         break;
      }
   }

FreeVector (Center);
FreeVector (Image);

return (Kernel);
}



/***************************************************************************
 *                                                                         *
 *  Function name : KernelComplex ()                                       *
 *                                                                         *
 *  Description : Computes the kernel of a irreducible real                *
 *                representation of a quasi-simple Liegroup G,             *
 *                if G is a complex group regarded as a real group,        *
 *                i.e. the field tag in *Group is COMPLEX.                 *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : KernelComplex (*Group, *HWeight)                            *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *    (struct Vector) *HWeight:                                            *
 *       Pointer to a Vector structure which contains the heighest weight  *
 *       vector of the irreducible representation.                         *
 *       Note that the length of this vector has to be even.               *
 *                                                                         *
 *  Return value :                                                         *
 *    (struct Matrix)                                                      *
 *       Pointer to a Matrix structure which will be filled with the       *
 *       kernel of the representation. The size of the matrix is 3x3.      *
 *       NULL is returned if no memory was available.                      *
 *                                                                         *
 ***************************************************************************/

struct Matrix  *KernelComplex (struct LiegroupClass *Group, struct Vector *HWeight)

{
struct Vector  HWeight1, HWeight2, *CGWeight, *Image1, *Image2;
struct Matrix *Kernel;
unsigned long  Order, j, Rows = 0;


HWeight1.length   = HWeight2.length = HWeight->length / 2;
HWeight1.elements = HWeight->elements;
HWeight2.elements = &(HWeight->elements[HWeight1.length]);

Kernel   = CreateMatrix (3, 3);
CGWeight = ContraGradient (Group, &HWeight2);
Image1   = ImageRep (Group, &HWeight1);
Image2   = ImageRep (Group, CGWeight);

if (Kernel && CGWeight && Image1 && Image2)
   {
   Order = Image1->elements[3];
   if (Order > 1)
      {
      if (VectorCmp (&HWeight1, CGWeight))                        /* representations are conjugate */
         if ((Group->type != 'D') || ((Group->rank % 2) == 1))    /* center is cyclic */
            if (Image1->elements[0] == 0)
               Kernel->prow[0][0] = 1;
            else
               Kernel->prow[0][0] = (Order / gcd (Order, 2 * Image1->elements[0])) % Order;
         else                                                     /* center is Z2 x Z2 */
            Kernel->prow[0][0] = Kernel->prow[1][1] = 1;
      else                                                        /* representations are not conjugate */
         if ((Group->type != 'D') || ((Group->rank % 2) == 1))    /* center is cyclic */
            {
            for (j = Order - 1; j >= 0; j--)
               if (((j * (Image1->elements[0] + Image2->elements[0])) % Order) == 0)
                  {
                  Kernel->prow[0][0] = (gcd (j, Order)) % Order;
                  break;
                  }
            }
         else                                                     /* center is Z2 x Z2 */
            {
            if (((Image1->elements[0] + Image2->elements[0]) % 2) == 0)
               Kernel->prow[Rows++][0] = 1;
            if (((Image1->elements[1] + Image2->elements[1]) % 2) == 0)
               Kernel->prow[Rows++][1] = 1;
            if (((Image1->elements[0] + Image2->elements[0] + Image1->elements[1] + Image2->elements[1]) % 2) == 0)
               if (Rows == 0)
                  Kernel->prow[Rows][0] = Kernel->prow[Rows][1] = 1;
            }
      }
   }

FreeVector (Image1);
FreeVector (Image2);
FreeVector (CGWeight);

return (Kernel);
}


/*--------------------------- END OF FILE ---------------------------------*/
