/***************************************************************************
 *                                                                         *
 *  Filename : LGList.c                                                    *
 *                                                                         *
 *  Description : Routines for creating and destroying objects.            *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *                         Modification History                            *
 *                                                                         *
 *  Date      Author       Comments                                        *
 * ----------------------------------------------------------------------  *
 *  9.8.93    M.Joswig     Created.                                        *
 * 30.3.94    MJ/RB        Added exit().                                   *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 * Copyright (C) 1993 Richard Boedi, Michael Joswig.  All rights reserved. *
 *                                                                         *
 ***************************************************************************/

/*----------------------------- INCLUDES ----------------------------------*/

#include "IrrReps.h"
#include "Protos.h"

/*-------------------------- EXTERNAL DATA --------------------------------*/

extern unsigned long Limits[7][2];      /* Maximal admissible ranks of the 7 different */
                                        /* types of groups (real and complex case). */

/*------------------------------- CODE ------------------------------------*/

/***************************************************************************
 *                                                                         *
 *  Function name : LGinList ()                                            *
 *                                                                         *
 *  Description : Checks if a given Lie group is contained in a list.      *
 *                If too high ranks are detected, the function *exits*.    *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : LGinList (Group, List)                                      *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass *) Group:                                      *
 *       The group which is to be checked. Wildcards allowed.              *
 *    (LGList) List:                                                       *
 *       List of Lie groups. Wildcards allowed.                            *
 *                                                                         *
 *  Return value :                                                         *
 *    (BOOL)                                                               *
 *       TRUE is returned, iff Group is contained in List.                 *
 *       _NOTE_ that, if a LiegroupClass item contains wildcards (see      *
 *			IrrReps.h) it describes a set of Lie groups rather than a single  *
 *			group. LGinList checks for set inclusion in this case.            *
 *                                                                         *
 ***************************************************************************/

BOOL LGinList (struct LiegroupClass *Group, LGList List)
{
BOOL found = FALSE;

if (((Group->field == COMPLEX) && (Group->rank > Limits[Group->type-'A'][0])) ||
    ((Group->field != COMPLEX) && (Group->rank > Limits[Group->type-'A'][1])))
   {
	fprintf (stderr, "Aborted. Cannot handle (%c,%lu,%d,%ld).\n", Group->type, Group->rank, (int)Group->field, Group->index);
	exit (-1);
   }

while ((!found) && List) {
	found  = ((Group->type  == List->LGC.type)  || (List->LGC.type  == '*'));
	found &= ((Group->rank  == List->LGC.rank)  || (List->LGC.rank  == 0));
	found &= ((Group->field == List->LGC.field) || (List->LGC.field == UNSPECIFIED));
	found &= ((Group->index == List->LGC.index) || (List->LGC.index == NO_INDEX));
	List = List->NextNode;
	}
return(found);
}

/***************************************************************************
 *                                                                         *
 *  Function name : LGinListNoCheck ()                                     *
 *                                                                         *
 *  Description : Checks if a given Lie group is contained in a list.      *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : LGinListNoCheck (Group, List)                               *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass *) Group:                                      *
 *       The group which is to be checked. Wildcards allowed.              *
 *    (LGList) List:                                                       *
 *       List of Lie groups. Wildcards allowed.                            *
 *                                                                         *
 *  Return value :                                                         *
 *    (BOOL)                                                               *
 *       TRUE is returned, iff Group is contained in List.                 *
 *       _NOTE_ that, if a LiegroupClass item contains wildcards (see      *
 *			IrrReps.h) it describes a set of Lie groups rather than a single  *
 *			group. LGinList checks for set inclusion in this case.            *
 *                                                                         *
 ***************************************************************************/

BOOL LGinListNoCheck (struct LiegroupClass *Group, LGList List)
{
BOOL found = FALSE;

while ((!found) && List) {
	found  = ((Group->type  == List->LGC.type)  || (List->LGC.type  == '*'));
	found &= ((Group->rank  == List->LGC.rank)  || (List->LGC.rank  == 0));
	found &= ((Group->field == List->LGC.field) || (List->LGC.field == UNSPECIFIED));
	found &= ((Group->index == List->LGC.index) || (List->LGC.index == NO_INDEX));
	List = List->NextNode;
	}
return(found);
}

/*--------------------------- END OF FILE ---------------------------------*/
