/***************************************************************************
 *                                                                         *
 *  Filename : Memory.c                                                    *
 *                                                                         *
 *  Description : Routines for creating and destroying objects.            *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *                         Modification History                            *
 *                                                                         *
 *  Date      Author       Comments                                        *
 * ----------------------------------------------------------------------  *
 * 10.4.93    R.Boedi      Created.                                        *
 * 22.5.93    R.Boedi      Adapted code to new struct Matrix definition.   *
 *  5.8.93    R.Boedi      Changed memory allocation for vectors and       *
 *                         matrices. You don't have to allocate a struct   *
 *                         Vector or Matrix before calling the Createxx()  *
 *                         functions. Instead, the size of the objects is  *
 *                         passed as a parameter.                          *
 *  9.8.93    R.Boedi      Added functions LGConstruct() and FreeLGList(), *
 *  9.8.93                 added functions RepNodeConstruct() and          *
 *                         FreeRepList().                                  *
 * 25.8.93    M.Joswig     Added Reverse.                                  *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 * Copyright (C) 1993 Richard Boedi, Michael Joswig.  All rights reserved. *
 *                                                                         *
 ***************************************************************************/

/*----------------------------- INCLUDES ----------------------------------*/

#include "IrrReps.h"
#include "Protos.h"

/*------------------------------- CODE ------------------------------------*/

/***************************************************************************
 *                                                                         *
 *  Function name : CreateVector ()                                        *
 *                                                                         *
 *  Description : Allocates memory for a Vector structure including memory *
 *                for the vector elements.                                 *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : CreateVector (Length)                                       *
 *                                                                         *
 *  Parameters :                                                           *
 *    (unsigned long) Length:                                              *
 *       The length of the vector that should be allocated.                *
 *                                                                         *
 *  Return value :                                                         *
 *    (struct Vector *)                                                    *
 *       Pointer to a fully initialized struct Vector of the given length. *
 *       All entries are set to zero and the length tag of struct vector   *
 *       is set appropriately.                                             *
 *       If no memory was available, a NULL pointer is returned.           *
 *                                                                         *
 ***************************************************************************/

struct Vector *CreateVector (unsigned long Length)

{
struct Vector  *Vector = NULL;
unsigned long    MemSize;

if (Length != 0)
   {
   MemSize = Length * sizeof (*(Vector->elements)) + sizeof(struct Vector);
   
#ifdef DEBUG
   printf ("Vector %ul; %ul bytes...", Length, MemSize);
#endif

   Vector = calloc (MemSize, 1);
   if (Vector)
      {
      Vector->length   = Length;
      Vector->elements = (long *)(Vector + 1);

#ifdef DEBUG
   printf ("ok.\n");
#endif

      }
   }

return (Vector);
}


/***************************************************************************
 *                                                                         *
 *  Function name : CreateMatrix ()                                        *
 *                                                                         *
 *  Description : Allocates memory for a Matrix structure including the    *
 *                memory for the matrix elements.                          *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : CreateMatrix (Rows, Columns)                                *
 *                                                                         *
 *  Parameters :                                                           *
 *    (unsigned long) Rows, Columns:                                       *
 *       Specifies the number of rows and the number of columns of the     *
 *       matrix.                                                           *
 *                                                                         *
 *  Return value :                                                         *
 *    (struct Matrix *)                                                    *
 *       Pointer to a fully initialized struct Matrix of the given size.   *
 *       All entries are set to zero and the length, the row and the       *
 *       column tags of struct Matrix are set appropriately.               *
 *       The pointers to each row of the matrix are set.                   *
 *       If no memory was available, a NULL pointer is returned.           *
 *                                                                         *
 ***************************************************************************/

struct Matrix *CreateMatrix (unsigned long Rows, unsigned long Columns)

{
struct Matrix *Matrix = NULL;
unsigned long  MemSize, elements, rowpointers, row;

if ((Rows != 0) && (Columns != 0))
   {
   elements    = Rows * Columns * sizeof (*(Matrix->elements));
   rowpointers = Rows * sizeof(*(Matrix->prow));
   MemSize     = elements + rowpointers + sizeof(struct Matrix);

#ifdef DEBUG
   printf ("Matrix %ul x %ul; %ul bytes...", Rows, Columns, MemSize);
#endif

   Matrix = calloc (MemSize, 1);
   if (Matrix)
      {

#ifdef DEBUG
   printf ("ok.\n");
#endif

      Matrix->rows      = Rows;
      Matrix->columns   = Columns;
      Matrix->length    = Rows * Columns;
      Matrix->elements  = (long *)(Matrix + 1);
      Matrix->prow      = (long **)(Matrix->elements + Matrix->length);

      /* Determine the row pointers */
      for (row = 0; row < Matrix->rows; row++)
         Matrix->prow[row] = &(Matrix->elements[row * Matrix->columns]);
      }
   }

return (Matrix);
}


/***************************************************************************
 *                                                                         *
 *  Function name : LGConstruct ()                                         *
 *                                                                         *
 *  Description : Allocates memory for a LGNode structure and links it     *
 *                to the top of a linked list of LGNodes.                  *
 *                The structure tags are initialized due to the function's *
 *                parameters.                                              *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : LGConstruct (Type, Rank, Field, Index, Tail)                *
 *                                                                         *
 *  Parameters :                                                           *
 *    (char) Type:                                                         *
 *       Type of the Liegroup                                              *
 *    (unsigned long) Rank:                                                *
 *       Rank of the Liegroup                                              *
 *    (enum FieldType) Field:                                              *
 *       The field of definition                                           *
 *    (long) Index:                                                        *
 *       The index of the Liegroup due to Tits                             *
 *    (LGList) Tail:                                                       *
 *       Pointer to the tail LGNode                                        *
 *                                                                         *
 *  Return value :                                                         *
 *    (struct LGNode *)                                                    *
 *       Pointer to a fully initialized struct LGNode.                     *
 *       If no memory was available, a NULL pointer is returned.           *
 *                                                                         *
 ***************************************************************************/

LGList LGConstruct (char Type, unsigned long Rank, enum FieldType Field, long Index, LGList Tail)

{
LGList  List;

List = malloc (sizeof(struct LGNode));
if (List)
   {
   List->NextNode = Tail;
   List->LGC.type = Type;
   List->LGC.rank = Rank;
   List->LGC.field = Field;
   List->LGC.index = Index;
   }

return (List);
}



/***************************************************************************
 *                                                                         *
 *  Function name : RepListConstruct ()                                    *
 *                                                                         *
 *  Description : Allocates memory for a RepNode structure and links it    *
 *                to the top of a linked list of Nodes.                    *
 *                The dimension and the Weight vector are copied into this *
 *                node.                                                    *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : RepListConstruct (Weight, Dimension, Centralizer, Tail)     *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct Vector *) Weight:                                            *
 *       The dominant weight of the representation.                        *
 *    (unsigned long) Dimension:                                           *
 *       The dimension of the representation.                              *
 *    (unsigned int) Centralizer:                                          *
 *       The centralizer of the representation.                            *
 *    (struct RepNode *) Tail:                                             *
 *       Pointer to the tail RepNode                                       *
 *                                                                         *
 *  Return value :                                                         *
 *    (struct RepNode *)                                                   *
 *       Pointer to a fully initialized struct RepNode.                    *
 *       If no memory was available, a NULL pointer is returned.           *
 *                                                                         *
 ***************************************************************************/

struct RepNode *RepListConstruct (struct Vector *Weight, unsigned long Dimension, unsigned int Centralizer, struct RepNode *Tail)

{
unsigned long    i, MemSize;
struct RepNode  *List;


MemSize = Weight->length * sizeof (*(Weight->elements)) + sizeof(struct RepNode);

#ifdef DEBUG
   printf ("RepNode %ul ; %ul bytes...", Weight->length, MemSize);
#endif

List = calloc (Weight->length * sizeof (*(Weight->elements)) + sizeof(struct RepNode), 1);
if (List)
   {
   
#ifdef DEBUG
   printf ("ok.\n");
#endif   

   List->NextNode        = Tail;
   List->dimension       = Dimension;
   List->centralizer     = Centralizer;
   List->weight.length   = Weight->length;
   List->weight.elements = (long *)(List + 1);
   for (i = 0; i < Weight->length; i++)
         List->weight.elements[i] = Weight->elements[i];
   }

return (List);
}



/***************************************************************************
 *                                                                         *
 *  Function name : Reverse ()                                             *
 *                                                                         *
 *  Description : Reverts a given RepList. Destructive.                    *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : Reverse (List)                                              *
 *                                                                         *
 *  Parameters :                                                           *
 *    (RepList) List:                                                      *
 *       List of representations.                                          *
 *                                                                         *
 *  Return value :                                                         *
 *    (RepList)                                                            *
 *       The same list given as input but in reverse order.                *
 *       _NOTE_ that no memory will be allocated here. Instead the         *
 *       pointers are bent.                                                *
 *			A typical call will therefore be like MyList = Reverse(MyList).   *
 *                                                                         *
 ***************************************************************************/

RepList Reverse (RepList TheList)
{
RepList OldTail = TheList, NewTail = NULL, Temporary;

while (OldTail != NULL) {
 	Temporary = OldTail->NextNode;
	OldTail->NextNode = NewTail;
	NewTail = OldTail;
	OldTail = Temporary;
	}
	
return (NewTail);
}
 

/***************************************************************************
 *                                                                         *
 *  Function name : FreeVector ()                                          *
 *                                                                         *
 *  Description : Deallocates memory used for a Vector structure.          *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : FreeVector (*Vector)                                        *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct Vector) *Vector:                                             *
 *       Pointer to a Vector structure.                                    *
 *                                                                         *
 *  Return value :                                                         *
 *    None                                                                 *
 *                                                                         *
 ***************************************************************************/

void  FreeVector (struct Vector *Vector)

{
if (Vector)
   free (Vector);
}


/***************************************************************************
 *                                                                         *
 *  Function name : FreeMatrix ()                                          *
 *                                                                         *
 *  Description : Deallocates memory used for a Matrix structure.          *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : FreeMatrix (*Matrix)                                        *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct Matrix) *Matrix:                                             *
 *       Pointer to a Matrix structure.                                    *
 *                                                                         *
 *  Return value :                                                         *
 *    None                                                                 *
 *                                                                         *
 ***************************************************************************/

void  FreeMatrix (struct Matrix *Matrix)

{
if (Matrix)
   free (Matrix);
}


/***************************************************************************
 *                                                                         *
 *  Function name : FreeLGList ()                                          *
 *                                                                         *
 *  Description : Deallocates memory of a whole LGList.                    *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : FreeLGList (List)                                           *
 *                                                                         *
 *  Parameters :                                                           *
 *    (LGList) List:                                                       *
 *       Pointer to the beginning of a LGList.                             *
 *                                                                         *
 *  Return value :                                                         *
 *    None                                                                 *
 *                                                                         *
 ***************************************************************************/

void  FreeLGList (LGList List)

{
LGList   Tail;

while (List)
   {
   Tail = List->NextNode;
   free (List);
   List = Tail;
   }
}


/***************************************************************************
 *                                                                         *
 *  Function name : FreeRepList ()                                         *
 *                                                                         *
 *  Description : Deallocates memory of a whole RepList.                   *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : FreeRepList (List)                                          *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct RepNode *) List:                                             *
 *       Pointer to the beginning of a RepList.                            *
 *                                                                         *
 *  Return value :                                                         *
 *    None                                                                 *
 *                                                                         *
 ***************************************************************************/

void  FreeRepList (struct RepNode *List)

{
struct RepNode   *Tail;

while (List)
   {
   Tail = List->NextNode;
   free (List);
   List = Tail;
   }
}



/*--------------------------- END OF FILE ---------------------------------*/
