/***************************************************************************
 *                                                                         *
 *  Filename : Merge.c                                                     *
 *                                                                         *
 *  Description : Takes two raw files (generated by the Generate command)  *
 *                and merges them into one single file that can be      `  *
 *                processed by the TeXit command.                          *
 *                Note: It is assumed that both files contain sorted data  *
 *                      (the Generate command creates such files).         *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *                         Modification History                            *
 *                                                                         *
 *  Date      Author       Comments                                        *
 * ----------------------------------------------------------------------  *
 *  3.1.94    R.Boedi      Created.                                        *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : Merge infile1 infile2                                       *
 *                                                                         *
 *  Output : (stdout)                                                      *
 *             The merged file containing the contents of infile1 and      *
 *             infile2.                                                    *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 * Copyright (C) 1994 Richard Boedi, Michael Joswig. All rights reserved.  *
 *                                                                         *
 ***************************************************************************/

/*----------------------------- INCLUDES ----------------------------------*/

#include "IrrReps.h"
#include "Protos.h"
#include <string.h>
#include <math.h>

/*----------------------------- DEFINES -----------------------------------*/

#define  FIRSTFILE   0
#define  SECONDFILE  1

#define  EQUAL       0
#define  GREATER     1
#define  LESS       -1

#define  STRLENGTH 256

#define  min(a,b)    ((a) <= (b) ? (a) : (b))
#define  max(a,b)    ((a) >= (b) ? (a) : (b))


/*------------------------------ PROTOS -----------------------------------*/

BOOL  ReadHeader (struct LiegroupClass *, long *, FILE *);
BOOL  ReadLine (struct LiegroupClass *, struct Vector **, FILE *);
void  WriteHeader (struct LiegroupClass *, long *);
void  WriteLeadingLine (FILE *);
void  WriteLine (struct Vector *);
void  WriteList (FILE *);
int   GroupCmp (struct LiegroupClass *, struct LiegroupClass *);
int   LineCmp (struct Vector *, struct Vector *);

/*--------------------------- GLOBAL DATA ---------------------------------*/

char  LineBuffer[STRLENGTH];

/*------------------------------- CODE ------------------------------------*/


main (int argc, char *argv[])

{
int                  infileindex, currentinfile = 0;
long                 limits[2][SECONDFILE + 1];
FILE                *Infile[SECONDFILE + 1];
BOOL                 EofF[SECONDFILE + 1], EofT[SECONDFILE + 1],
                     EndFile, EndTable;
struct Vector       *TableLine[SECONDFILE + 1];
struct LiegroupClass	Grp[SECONDFILE + 1];

if (argc == 3)
   {
   for (infileindex = FIRSTFILE; infileindex <= SECONDFILE; infileindex++)
      Infile[infileindex] = fopen (argv[infileindex+1], "r");

   if (Infile[FIRSTFILE] && Infile[SECONDFILE])
      {
      EofF[FIRSTFILE]  = ReadHeader (&(Grp[FIRSTFILE]), limits[FIRSTFILE], Infile[FIRSTFILE]);
      EofF[SECONDFILE] = ReadHeader (&(Grp[SECONDFILE]), limits[SECONDFILE], Infile[SECONDFILE]);
      EndFile = EofF[FIRSTFILE] && EofF[SECONDFILE];

      while (!EndFile)
         {
         if (GroupCmp (&(Grp[FIRSTFILE]), &(Grp[SECONDFILE])) == EQUAL)
            {
            if ((limits[0][0] > limits[1][1]) || (limits[1][0] > limits[0][1]))
               fprintf (stderr, "WARNING: dmin and dmax intervals of type (%c,%d,%d,%d) contain gaps after joining them\n",
                        Grp[0].type, Grp[0].rank, Grp[0].field, Grp[0].index);

            limits[0][0] = min (limits[0][0], limits[1][0]);
            limits[0][1] = max (limits[0][1], limits[1][1]);

            WriteHeader (&(Grp[FIRSTFILE]), limits[FIRSTFILE]);
            WriteLeadingLine (Infile[FIRSTFILE]);
            ReadLine (&(Grp[SECONDFILE]), &(TableLine[SECONDFILE]), Infile[SECONDFILE]);

            EofT[FIRSTFILE]  = ReadLine (&(Grp[FIRSTFILE]), &(TableLine[FIRSTFILE]), Infile[FIRSTFILE]);
            EofT[SECONDFILE] = ReadLine (&(Grp[SECONDFILE]), &(TableLine[SECONDFILE]), Infile[SECONDFILE]);
            EndTable = FALSE;

            while (!EndTable)
               {
               if ((!EofT[FIRSTFILE]) && (!EofT[SECONDFILE]))
                  {
                  if (LineCmp (TableLine[FIRSTFILE], TableLine[SECONDFILE]) == GREATER)
                     currentinfile = SECONDFILE;
                  else
                     {
                     if (LineCmp (TableLine[FIRSTFILE], TableLine[SECONDFILE]) == EQUAL)
                        if (!EofT[SECONDFILE])
                           EofT[SECONDFILE] = ReadLine (&(Grp[SECONDFILE]), &(TableLine[SECONDFILE]), Infile[SECONDFILE]);
                        else
                           FreeVector (TableLine[SECONDFILE]);
                     currentinfile = FIRSTFILE;
                     }
                  }
               else
                  if (EofT[FIRSTFILE]) currentinfile = SECONDFILE;
                  else                 currentinfile = FIRSTFILE;

               WriteLine (TableLine[currentinfile]);
               FreeVector (TableLine[currentinfile]);

               if (!EofT[currentinfile])
                  EofT[currentinfile] = ReadLine (&(Grp[currentinfile]), &(TableLine[currentinfile]), Infile[currentinfile]);

               EndTable = EofT[FIRSTFILE] && EofT[SECONDFILE];
               if (!EndTable) printf ("%c\n", Delim);
               else           printf ("%c\n", OuterRight);
               }

            EofF[FIRSTFILE]  = ReadHeader (&(Grp[FIRSTFILE]), limits[FIRSTFILE], Infile[FIRSTFILE]);
            EofF[SECONDFILE] = ReadHeader (&(Grp[SECONDFILE]), limits[SECONDFILE], Infile[SECONDFILE]);
            EndFile = EofF[FIRSTFILE] && EofF[SECONDFILE];
            }
         else
            {
            if (GroupCmp (&(Grp[FIRSTFILE]), &(Grp[SECONDFILE])) == GREATER)
               currentinfile = SECONDFILE;
            else
               currentinfile = FIRSTFILE;

            WriteHeader (&(Grp[currentinfile]), limits[currentinfile]);
            WriteList (Infile[currentinfile]);
            EofF[currentinfile] = ReadHeader (&(Grp[currentinfile]), limits[currentinfile], Infile[currentinfile]);
            }

         EndFile = EofF[FIRSTFILE] && EofF[SECONDFILE];
         }

      FreeVector (TableLine[FIRSTFILE]);
      FreeVector (TableLine[SECONDFILE]);

      return (0);
      }
   else
      {
      if (!Infile[FIRSTFILE])
         fprintf (stderr, "Couldn't open file %s.\n", argv[1]);
      if (!Infile[SECONDFILE])
         fprintf (stderr, "Couldn't open file %s.\n", argv[2]);

      if (Infile[FIRSTFILE]) fclose (Infile[FIRSTFILE]);
      if (Infile[SECONDFILE]) fclose (Infile[SECONDFILE]);

      return (10);
      }
   }
else
   {
   fprintf (stderr, "Usage: %s infile1 infile2\n", argv[0]);
	fprintf (stderr, "       Output is written to stdout.\n");
	return (10);
	}
}



/***************************************************************************
 *                                                                         *
 * ReadHeader (struct LiegroupClass *Grp, long *limits, FILE *fp)          *
 *                                                                         *
 * Reads the first two lines of a representation table and stores the data *
 * in Grp and limits,                                                      *
 * Returns FALSE if operation was successful                               *
 *     and TRUE  if EOF has been reached or fscanf() has failed.           *
 *                                                                         *
 ***************************************************************************/

BOOL  ReadHeader (struct LiegroupClass *Grp, long *limits, FILE *fp)

{
int   matchs, totalmatchs = 0;


matchs = totalmatchs = fscanf (fp, "%c%ld %d %d \n", &(Grp->type), &(Grp->rank), &(Grp->field), &(Grp->index));
if (matchs != EOF)
   {
   matchs = fscanf (fp, "%ld %ld \n", &(limits[0]), &(limits[1]));
   totalmatchs += matchs;
   }

if ((totalmatchs != 6) || (matchs == EOF))
   {
   Grp->type = 'H';
   Grp->rank = 1;
   Grp->field = 0;
   Grp->index = 0;
   return (TRUE);
   }
else
   return (FALSE);
}



/***************************************************************************
 *                                                                         *
 * ReadLine (struct LiegroupClass *Grp, struct Vector **Line, FILE *fp)    *
 *                                                                         *
 * Reads a single ordinary line of a representation table and stores the   *
 * data in the vector Line.                                                *
 * The memory for the vector is allocated by this very function.           *
 * It must be released by FreeVector() manually.                           *
 * Also do not forget to check *Line for being nonzero after calling this  *
 * function.                                                               *
 * Returns FALSE if operation was successful                               *
 *     and TRUE  if EOT has been reached, or fscanf() has failed, or no    *
 *               memory has been availbale for vector.                     *
 *                                                                         *
 ***************************************************************************/

BOOL  ReadLine (struct LiegroupClass *Grp, struct Vector **Line, FILE *fp)

{
int            index;
char          *Bufptr;
unsigned long  VectorLength;
enum ErrorType Error;
BOOL           EOT = FALSE;


VectorLength = Grp->rank;
if (Grp->field == COMPLEX) VectorLength *= 2;
VectorLength += 11;

if (!(*Line = CreateVector (VectorLength)))
   return (TRUE);

fgets (LineBuffer, STRLENGTH, fp);
if (feof (fp) == EOF)
   return (TRUE);

if (LineBuffer[0] == OuterRight) EOT = TRUE;
else
   {
   if (LineBuffer[strlen(LineBuffer)-2] == OuterRight)
      fseek (fp, -2, 1);
   if (LineBuffer[strlen(LineBuffer)-2] != InnerRight)
      LineBuffer[strlen(LineBuffer)-2] = 0;
   if (LineBuffer[strlen(LineBuffer)-1] == '\n')
      LineBuffer[strlen(LineBuffer)-1] = 0;
   for (Bufptr = LineBuffer, index = 0; index < strlen (LineBuffer); index++)
      if (isspace (*Bufptr)) Bufptr++;
      else break;
   Error = StringToVector (Bufptr, *Line);
   if (Error != NO_ERROR) EOT = TRUE;
   }

return (EOT);
}



/***************************************************************************
 *                                                                         *
 * WriteHeader (struct LiegroupClass *Grp, long *limits)                   *
 *                                                                         *
 * Writes the first two lines of a representation table to stdout.         *
 *                                                                         *
 ***************************************************************************/

void  WriteHeader (struct LiegroupClass *Grp, long *limits)

{
printf ("%c%ld %d %d\n", Grp->type, Grp->rank, Grp->field, Grp->index);
printf ("%ld %ld\n", limits[0], limits[1]);
}



/***************************************************************************
 *                                                                         *
 * WriteLeadingLine (FILE *fp)                                             *
 *                                                                         *
 * Copies the lreading (i.e. the third) line of a representation table     *
 * read from fp to stdout.                                                 *
 *                                                                         *
 ***************************************************************************/

void  WriteLeadingLine (FILE *fp)

{
fgets (LineBuffer, STRLENGTH, fp);
LineBuffer[strlen(LineBuffer)-1] = 0;
if ((LineBuffer[0] == OuterLeft) && (LineBuffer[1] != InnerLeft))
   fgets (&LineBuffer[1], STRLENGTH, fp);
LineBuffer[strlen(LineBuffer)-1] = 0;
puts (LineBuffer);
}



/***************************************************************************
 *                                                                         *
 * WriteLine (struct Vector *Line)                                         *
 *                                                                         *
 * Writes a line of a representation table to stdout.                      *
 * No line feed is added.                                                  *
 *                                                                         *
 ***************************************************************************/

void  WriteLine (struct Vector *Line)

{
printf (" ");
PrintVector (Line, NULL, UNSIGNED);
}



/***************************************************************************
 *                                                                         *
 * WriteList (FILE *fp)                                                    *
 *                                                                         *
 * Copies a complete representation table read from fp to stdout.          *
 *                                                                         *
 ***************************************************************************/

void  WriteList (FILE *fp)

{
do {
   fgets (LineBuffer, STRLENGTH, fp);
   LineBuffer[strlen(LineBuffer)-1] = 0;
   puts (LineBuffer);
   } while (LineBuffer[strlen(LineBuffer)-1] != OuterRight);
}


/***************************************************************************
 *                                                                         *
 * GroupCmp (struct LiegroupClass *Grp1, struct LiegroupClass *Grp2)       *
 *                                                                         *
 * Compares two Lie group structures Grp1 and Grp2.                        *
 * The comparison is done lexographically.                                 *
 * Returns EQUAL, if both structures have the same contents,               *
 *         GREATER, if Grp1 is greater than Grp2, and                      *
 *         LESS, if Grp1 is less than Grp2.                                *
 *                                                                         *
 ***************************************************************************/

int   GroupCmp (struct LiegroupClass *Grp1, struct LiegroupClass *Grp2)

{
if (Grp1->type == Grp2->type)
   {
   if (Grp1->rank == Grp2->rank)
      {
      if (Grp1->field == Grp2->field)
         {
         if (Grp1->index == Grp2->index)
            return (EQUAL);
         else
            if (Grp1->index > Grp2->index) return (GREATER);
            else return (LESS);
         }
      else
         if (Grp1->field > Grp2->field) return (GREATER);
         else return (LESS);
      }
   else
      if (Grp1->rank > Grp2->rank) return (GREATER);
      else return (LESS);
   }
else
   if (Grp1->type > Grp2->type) return (GREATER);
   else return (LESS);
}



/***************************************************************************
 *                                                                         *
 * LineCmp (struct Vector *Line1, struct Vector *Line2)                    *
 *                                                                         *
 * Compares two representation lines given by Line1 and Line2.             *
 * Both vectors are assumed to have the same size. If Line1 has n entries, *
 * then only the first n-9 entries are compared.                           *
 * The comparison is done lexographically for the first two columns and    *
 * antilexicographically for the remaining valid columns.                  *
 * Returns EQUAL, if both structures have the same contents,               *
 *         GREATER, if Line1 is greater than Line2, and                    *
 *         LESS, if Line1 is less than Line2.                              *
 *                                                                         *
 ***************************************************************************/

int   LineCmp (struct Vector *Line1, struct Vector *Line2)

{
int   index;

if (Line1->elements[0] == Line2->elements[0])
   {
   if (Line1->elements[1] == Line2->elements[1])
      {
      for (index = 2; index < Line1->length - 9; index++)
         {
         if (Line1->elements[index] > Line2->elements[index]) return (GREATER);
         if (Line1->elements[index] < Line2->elements[index]) return (LESS);
         }
      return (EQUAL);
      }
   else
      if (Line1->elements[1] > Line2->elements[1]) return (GREATER);
      else return (LESS);
   }
else
   if (Line1->elements[0] > Line2->elements[0]) return (GREATER);
   else return (LESS);

return (EQUAL);
}



/*--------------------------- END OF FILE ---------------------------------*/


