/***************************************************************************
 *                                                                         *
 *  Filename : Output.c                                                    *
 *                                                                         *
 *  Description : Output routines for printing vectors and matrices.       *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *                         Modification History                            *
 *                                                                         *
 *  Date      Author       Comments                                        *
 * ----------------------------------------------------------------------  *
 * 10.4.93    R.Boedi      Created.                                        *
 * 22.5.93    R.Boedi      Adapted code to new struct Matrix definition.   *
 * 10.8.93    R.Boedi      Added Type parameter in PrintVector() and       *
 *                         PrintMatrix().                                  *
 * 22.9.93    R.Boedi      Added NewLine parameter to PrintMatrix().       *
 * 31.3.94    RB/MJ        Added PrintLGClass().                           *
 * 10.6.96    MJ           Exchanged sprintf construction for log10.       *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 * Copyright (C)) 1993 Michael Joswig, Richard Boedi.  All rights reserved.*
 *                                                                         *
 ***************************************************************************/

/*----------------------------- INCLUDES ----------------------------------*/

#include "IrrReps.h"
#include "Protos.h"

/*------------------------------- CODE ------------------------------------*/

/***************************************************************************
 *                                                                         *
 *  Function name : PrintVector ()                                         *
 *                                                                         *
 *  Description : Prints a vector in the format ( x,...., x, x ).          *
 *                No line feed is added.                                   *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : PrintVector (*Vector, *Spaces, Type)                        *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct Vector) *Vector:                                             *
 *       Pointer to a Vector structure which will be printed.              *
 *    (unsigned short) *Spaces:                                            *
 *       Specifies the minimal lengths for each entry.                     *
 *       This tag is omitted if a NULL pointer is supported.               *
 *    (enum ElementType) Type:                                             *
 *       If set to SIGNED, the elements of the vector are printed as       *
 *          signed values.                                                 *
 *       If set to UNSIGNED, the elements of the vector are printed as     *
 *          unsigned values.                                               *
 *                                                                         *
 *  Return value :                                                         *
 *    (BOOL)                                                               *
 *       TRUE, if operation was successful                                 *
 *      FALSE, if no memory for output buffer was available                *
 *                                                                         *
 ***************************************************************************/

BOOL  PrintVector (struct Vector *Vector, unsigned short *Spaces, enum ElementType Type)

{
char    *Output;

if (Output = malloc (COMPSIZE * Vector->length))
   {
   VectorToString (Vector, Output, Spaces, Type);
   printf ("%c%s%c", InnerLeft, Output, InnerRight);
   free (Output);
   return (TRUE);
   }
else
   return (FALSE);
}


/***************************************************************************
 *                                                                         *
 *  Function name : PrintMatrix ()                                         *
 *                                                                         *
 *  Description : Prints a matrix in the format                            *
 *                     [( x,...., x, x )                                   *
 *                      ( x,...., x, x )                                   *
 *                                                                         *
 *                      ( x,...., x, x )                                   *
 *                      ( x,...., x, x )]                                  *
 *                                                                         *
 *                The colons are printed nicely in a vertical row.         *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : PrintMatrix (*Matrix, Type, NewLine)                        *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct Matrix) *Matrix:                                             *
 *       Pointer to a Matrix structure which will be printed.              *
 *    (enum ElementType) Type:                                             *
 *       If set to SIGNED, the elements of the vector are printed as       *
 *          signed values.                                                 *
 *       If set to UNSIGNED, the elements of the vector are printed as     *
 *          unsigned values.                                               *
 *    (BOOL) NewLine:                                                      *
 *       if FALSE, the table is printed in one line.                       *
 *                                                                         *
 *  Return value :                                                         *
 *    (BOOL)                                                               *
 *       TRUE, if operation was successful                                 *
 *      FALSE, if no memory for output buffer was available                *
 *                                                                         *
 ***************************************************************************/

BOOL  PrintMatrix (struct Matrix *Matrix, enum ElementType Type, BOOL NewLine)

{
long              Min = 0x12345678, Max = 0x24680246;
unsigned short   *Spaces;
unsigned long     column, row, length = 0x13243546;
struct Vector     Vector;
BOOL              Success = FALSE;

if (Matrix->length > 0)
   if (Spaces = calloc (Matrix->columns, sizeof(short)))
      {
      /* Determine the maximum length for the entries of a column */
      for (column = 0; column < Matrix->columns; column++)
         for (Min = 0, Max = 0, row = 0; row < Matrix->rows; row++)
            {
            if (Matrix->prow[row][column] > Max)
               {
               Max = Matrix->prow[row][column];
               if ((length = (unsigned long)log10((double)Max)+1) > Spaces[column])
                  {
                  Spaces[column] = length;
                  if (length >= COMPSIZE)
                     {
                     fprintf (stderr, "FATAL ERROR (length=%ld)\n",length);
                     exit (-1);
                     }
                  }
               }
            if (Matrix->prow[row][column] < Min)
               {
               Min = Matrix->prow[row][column];
               if ((length = (unsigned long)log10((double)Max)+1) >= Spaces[column])
                  {
                  Spaces[column] = length;
                  if (length >= COMPSIZE)
                     {
                     fprintf (stderr, "FATAL ERROR (length=%ld)\n",length);
                     exit (-1);
                     }
                  }
               }
            }

      /* Now print the matrix */
      for (row = 0; row < Matrix->rows; row++)
         {
         if (row == 0) printf ("%c", OuterLeft);
         else          printf (" ");

         Vector.length   = Matrix->columns;
         Vector.elements = Matrix->prow[row];
         PrintVector (&Vector, Spaces, Type);

         if (row == (Matrix->rows - 1)) printf ("%c\n", OuterRight);
         else
            {
            printf ("%c", Delim);
            if (NewLine) printf ("\n");
            }
         }

      Success = TRUE;
      }

return (Success);
}

/***************************************************************************
 *                                                                         *
 *  Function name : PrintLGClass()                                         *
 *                                                                         *
 *  Description : Prints a Lie group class (with wildcards) to stderr.     *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : PrintLGClass (*LGClass)                                     *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LieGroupClass) *LGClass                                      *
 *                                                                         *
 ***************************************************************************/

void PrintLGClass  (struct LiegroupClass *LGClass)
{
fprintf(stderr,"(%c,",LGClass->type);
if (LGClass->rank == 0) fprintf(stderr,"*,");
else                    fprintf(stderr,"%lu,",LGClass->rank);
if (LGClass->field == UNSPECIFIED) fprintf(stderr,"*,");
else                               fprintf(stderr,"%d,",(int)LGClass->field);
if (LGClass->index == NO_INDEX) fprintf(stderr,"*)");
else                            fprintf(stderr,"%ld,",LGClass->index);
}
/*--------------------------- END OF FILE ---------------------------------*/
