/***************************************************************************
 *                                                                         *
 *  Filename : PositiveRoots.c                                             *
 *                                                                         *
 *  Description : Routines dealing with positive roots.                    *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *                         Modification History                            *
 *                                                                         *
 *  Date      Author       Comments                                        *
 * ----------------------------------------------------------------------  *
 *  9.4.93    R.Boedi      Created.                                        *
 * 22.5.93    R.Boedi      Adapted code to new struct Matrix definition.   *
 *  5.8.93    R.Boedi      Changed memory allocation for vectors and       *
 *                         matrices, see file Memory.c                     *
 * 28.3.94    R.Boedi      Added check for validity of the number of       *
 *                         positive roots (overflow problem).              *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 * Copyright ) 1993 Michael Joswig, Richard Boedi.  All rights reserved.   *
 *                                                                         *
 ***************************************************************************/

/*----------------------------- INCLUDES ----------------------------------*/

#include "IrrReps.h"
#include "Protos.h"

/*------------------------------- CODE ------------------------------------*/

/***************************************************************************
 *                                                                         *
 *  Function name : NoOfPositiveRoots ()                                   *
 *                                                                         *
 *  Description : Determines the number of simple positive roots for the   *
 *                quasi-simple Liealgebra or Liegroup.                     *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : NoOfPositiveRoots (*Group)                                  *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *                                                                         *
 *  Return value :                                                         *
 *    (unsigned long)                                                      *
 *       Number of simple positive roots.                                  *
 *       Returns 0, if number of simple roots does not fit in a long int.  *
 *                                                                         *
 ***************************************************************************/

unsigned long  NoOfPositiveRoots (struct LiegroupClass *Group)

{
unsigned long  n;
unsigned long  rank = Group->rank;

const unsigned long  SQR_ULONG_MAX = (1<<(4*sizeof(long)))-1;

if (Group->rank < SQR_ULONG_MAX)
   {
   switch (Group->type)
      {
      case 'A':
         n = (rank * (rank + 1)) / 2;
         break;

      case 'B':
      case 'C':
         n = rank * rank;
         break;

      case 'D':
         n = rank * (rank - 1);
         break;

      case 'E':
         n = (rank == 6) ? 36 : (rank == 7) ? 63 : 120;
         break;

      case 'F':
         n = 24;
         break;

      case 'G':
         n = 6;
         break;
      }

   return (n);
   }
else
   return (0);
}


/***************************************************************************
 *                                                                         *
 *  Function name : PositiveRootNorms ()                                   *
 *                                                                         *
 *  Description : Determines the norms of the simple positive roots for    *
 *                the given quasi-simple Liealgebra or Liegroup.           *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : PositiveRootNorms (*Group)                                  *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *                                                                         *
 *  Return value :                                                         *
 *    (struct Vector)                                                      *
 *       Vector of the norms of the simple positive roots.                 *
 *       NULL, if no memory was available.                                 *
 *                                                                         *
 ***************************************************************************/

struct Vector    *PositiveRootNorms (struct LiegroupClass *Group)

{
struct Vector    *Norms;
unsigned long     rank = Group->rank;
long              i;

if (Norms = CreateVector (rank))
   {
   for (i = 0; i < rank; i++)  Norms->elements[i] = 1;

   switch (Group->type)
      {
      case 'A':   /* 1,...,1 */
      case 'D':
      case 'E':
         /* nothing left to do */
         break;

      case 'B':   /* 2,...,2,1 */
         for (i = rank - 2; i >= 0; i--)  Norms->elements[i] = 2;
         break;

      case 'C':   /* 1,...,1,2 */
         Norms->elements[rank - 1] = 2;
         break;

      case 'F':   /* 2,2,1,1 */
         Norms->elements[0] = Norms->elements[1] = 2;
         break;

      case 'G':   /* 1,3 */
         Norms->elements[1] = 3;
         break;
      }
   }

return (Norms);
}


/***************************************************************************
 *                                                                         *
 *  Function name : PositiveRoots ()                                       *
 *                                                                         *
 *  Description : Determines the set of all simple positive roots for a    *
 *                given quasi-simple Liealgebra or Liegroup.               *
 *                For the algorithm see Jacobson, Lie Algebras p.122       *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : PositiveRoots (*Group)                                      *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *                                                                         *
 *  Return value :                                                         *
 *    (struct Matrix)                                                      *
 *       Matrix whose rows correspond to the simple positive roots.        *
 *       NULL pointer if error (no memory available)                       *
 *                                                                         *
 ***************************************************************************/

struct Matrix    *PositiveRoots (struct LiegroupClass *Group)

{
char             type = Group->type;
unsigned long    rank = Group->rank;
unsigned long    Level, i, j, k, CurrRootIndex;
long             sum;
struct Vector   *Expon, *Levels = NULL, *RootNorms = NULL, *Weights = NULL;
struct Matrix   *Cartan = NULL, *PosRoots = NULL;
BOOL             AddRoot;


if (Expon = Exponents (Group))
   {
   Levels      = CreateVector (Expon->elements[rank - 1] + 1);
   Weights     = CreateVector (rank);
   PosRoots    = CreateMatrix (NoOfPositiveRoots (Group), rank);
   RootNorms   = PositiveRootNorms (Group);
   Cartan      = CartanMatrix (Group);

   if (Levels && RootNorms && Weights && Cartan && PosRoots)
      {
      /* The first r roots are fundamental roots */
      for (i = 0; i < rank; i++) PosRoots->prow[i][i] = 1;

      /* Now we loop over all levels ... */
      Levels->elements[0] = 0; CurrRootIndex = rank;
      for (Level = 0; CurrRootIndex > Levels->elements[Level]; Level++)
         {
         Levels->elements[Level + 1] = CurrRootIndex;

         /* Now loop over all roots in the level ... */
         for (i = Levels->elements[Level]; i < Levels->elements[Level + 1]; i++)
            {
            for (j = 0; j < rank; j++)
               {
               for (sum = 0, k = 0; k < rank; k++)
                  sum += PosRoots->prow[i][k] * Cartan->prow[k][j];
               Weights->elements[j] = sum;
               }

            /* Now we loop over all fundamental roots */
            for (j = 0; j < rank; j++)
               {
               AddRoot = (Weights->elements[j] < 0);

               if (!AddRoot)
                  if ((type != 'A') && (type != 'D') && (type != 'E') &&
                      (RootNorms->elements[j] == 1))
                     if (type == 'G')
                        AddRoot = (Weights->elements[j] == 1);
                     else
                        if ((Weights->elements[j] == 0) && (PosRoots->prow[i][j] > 0))
                           {
                           PosRoots->prow[i][j]--;
                           for (k = Levels->elements[Level - 1]; k < Levels->elements[Level]; k++)
                              if (!memcmp (PosRoots->prow[k], PosRoots->prow[i], sizeof (*(PosRoots->elements)) * rank))
                                 {
                                 AddRoot = TRUE;
                                 break;
                                 }
                           PosRoots->prow[i][j]++;
	                        }

               if (AddRoot)
                  {
                  PosRoots->prow[i][j]++;
                  for (k = Levels->elements[Level + 1]; k < CurrRootIndex; k++)
                     if (!memcmp (PosRoots->prow[k], PosRoots->prow[i], sizeof (*(PosRoots->elements)) * rank))
                        {
                        AddRoot = FALSE;
                        break;
                        }

                  if (AddRoot)
                     {
                     memcpy (PosRoots->prow[CurrRootIndex], PosRoots->prow[i], sizeof (*(PosRoots->elements)) * rank);
                     CurrRootIndex++;
                     }
                  PosRoots->prow[i][j]--;
                  }
               }
            }
         }
      }
   }

FreeVector (Expon);
FreeMatrix (Cartan);
FreeVector (Weights);
FreeVector (RootNorms);
FreeVector (Levels);

return (PosRoots);
}



/***************************************************************************
 *                                                                         *
 *  Function name : CartanMatrix ()                                        *
 *                                                                         *
 *  Description : Determines the Cartan matrix of the given quasi-simple   *
 *                Liealgebra or Liegroup.                                  *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : CartanMatrix (*Group)                                       *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *                                                                         *
 *  Return value :                                                         *
 *    (struct Matrix)                                                      *
 *       Matrix pointer to the Cartan matrix.                              *
 *       NULL, if no memory was available.                                 *
 *                                                                         *
 ***************************************************************************/

struct Matrix    *CartanMatrix (struct LiegroupClass *Group)

{
struct Matrix    *Cartan;
unsigned long     i, rank = Group->rank;


if (Cartan = CreateMatrix (rank, rank))
   {
   for (i = 0; i < rank; i++)
      Cartan->prow[i][i] = 2;   /* default is type An */

   for (i = 1; i < rank; i++)
      Cartan->prow[i-1][i] = Cartan->prow[i][i-1] = -1;

   switch (Group->type)
      {
      case 'A':
         /* Nothing left to do */
         break;

      case 'B':
         Cartan->prow[rank-2][rank-1] = -2;
         break;

      case 'C':
         Cartan->prow[rank-1][rank-2] = -2;
         break;

      case 'D':
         Cartan->prow[rank-3][rank-1] = -1;
         Cartan->prow[rank-1][rank-3] = -1;
         Cartan->prow[rank-2][rank-1] =  0;
         Cartan->prow[rank-1][rank-2] =  0;
         break;

      case 'E':
         Cartan->prow[0][1] =  0;
         Cartan->prow[1][0] =  0;
         Cartan->prow[1][2] =  0;
         Cartan->prow[2][1] =  0;
         Cartan->prow[0][2] = -1;
         Cartan->prow[2][0] = -1;
         Cartan->prow[1][3] = -1;
         Cartan->prow[3][1] = -1;
         break;

      case 'F':
         Cartan->prow[1][2] = -2;
         break;

      case 'G':
         Cartan->prow[1][0] = -3;
         break;
      }
   }

return (Cartan);
}



/***************************************************************************
 *                                                                         *
 *  Function name : Exponents ()                                           *
 *                                                                         *
 *  Description : Determines the exponents of the given quasi-simple       *
 *                Liealgebra or Liegroup.                                  *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : Exponents (*Group)                                          *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *                                                                         *
 *  Return value :                                                         *
 *    (struct Matrix)                                                      *
 *       Vector pointer to the exponent vector.                            *
 *       The exponents are ordered increasingly.                           *
 *       NULL, if no memory was available.                                 *
 *                                                                         *
 ***************************************************************************/

struct Vector    *Exponents (struct LiegroupClass *Group)

{
struct Vector  *ExpVector;
unsigned long   i, rank = Group->rank;
long           *expo;

if (ExpVector = CreateVector (rank))
   {
   expo     = ExpVector->elements;
   expo[0]  = 1;

   switch (Group->type)
      {
      case 'A':
         for (i = 1; i < rank; i++) expo[i] = i + 1;
         break;

      case 'B':
      case 'C':
         for (i = 1; i < rank; i++) expo[i] = 2 * i + 1;
         break;

      case 'D':
         for (i = 1; i < rank; i++) expo[i] = (2 * i + 1 < rank) ? 2 * i + 1 : 2 * i - 1;
         expo[rank / 2] = rank - 1;
         break;

      case 'E':
         switch (rank)
            {
            case 6:
               expo[1] =  4;  expo[2] =  5;  expo[3] =  7;
               expo[4] =  8;  expo[5] = 11;
               break;
            case 7:
               expo[1] =  5;  expo[2] =  7;  expo[3] =  9;
               expo[4] = 11;  expo[5] = 13;  expo[6] = 17;
               break;
            case 8:
               expo[1] =  7;  expo[2] = 11;  expo[3] = 13;
               expo[4] = 17;  expo[5] = 19;  expo[6] = 23;
               expo[7] = 29;
               break;
            }
         break;

      case 'F':
         expo[1] = 5;  expo[2] = 7;  expo[3] = 11;
         break;

      case 'G':
         expo[1] = 5;
         break;
      }
   }

return (ExpVector);
}


/***************************************************************************
 *                                                                         *
 *  Function name : SetLimits                                              *
 *                                                                         *
 *  Description : Determines the maximal ranks which one can handle with   *
 *                the program.                                             *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : SetLimits ()                                                *
 *                                                                         *
 *  Parameters : None                                                      *
 *                                                                         *
 *  Return value : None                                                    *
 *                                                                         *
 ***************************************************************************/
 
unsigned long Limits[7][2]; /* 7 types, complex and real */
#define  Z(p,q)   (unsigned long)(pow(-(q)+sqrt((p)*(p)*(p)+(q)*(q)),1./3.)+pow(-(q)-sqrt((p)*(p)*(p)+(q)*(q)),1./3.)-1)

void  SetLimits ()

{
double   x;

x = ((double)ULONG_MAX - (double)sizeof(struct Matrix)) / 2.;
Limits[0][0] = Z(-1./12., -x);
Limits[0][1] = Z(-1./9., -1./27.-2.*x);
x = ((double)ULONG_MAX - (double)sizeof(struct Matrix));
Limits[1][0] = Limits[2][0] = Z(-1./36., 1./216.-(x/4.));
Limits[1][1] = Limits[2][1] = Z(-1./9., 1./27.-x/2.);
Limits[3][0] = Z(-1./36., -1./216.-(x/4.));
Limits[3][1] = Z(-1./9., -1./27.-x/2.);
Limits[4][0] = Limits[4][1] = 8;
Limits[5][0] = Limits[5][1] = 4;
Limits[6][0] = Limits[6][1] = 2;
}



/*--------------------------- END OF FILE ---------------------------------*/
