/***************************************************************************
 *                                                                         *
 *  Filename : Table.c                                                     *
 *                                                                         *
 *  Description : Routine for calculating and printing tables of           *
 *                representations of quasi-simnple Liegroups.              *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *                         Modification History                            *
 *                                                                         *
 *  Date      Author       Comments                                        *
 * ----------------------------------------------------------------------  *
 *  9.8.93    R.Boedi      Created.                                        *
 * 11.8.93    R.Boedi      Changed return value of function CreateRepTable.*
 * 25.8.93    M.Joswig     Included list reversion before deletion of      *
 *                         equivalent representations in CreateRepTable.   *
 * 22.9.93    R.Boedi      Added NewLine parameter to CreateRepTable().    *
 * 29.3.94    R.Boedi      Fixed bug in CreateRepTable(). Overflow errors  *
 *                         in DimIrrRep() caused this function to exit.    *
 *                         Now CreateRepTable() exits only in case of a    *
 *                         memory allocation failure.                      *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 * Copyright (C) 1993 Richard Boedi, Michael Joswig.  All rights reserved. *
 *                                                                         *
 ***************************************************************************/

/*----------------------------- INCLUDES ----------------------------------*/

#include "IrrReps.h"
#include "Protos.h"

/*--------------------------- GLOBAL DATA ---------------------------------*/

/* This is needed for the qsort comparison routine called qsortCompare(). */
static unsigned long  Global_Rank;

/*------------------------------- CODE ------------------------------------*/

/***************************************************************************
 *                                                                         *
 *  Function name : CreateRepTable ()                                      *
 *                                                                         *
 *  Description : Calculates and prints a table of representations of a    *
 *                quasi-simple Liegroup. The format of the table is a      *
 *                matrix (see PrintMatrix() in file Output.c).             *
 *                                                                         *
 *  For a Liegroup G of rank n the first row of that matrix is given by    *
 *    columns 1   .. n+2 : 0                                               *
 *    columns n+3 .. n+5 : center of the universal covering of G which is  *
 *                         generated by at most three elements. Here the   *
 *                         orders of these generators are specified, where *
 *                         0 means infinite order.                         *
 *    columns n+6 .. n+8 : same as columns n+3 .. n+5.                     *
 *    columns n+9 .. n+11 : same as columns n+3 .. n+5.                    *
 *                                                                         *
 * The rows following the first row contain the representations given by   *
 * the format                                                              *
 *    column 1           : dimension of the representation.                *
 *    column 2           : centralizer of the representation               *
 *                         (1 = R, 2 = C, 4 = H).                          *
 *    columns 3   .. n+2 : dominant weight of the representation.          *
 *    columns n+3 .. n+11 : the kernel of the representation.              *
 *                          Since this kernel is generated by at most      *
 *                          three elements, we have 9 columns for its      *
 *                          specification.                                 *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : CreateRepTable (*Group, Dim_min, Dim_max, NewLine)          *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *    (unsigned long) Dim_min, Dim_max:                                    *
 *       The minimal and maximal dimensions of representations occuring    *
 *       in the table.                                                     *
 *    (BOOL) NewLine:                                                      *
 *       if FALSE, the table is printed in one line.                       *
 *                                                                         *
 *  Return value :                                                         *
 *    (long)                                                               *
 *          0, if no representation of complex dimension <= Dim_max exists.*
 *          1, if at least one representation of complex dimension         *
 *             <= Dim_max exists.                                          *
 *         -1, if no memory was available.                                 *
 *                                                                         *
 ***************************************************************************/

long  CreateRepTable (struct LiegroupClass *Group, unsigned long Dim_min, unsigned long Dim_max, BOOL NewLine)

{
RepList    			MyRepList = NULL, TempNode, TempNode2, DelNode;
struct Vector    *Weight = NULL, Weight1, Weight2, *ConjWeight = NULL , *Center = NULL;
struct Matrix    *Table = NULL, *KernelMat = NULL;
unsigned long     ComplexDimen, ComplexDimen1, RealDimen, Digit, i, Rows;
long              RepsExist = 0;
enum FieldType    Central;
enum ErrorType    RetCode;
BOOL              Success = FALSE;

if (Group->field == COMPLEX)
   {
   Weight = CreateVector (2 * Group->rank);
   Weight1.length = Weight2.length = Group->rank;
   Weight1.elements = Weight->elements; Weight2.elements = &Weight->elements[Group->rank];
   }
else
   Weight = CreateVector (Group->rank);

Center = CenterUC (Group);

/* Create linked list of representations */
if (Weight && Center)
   {
   for (Digit = 0, Weight->elements[0] = 1; ; )
      {
      if (Group->field == COMPLEX)
         {
         RetCode = DimIrrRep (Group, &Weight1, &ComplexDimen);
         if (RetCode == MEM_ERROR)
            {
            Success = FALSE;
            break;
            }
         if (RetCode == NO_ERROR)
            RetCode = DimIrrRep (Group, &Weight2, &ComplexDimen1);
         if (RetCode == NO_ERROR)
            RetCode = VMult (ComplexDimen, ComplexDimen1, &ComplexDimen);
         }
      else
         RetCode = DimIrrRep (Group, Weight, &ComplexDimen);
         
#ifdef DEBUG
   printf ("Dimension : %lu, Weight : %lu, RC : %d \n", ComplexDimen, Digit, (int) RetCode);
#endif

      if (RetCode == MEM_ERROR)
         {
         Success = FALSE;
         break;
         }

      Central = Centralizer (Group, Weight);
      if (Central == R) RealDimen = ComplexDimen;
      else
         if (VMult (ComplexDimen, 2, &RealDimen) == OVERFLOW_ERROR)
            RetCode = OVERFLOW_ERROR;

      if (Central != UNSPECIFIED)
         {
         if ((RealDimen >= Dim_min) && (RealDimen <= Dim_max) && (RetCode == NO_ERROR))
            if (TempNode = RepListConstruct (Weight, RealDimen, Central, MyRepList))
               MyRepList = TempNode;
             else
               {
               RetCode = MEM_ERROR;
               break;
               }

         if ((ComplexDimen > Dim_max) || (RetCode == OVERFLOW_ERROR))
            {
            for (i = 0; i <= Digit; i++)  Weight->elements[i] = 0;
            if ((Digit + 1) >= Weight->length)
               {
               Success = TRUE;
               break;
               }
            else
               {
               Digit++;
               Weight->elements[Digit]++;
               }
            }
         else
            {
            RepsExist = 1;
            Weight->elements[0]++;
            Digit = 0;
            }
         }
      else
         {
         RetCode = MEM_ERROR;
         break;
         }
      }
   }

if (Success)
   {
	/* Reverse the list of all representations.
		This is necessary, because we want to keep the "first" one of an equivalence
		class in the next step. */
	MyRepList = Reverse (MyRepList);
   /* Delete equivalent representations */
   for (TempNode = MyRepList; TempNode != NULL; TempNode = TempNode->NextNode)
      {
      ConjWeight = WeightConj (Group, &(TempNode->weight));
      for (TempNode2 = TempNode; TempNode2->NextNode != NULL; TempNode2 = TempNode2->NextNode)
         if (TempNode->dimension == TempNode2->NextNode->dimension)
            if (VectorCmp (&(TempNode2->NextNode->weight), ConjWeight))
               {
               DelNode = TempNode2->NextNode;
               TempNode2->NextNode = (TempNode2->NextNode)->NextNode;
               free (DelNode);
               if (TempNode2->NextNode == NULL) break;
               }
      FreeVector (ConjWeight);
      }

   /* Create representation table matrix */
   for (TempNode = MyRepList, Rows = 0; TempNode != NULL; TempNode = TempNode->NextNode, Rows++);

   if (Table = CreateMatrix (Rows + 1, Weight->length + 11))
      {
      /* Create first header row of the representation table matrix */
      for (i = 0; i < (Weight->length + 2); i++)
         Table->prow[0][i] = 0;
      for (i = Weight->length + 2; i < (Weight->length + 2 + Center->length); i++)
         Table->prow[0][i] =
         Table->prow[0][i + Center->length] =
         Table->prow[0][i + 2 * Center->length] =
         Center->elements[i - Weight->length - 2];

      /* Copy the linked list into the matrix */
      for (TempNode = MyRepList, Rows = 1; Rows < Table->rows; TempNode = TempNode->NextNode, Rows++)
         {
         Table->prow[Rows][0] = (long)TempNode->dimension;
         Table->prow[Rows][1] = (long)TempNode->centralizer;
         for (i = 2; i < (Weight->length + 2); i++)
            Table->prow[Rows][i] = TempNode->weight.elements[i-2];

         /* Compute the kernel of the representation and store it in the matrix */
         if (KernelMat = Kernel (Group, &(TempNode->weight)))
            {
            for (i = 0; i < KernelMat->length; i++)
               Table->prow[Rows][i + Weight->length + 2] = KernelMat->elements[i];
            FreeMatrix (KernelMat);
            }
         else
            {
            RetCode = MEM_ERROR;
            break;
            }
         }

      /* Sort the rows of the matrix increasingly */
      if (RetCode != MEM_ERROR)
         {
         Global_Rank = Weight->length;
         qsort (Table->prow[1], Table->rows - 1, Table->columns * sizeof (*(Table->elements)), qsortCompare); /* MIC: "&" erased */
         printf ("%c%lu %lu %ld ", Group->type, Group->rank, Group->field, Group->index);
         if (NewLine) printf ("\n");
         printf ("%lu %lu ", Dim_min, Dim_max);
         if (NewLine) printf ("\n");
         PrintMatrix (Table, UNSIGNED, NewLine);
         }
      }
   }

FreeVector (Weight);
FreeVector (Center);
FreeRepList (MyRepList);
FreeMatrix (Table);

if (RetCode != MEM_ERROR) return (RepsExist);
else                      return (-1);
}


/***************************************************************************
 *                                                                         *
 *  Function name : qsortCompare ()                                        *
 *                                                                         *
 *  Description : A comparison routine for the ANSI qsort procedure.       *
 *                Compares two rows of a representation table matrix.      *
 *                The sorting criterions are the dimension (sorted         *
 *                increasingly) and the dominant weight vector (sorted     *
 *                decreasingly).                                           *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : qsortCompare (Row1, Row2)                                   *
 *                                                                         *
 *  Parameters :                                                           *
 *    (const void *) Row1, Row2:                                           *
 *       Pointers to the two rows being compared.                          *
 *                                                                         *
 *  Return value :                                                         *
 *    (int)                                                                *
 *       0, if both objects are equal                                      *
 *      -1, if first object is below the second object                     *
 *       1, if first object is above the second object                     *
 *                                                                         *
 ***************************************************************************/

int   qsortCompare (const void *Row1, const void *Row2)

{
unsigned long  i;

if ((unsigned long)((unsigned long *)Row1)[0] == (unsigned long)((unsigned long *)Row2)[0])
   {
   for (i = 2; i < (Global_Rank + 2); i++)
      {
      if ((unsigned long)((unsigned long *)Row1)[i] > (unsigned long)((unsigned long *)Row2)[i])
         return (-1);
      if ((unsigned long)((unsigned long *)Row1)[i] < (unsigned long)((unsigned long *)Row2)[i])
         return (1);
      }
   return (0);
   }
else
   if ((unsigned long)((unsigned long *)Row1)[0] > (unsigned long)((unsigned long *)Row2)[0])
      return (1);
   else
      return (-1);
}

/*--------------------------- END OF FILE ---------------------------------*/
