/***************************************************************************
 *                                                                         *
 *  Filename : TeXit.c                                                     *
 *                                                                         *
 *  Description : TeX back end for the Lie algebra representation package. *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *                         Modification History                            *
 *                                                                         *
 *  Date      Author       Comments                                        *
 * ----------------------------------------------------------------------  *
 * sometime   R.Boedi      Created.                                        *
 * 24.8.93	  M.Joswig     Changed to new file format.                     *
 * 27.8.93    R.Boedi      Fixed a few bugs. More modular setup.           *
 *  7.9.93    R.Boedi      Completely redesigned and restructured.         *
 * 13.9.93    M.Joswig     Some minor changes in tex macros.               *
 * 15.9.93    RB/MJ        Some major changes in tex macros.               *
 * 23.9.93    R.Boedi      Adapted to contragradient representations in    *
 *                         the complex case.                               *
 * 27.9.93    R.Boedi      Fixed bug with the GroupMode flag.              *
 * 29.9.93    M.Joswig     Fixed scanf statement with enum types for MPW.  *
 * 1.10.93    R.Boedi      Fixed function group_string() for exceptional   *
 *                         isomorphies.                                    *
 * 10.12.94   R.Boedi      TeXit now creates only ONE table for each of    *
 *                         the real forms of E8, F4 and G2, since they     *
 *                         don't differ at all.                            *
 * 27.1.95    RB/MJ        Introduced -full flag and -v flag.              *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 * Copyright  1993 Richard Boedi, Michael Joswig.  All rights reserved.    *
 *                                                                         *
 ***************************************************************************/

/*----------------------------- INCLUDES ----------------------------------*/

#include "IrrReps.h"
#include "Protos.h"

#include "stdio.h"
#include "stdlib.h"
#include "string.h"

/*----------------------------- DEFINES -----------------------------------*/

#define  STRLENGTH      256
#define  FORMATLENGTH    32

/*----------------------- STRUCTURE DEFINITIONS ---------------------------*/

struct   TableHeader
   {
   struct LiegroupClass    group;
   unsigned long           virtualrank;
   BOOL                    A1R;
   unsigned long           dim_min, dim_max;
   unsigned long           realdimension;
   struct Vector          *center;
   long                    no_of_generators;
   struct Matrix          *linearizer;
   struct Vector          *natural_rep;
   };

struct   TableEntry
   {
   unsigned long     dimension;
   enum FieldType    centralizer;
   struct Vector    *weight;
   struct Matrix    *kernel;
   char              kernelstrg[STRLENGTH];
   };

struct   GroupRange
   {
   struct LiegroupClass    group;
   unsigned long           index_min, index_max;
   unsigned long           dim_min, dim_max;
   };

struct   Settings
   {
   BOOL     FullOutput;
   };

/*------------------------------ PROTOS -----------------------------------*/

enum ErrorType ReadArgs (int, char **, struct Settings *);
BOOL           ReadHeader (struct TableHeader *);
BOOL           ReadFirstLine (struct TableHeader *);
BOOL           ReadTableEntry (struct TableEntry *, struct TableHeader *);
BOOL           CheckRealForms (struct LiegroupClass *);
void           PrintTeXHeader (void);
void           PrintTableHeader (struct TableHeader *);
void           PrintTitle (struct TableHeader *);
void           PrintTableEntry (struct TableEntry *, struct TableHeader *);
void           PrintEndOfTable (char);
void           PrintShortHeader (struct GroupRange *);
void           PrintGroupRange (struct GroupRange *);
void           FillGroupRange (struct GroupRange *, struct TableHeader *);
char          *linearizer (char *, struct LiegroupClass *);
char          *rep_string (char *, struct Vector *, BOOL);
char          *algebraclass_string (char *, struct LiegroupClass *);
char          *algebra_string (char *, struct LiegroupClass *);
char          *group_string (char *, struct LiegroupClass *, char *);

/*--------------------------- GLOBAL DATA ---------------------------------*/

char *field_symbol[]       = { "", "\\RR", "\\CC", "", "\\HH" };
char *generator_symbol[]   = { "z", "z'", "z''" };
char *full_center[]        = { "z", "z,z'", "z,z',z''" };

char  Buffer[STRLENGTH];

BOOL  RealFormE8 = FALSE, RealFormF4 = FALSE, RealFormG2 = FALSE;

/*------------------------------- CODE ------------------------------------*/


main (int argc, char *argv[])

{
struct Settings         Settings;
struct TableHeader      TableHeader;
struct TableEntry       TableEntry;
struct GroupRange       GroupRange;
enum ErrorType          TheInput;
BOOL                    ShortMode = FALSE, GroupMode = FALSE, MoreLines, SkipIt;


if ((TheInput = ReadArgs (argc, argv, &Settings)) != NO_ERROR)
   return ((int) TheInput);

TableHeader.center     = CreateVector (3);
TableHeader.linearizer = CreateMatrix (3, 3);
TableEntry.kernel      = CreateMatrix (3, 3);

if (!TableHeader.center || !TableHeader.linearizer || !TableEntry.kernel)
   return (-1);

PrintTeXHeader ();

while (ReadHeader (&TableHeader))
   {
   TableEntry.weight       = CreateVector (TableHeader.virtualrank);
   TableHeader.natural_rep = CreateVector (TableHeader.virtualrank);

   if (!TableEntry.weight || !TableHeader.natural_rep)
      return (-1);

   if (ReadFirstLine (&TableHeader))
      {
		/* only the classical types have "natural" representations */
      if (TableHeader.group.type < 'E')
			TableHeader.natural_rep->elements[0] = 1;
      MoreLines = ReadTableEntry (&TableEntry, &TableHeader);

      if ((Settings.FullOutput == FALSE) && (!MoreLines) && VectorCmp (TableEntry.weight, TableHeader.natural_rep))
         {
         if (!ShortMode)
            {
            ShortMode = TRUE;
            FillGroupRange (&GroupRange, &TableHeader);
            PrintShortHeader (&GroupRange);
            }

         GroupMode = TRUE;

         if ((GroupRange.group.type == TableHeader.group.type) &&
             (GroupRange.group.rank == TableHeader.group.rank) &&
             (GroupRange.group.field == TableHeader.group.field) &&
             (GroupRange.dim_min == TableHeader.dim_min) &&
             (GroupRange.dim_max == TableHeader.dim_max))
            {
            if (GroupRange.index_min > TableHeader.group.index)
               GroupRange.index_min = TableHeader.group.index;
            if (GroupRange.index_max < TableHeader.group.index)
               GroupRange.index_max = TableHeader.group.index;
            }
         else
            {
            PrintGroupRange (&GroupRange);
            FillGroupRange (&GroupRange, &TableHeader);
            }
         }
      else /* there are other representations different from the natural one */
         {
         if (GroupMode)
            {
            PrintGroupRange (&GroupRange);
            printf ("\\bigskip\n");
            }

         if (ShortMode) printf("\\vfil\\par\n");
         ShortMode = GroupMode = FALSE;

         SkipIt = CheckRealForms (&(TableHeader.group));

         if (!SkipIt)
            {
            PrintTitle (&TableHeader);
            PrintTableHeader (&TableHeader);
            }

         if (MoreLines)
            do {
               if (!SkipIt)
                  PrintTableEntry (&TableEntry, &TableHeader);
               } while (ReadTableEntry (&TableEntry, &TableHeader));

         if (!SkipIt)
            {
            PrintTableEntry (&TableEntry, &TableHeader);
            PrintEndOfTable (TableHeader.group.type);
            }
         }
      }

   FreeVector (TableHeader.natural_rep);
   FreeVector (TableEntry.weight);
   }

if (GroupMode) PrintGroupRange (&GroupRange);

FreeVector (TableHeader.center);
FreeMatrix (TableHeader.linearizer);
FreeMatrix (TableEntry.kernel);

printf ("\n\n\\bye\n");
return (0);
}



/***************************************************************************
 *                                                                         *
 *  Function name : ReadArgs ()                                            *
 *                                                                         *
 *  Description : Reads the command line arguments and sets the tags       *
 *                of the struct Settings accordingly.                      *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : ReadArgs (argc, *argv[], *Settings)                         *
 *                                                                         *
 *  Parameters :                                                           *
 *    (int) argc:                                                          *
 *       Number of arguments.                                              *
 *    (char **) argv:                                                      *
 *       Array of arguments.                                               *
 *    (struct Settings *) Settings:                                        *
 *       The tags of this structure are filled.                            *
 *                                                                         *
 *  Return value :                                                         *
 *    (enum ErrorType)                                                     *
 *       SYNTAX_ERROR,   if an argument was invalid.                       *
 *       NO_ERROR,       if argument parsing was successful.               *
 *                                                                         *
 ***************************************************************************/

enum ErrorType  ReadArgs (int argc, char *argv[], struct Settings *Settings)

{
int            i = 1;
enum ErrorType TheInput = NO_ERROR;


Settings->FullOutput = FALSE;

while ((i < argc) && (TheInput == NO_ERROR))
   {

	if (strcmp(argv[i],"-full") == 0)
      {
      Settings->FullOutput = TRUE;
      i++;
      continue;
      }

	if (strcmp(argv[i],"-v") == 0)
      {
		fprintf(stderr,"Version: %s\n",VERSION);
      i++;
      continue;
      }

	TheInput = SYNTAX_ERROR;
	}

return (TheInput);
}


/***************************************************************************
 *                                                                         *
 *  Function name : ReadHeader ()                                          *
 *                                                                         *
 *  Description : Reads the table header and stores the type of the        *
 *                algebra, the minimal and maximal dimensions of           *
 *                representations, and the virtual rank in the TableHeader *
 *                structure.                                               *
 *                Sets the structure tag A1R accordingly.                  *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : ReadHeader (TableHeader)                                    *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct TableHeader *) TableHeader:                                  *
 *       This structure is filled.                                         *
 *                                                                         *
 *  Return value : (BOOL)                                                  *
 *    TRUE,  if a header line could be read successfully.                  *
 *   FALSE,  if not.                                                       *
 *                                                                         *
 ***************************************************************************/

BOOL  ReadHeader (struct TableHeader *TableHeader)

{
char  FormatStr[FORMATLENGTH] = " %c%ld %ld %ld %lu %lu ? ";
long	f;

FormatStr[23] = OuterLeft;
if (scanf (FormatStr, &(TableHeader->group.type),
                      &(TableHeader->group.rank),
							 &f, /* at least Apple's MPW C is not able to read enums */
                      &(TableHeader->group.index),
                      &TableHeader->dim_min, &TableHeader->dim_max) == 6)
   {
   TableHeader->group.field = (enum FieldType) f;
   TableHeader->A1R = ((TableHeader->group.type == 'A') &&
                       (TableHeader->group.rank == 1) &&
                       (TableHeader->group.field == 1));
   TableHeader->virtualrank = (TableHeader->group.field == 0) ? TableHeader->group.rank * 2
                                                              : TableHeader->group.rank;
   return (TRUE);
   }
else
   {
   TableHeader->A1R = FALSE;
   TableHeader->virtualrank = 0;
   return (FALSE);
   }
}


/***************************************************************************
 *                                                                         *
 *  Function name : ReadFirstLine ()                                       *
 *                                                                         *
 *  Description : Reads the first line of a representation table and       *
 *                stores the center of the universal covering of the       *
 *                given group in TableHeader->center.                      *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : ReadFirstLine (*TableHeader)                                *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct TableHeader *) TableHeader:                                  *
 *       This structure is filled.                                         *
 *                                                                         *
 *  Return value :                                                         *
 *    (BOOL)                                                               *
 *       TRUE, if a second line in the representation table is present.    *
 *      FALSE, if not.                                                     *
 *                                                                         *
 ***************************************************************************/

BOOL  ReadFirstLine (struct TableHeader *TableHeader)

{
long  i, Eof;
char  bracket;
char  FormatStr1[FORMATLENGTH] = " ? 0 ? 0 ?";
char  FormatStr2[FORMATLENGTH] = " 0 ?";
char  FormatStr3[FORMATLENGTH] = " %ld ? %ld ? %ld ?";

FormatStr1[1]  = InnerLeft;
FormatStr1[5]  = FormatStr1[9]  = Delim;
FormatStr2[3]  = Delim;
FormatStr3[5]  = FormatStr3[11] = FormatStr3[17] = Delim;

/* Skip the column of the dimension and the column of the centralizer. */
scanf (FormatStr1);

/* Skip the columns of the weight. */
for (i = 0; i < TableHeader->virtualrank; i++) scanf (FormatStr2);

/* Read the center of the universal covering group. */
scanf (FormatStr3, &(TableHeader->center->elements[0]),
                   &(TableHeader->center->elements[1]),
                   &(TableHeader->center->elements[2]));

/* Skip til InnerRight. */
for (Eof = scanf ("%c", &bracket); Eof && (bracket != InnerRight); Eof = scanf ("%c",&bracket));

/* Skip Delim. */
scanf (" %c", &bracket);

if (bracket == Delim) return (TRUE);
else                  return (FALSE);
}


/***************************************************************************
 *                                                                         *
 *  Function name : ReadTableEntry ()                                      *
 *                                                                         *
 *  Description : Reads an ordinary line of a representation table and     *
 *                stores the values in struct TableEntry.                  *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : ReadTableEntry (*TableEntry, *TableHeader)                  *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct TableEntry *) TableEntry:                                    *
 *       This structure is filled.                                         *
 *    (struct TableHeader *) TableHeader:                                  *
 *       Contains info about the algebra.                                  *
 *                                                                         *
 *  Return value :                                                         *
 *    (BOOL)                                                               *
 *       TRUE, if a further line in the representation table is present.   *
 *      FALSE, if not.                                                     *
 *                                                                         *
 ***************************************************************************/

BOOL  ReadTableEntry (struct TableEntry *TableEntry, struct TableHeader *TableHeader)

{
int   i;
long  cent;
char  bracket;
char	FormatStr1[FORMATLENGTH] = " ? %ld ? %ld";
char  FormatStr2[FORMATLENGTH] = " ?";
char  FormatStr3[FORMATLENGTH] = " ? %ld";
char	FormatStr4[FORMATLENGTH] = " ? %ld ? %ld ? %ld";

FormatStr1[1] = InnerLeft;
FormatStr1[7] = Delim;
FormatStr2[1] = InnerRight;
FormatStr3[1] = Delim;
FormatStr4[1] = FormatStr4[7] = FormatStr4[13] = Delim;

if (scanf (FormatStr1, &(TableEntry->dimension), &cent) == 2)
   {
   TableEntry->centralizer = (enum FieldType) cent;
   for (i = 0; i < TableHeader->virtualrank; i++)
      scanf (FormatStr3, &(TableEntry->weight->elements[i]));
   for (i = 0; i < 3; i++)
      scanf (FormatStr4, &(TableEntry->kernel->prow[i][0]),
                         &(TableEntry->kernel->prow[i][1]),
                         &(TableEntry->kernel->prow[i][2]));
   scanf (FormatStr2);

   /* Read Delim or OuterRight. */
   scanf (" %c", &bracket);

   if (bracket == Delim) return (TRUE);
   else                  return (FALSE);
   }
else
   return (FALSE);
}


/***************************************************************************
 *                                                                         *
 *  Function name : CheckRealForms ()                                      *
 *                                                                         *
 *  Description : Checks whether or not a real form of E8, F4 or G2 has    *
 *                already been printed out. Uses the global booleans       *
 *                RealFormxx, where xx = E8,F4,G2.                         *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : CheckRealForms (*LieGroupClass)                             *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *                                                                         *
 *  Return value :                                                         *
 *    (BOOL)                                                               *
 *       TRUE, if the real form has already been printed                   *
 *      FALSE, if not.                                                     *
 *                                                                         *
 ***************************************************************************/

BOOL  CheckRealForms (struct LiegroupClass *Group)

{
BOOL  Found = FALSE;

if (RealFormE8)
   if (Group->type == 'E')
      if (Group->rank == 8)
         if (Group->field == 1) Found = TRUE;

if (RealFormF4)
   if (Group->type == 'F')
      if (Group->field == 1) Found = TRUE;

if (RealFormG2)
   if (Group->type == 'G')
      if (Group->field == 1) Found = TRUE;

return (Found);
}



/***************************************************************************
 *                                                                         *
 *  Function name : PrintTeXHeader ()                                      *
 *                                                                         *
 *  Description : Prints the TeX prologue.                                 *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : PrintTeXHeader ()                                           *
 *                                                                         *
 *  Parameters : None                                                      *
 *                                                                         *
 *  Return value : None                                                    *
 *                                                                         *
 ***************************************************************************/

void  PrintTeXHeader ()

{
printf ("%% This file has been generated automagically ;-) \n\n");
printf ("\\input Gruppen.def\n");
printf ("\\input AMSSYM.DEF\n");
printf ("\\input AMSBourbaki.def\n");
printf ("\\input AMSFraktur12.def\n");
printf ("\\input twelve.def\n");
printf ("\\input seventeen.def\n");
printf ("\\input Magic.def\n\n");

printf ("\\parindent=0pt\n");
printf ("\\nopagenumbers\n");
printf ("\\frenchspacing\n");
printf ("\\twelvepoint\n");
printf ("\\headline={\\vbox {\\tenrm Real representations of quasi-simple Liegroups. \\hfill \\rm \\number\\pageno\\smallskip\\hrule}}\n\n");
printf ("\\newbox\\dimbox\n");

printf ("\\def\\title{\\noalign {\\hrule}");
printf ("height 14pt & dimension && centralizer && dominant weight && kernel && represented group &\\cr\n");
printf ("height 2pt & \\omit && \\omit && \\omit && \\omit && \\omit &\\cr}\n\n");

printf ("\\def\\titlereduced{\\noalign {\\hrule}");
printf ("height 14pt & dimension && centralizer && dominant weight && kernel &\\cr\n");
printf ("height 2pt & \\omit && \\omit && \\omit && \\omit &\\cr}\n\n");

printf ("\\def\\hline{height 3pt & \\omit && \\omit && \\omit && \\omit && \\omit &\\cr}\n");
printf ("\\def\\hlinereduced{height 3pt & \\omit && \\omit && \\omit && \\omit &\\cr}\n\n");

printf ("\\setbox\\TableHeader=\\vbox{\n");
printf ("\\offinterlineskip\n");
printf (" \\halign {\n");
printf (" \\vrule # & \\hskip 0.8em \\hfil # \\hfil &\n");
printf (" \\vrule # & \\hskip 0.8em \\hfil # \\hfil &\n");
printf (" \\vrule # & \\hskip 0.8em \\hfil # \\hfil &\n");
printf (" \\vrule # & \\hskip 0.8em \\hfil # \\hfil &\n");
printf (" \\vrule # & \\hskip 0.8em \\hfil # \\hfil &\n");
printf (" \\vrule# \\cr\n");
printf ("\\title}}\n");
printf ("\\headsize=\\ht\\TableHeader\n\n");

}


/***************************************************************************
 *                                                                         *
 *  Function name : PrintTableHeader ()                                    *
 *                                                                         *
 *  Description : Prints the definition header of a TeX table.             *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : PrintTableHeader (TableHeader)                              *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct TableHeader *) TableHeader:                                  *
 *       Contains the info for the table header.                           *
 *                                                                         *
 *  Return value : None                                                    *
 *                                                                         *
 ***************************************************************************/

void  PrintTableHeader (struct TableHeader *TableHeader)

{
printf ("{\n");
printf ("\\setbox\\dimbox=\\hbox{%ld}\n",TableHeader->dim_max);
printf ("\\offinterlineskip\n");
printf (" \\halign {\n");
printf (" \\vrule # & \\hskip 0.8em \\hfil # \\hfil &\n");
printf (" \\vrule # & \\hskip 0.8em \\hfil # \\hfil &\n");
printf (" \\vrule # & \\hskip 0.8em \\hfil # \\hfil &\n");
printf (" \\vrule # & \\hskip 0.8em \\hfil # \\hfil &\n");

if (TableHeader->group.type <= 'D') printf (" \\vrule # & \\hskip 0.8em \\hfil # \\hfil &\n");

printf (" \\vrule# \\cr\n");

if (TableHeader->group.type <= 'D') printf ("\\title\n");
else                                printf ("\\titlereduced\n");
}


/***************************************************************************
 *                                                                         *
 *  Function name : PrintTitle ()                                          *
 *                                                                         *
 *  Description : Prints the title of a representation table.              *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : PrintTitle (*TableHeader)                                   *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct TableHeader *) TableHeader:                                  *
 *       Contains the info for the title.                                  *
 *                                                                         *
 *  Return value : None                                                    *
 *                                                                         *
 ***************************************************************************/

void  PrintTitle (struct TableHeader *TableHeader)

{
long  i;


printf ("\\Table {\\hbox{\\hbox to5.5cm{\\seventeenpoint\\bf %s\\hfil}", algebraclass_string (Buffer, &(TableHeader->group)));

if (strcmp (algebra_string (Buffer, &(TableHeader->group)), "{}"))
   printf ("\\hbox to3.5cm{---\\hfil $%s$ \\hfil---}", algebra_string (Buffer, &(TableHeader->group)));

if (TableHeader->group.field == COMPLEX)
   printf ("\\hskip2cm{real dimension %ld}}\n\\smallskip\n", (unsigned long)(2 * GroupDimension (&(TableHeader->group))));
else
   printf ("\\hskip2cm{real dimension %ld}}\n\\smallskip\n", (unsigned long)GroupDimension (&(TableHeader->group)));

/* We treat the real forms of G2 separately because we merge their tables */
if ((TableHeader->group.type == 'G') && (TableHeader->group.field == 1))
   {
   printf ("Center $Z$ of the universal covering is $Z = 1$ (if $r=-14$).\\par\n");
   printf ("Center $Z$ of the universal covering is $Z = \\langle z \\rangle \\cong \\ZZ_2$ (if $r=2$).\\par\n");
   printf ("Maximally representable group is characterized by kernel $Z$.\\par\n");
   }
else   
   {
   printf ("Center $Z$ of the universal covering is $Z =");

   for (i = TableHeader->no_of_generators = 0; i < TableHeader->center->length; i++)
      if (TableHeader->center->elements[i] != 1)
         TableHeader->no_of_generators++;

   if (TableHeader->no_of_generators == 0) printf (" 1$.\\par\n");
   else
      {
      printf (" \\langle %s", generator_symbol[0]);

      for (i = 1; i < TableHeader->no_of_generators; i++)
         printf (",%s", generator_symbol[i]);

      Buffer[0] = 0;

      if (TableHeader->center->elements[0] != 0)
         sprintf (Buffer, "%ld", TableHeader->center->elements[0]);

      printf (" \\rangle \\cong \\ZZ_{%s}", Buffer);

      for (i = 1; i < TableHeader->no_of_generators; i++)
         {
         Buffer[0] = 0;
         if (TableHeader->center->elements[i] != 0) sprintf (Buffer, "%ld", TableHeader->center->elements[i]);
         printf (" \\times \\ZZ_{%s}", Buffer);
         }
      printf ("$.\\par\n");
      }

   printf ("Maximally representable group is characterized by kernel $%s$.\\par\n", linearizer (Buffer, &(TableHeader->group)));
   }

printf ("Irreducible real representations\
 $\\rho$ with $%lu \\leq \\dim_\\RR \\rho \\leq %lu$:}\n", TableHeader->dim_min, TableHeader->dim_max);
}


/***************************************************************************
 *                                                                         *
 *  Function name : PrintTableEntry ()                                     *
 *                                                                         *
 *  Description : Prints an ordinary line of a representation table.       *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : PrintTableEntry (*TableEntry, *TableHeader)                 *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct TableEntry *) TableEntry:                                    *
 *       Contains info about a representation of the algebra.              *
 *    (struct TableHeader *) TableHeader:                                  *
 *       Contains info about the algebra.                                  *
 *                                                                         *
 *  Return value : None                                                    *
 *                                                                         *
 ***************************************************************************/

void  PrintTableEntry (struct TableEntry *TableEntry, struct TableHeader *TableHeader)

{
int               i, j;
struct Vector     Weight1, Weight2, KernelVec;
char              Temp[STRLENGTH];


if (!TableHeader->A1R || (TableEntry->dimension < (TableHeader->dim_min + 10)))
   {
   printf (" height 12pt & \\hbox to \\wd\\dimbox{\\hfill %ld} && $%s$ && ", TableEntry->dimension, field_symbol[(int)TableEntry->centralizer]);

   if (TableHeader->group.field != COMPLEX)
      printf ("$%s$ &&", rep_string (Buffer, TableEntry->weight, FALSE));
   else
      {
      Weight1.length = Weight2.length = TableHeader->virtualrank / 2;
      Weight1.elements = TableEntry->weight->elements;
      Weight2.elements = &(TableEntry->weight->elements[Weight1.length]);
      printf ("$(%s,", rep_string (Buffer, &Weight1, FALSE));
      printf ("%s)$ &&", rep_string (Buffer, &Weight2, TRUE));
      }

   KernelVec.length   = TableEntry->kernel->columns;
   KernelVec.elements = TableEntry->kernel->prow[0];

   if (VectorNull (&KernelVec))
      {
      strcpy (TableEntry->kernelstrg, "1");
      printf ("1 &");
      }
   else /* Number of generators > 0 */
      {
      for (Buffer[0] = 0, i = 0; i < TableEntry->kernel->rows; i++)
         {
         for (j = 0; j < TableHeader->no_of_generators; j++)
            {
            switch (TableEntry->kernel->prow[i][j])
               {
               case 0:
                  Temp[0] = 0;
                  break;
               case 1:
                  strcpy (Temp, generator_symbol[j]);
                  break;
               default:
                  sprintf (Temp, "{%s}^{%ld}", generator_symbol[j], TableEntry->kernel->prow[i][j]);
                  break;
               }
            strcat (Buffer, Temp);
            }
         strcat (Buffer, ",");
         }

      /* Delete Delim. */
      while (Buffer[strlen(Buffer) - 1] == Delim)
         Buffer[strlen(Buffer) - 1] = 0;

      if (strcmp (Buffer, full_center[TableHeader->no_of_generators - 1]))
         {
         strcpy (TableEntry->kernelstrg, Buffer);
         printf ("$\\langle %s \\rangle$ &", Buffer);
         }
      else
         {
         strcpy (TableEntry->kernelstrg, "Z");
         printf ("$Z$ &");
         }
      }

   /* Name of the group. */
   if (TableHeader->group.type <= 'D')
      printf ("& %s & \\cr\n", group_string (Buffer, &(TableHeader->group), TableEntry->kernelstrg));
   else
      printf (" \\cr\n");
   }

if (TableHeader->A1R && (TableEntry->dimension == (TableHeader->dim_min + 10)))
   printf(" height 16pt & $\\vdots$ && $\\vdots$ && $\\vdots$ && $\\vdots$ && $\\vdots$ &\\cr\n");
}


/***************************************************************************
 *                                                                         *
 *  Function name : PrintEndOfTable ()                                     *
 *                                                                         *
 *  Description : Prints the ending sequence of a TeX table.               *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : PrintEndOfTable (Type)                                      *
 *                                                                         *
 *  Parameters :                                                           *
 *    (char) Type : The type of the algebra.                               *
 *                                                                         *
 *  Return value : None                                                    *
 *                                                                         *
 ***************************************************************************/

void  PrintEndOfTable (char Type)

{
if (Type <= 'D')  printf ("\\hline");
else              printf ("\\hlinereduced");

printf ("}}\n\n");
}


/***************************************************************************
 *                                                                         *
 *  Function name : PrintShortHeader ()                                    *
 *                                                                         *
 *  Description : Prints a short header for groups which have only the     *
 *                natural representation in the specified dimension        *
 *                interval.                                                *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : PrintShortHeader (*GroupRange)                              *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct GroupRange *) GroupRange:                                    *
 *       Contains info about the groups which have only the natural        *
 *       representation in the specified dimension interval.               *
 *                                                                         *
 *  Return value : None                                                    *
 *                                                                         *
 ***************************************************************************/

void  PrintShortHeader (struct GroupRange *GroupRange)

{
printf ("The following local isomorphy classes of groups have the\
 natural representation as the only irreducible real representation\
 $\\rho$ with $%lu \\leq \\dim_\\RR \\rho \\leq %lu$:\\medskip\n", GroupRange->dim_min, GroupRange->dim_max);
}


/***************************************************************************
 *                                                                         *
 *  Function name : PrintGroupRange ()                                     *
 *                                                                         *
 *  Description : Prints a range of groups which have only the             *
 *                natural representation in the specified dimension        *
 *                interval.                                                *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : PrintGroupRange (*GroupRange)                               *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct GroupRange *) GroupRange:                                    *
 *       Contains info about the groups which have only the natural        *
 *       representation in the specified dimension interval.               *
 *                                                                         *
 *  Return value : None                                                    *
 *                                                                         *
 ***************************************************************************/


void  PrintGroupRange (struct GroupRange *GroupRange)

{
int   i;
char  NoIndexStrg[FORMATLENGTH];
char *Match;

if (GroupRange->index_min == GroupRange->index_max)
   {
   GroupRange->group.index = GroupRange->index_min;
   printf ("\\hbox{\\hbox to5.5cm{\\seventeenpoint\\bf %s\\hfil}", algebraclass_string (Buffer, &(GroupRange->group)));

   if (strcmp (algebra_string (Buffer, &(GroupRange->group)), "{}"))
      printf ("\\hbox to3.5cm{---\\hfil$%s$", algebra_string (Buffer, &(GroupRange->group)));
   }
else
   {
   GroupRange->group.index = NO_INDEX;
   sprintf (NoIndexStrg, "%ld", NO_INDEX);
   algebraclass_string (Buffer, &(GroupRange->group));
   Match = strstr (Buffer, NoIndexStrg);
   Match[0] = 'r';
   for (i = 1; i < strlen (NoIndexStrg); i++) Match[i] = ' ';
   printf ("\\hbox{\\hbox to5.5cm{{\\seventeenpoint\\bf %s}\\ \\ ($%lu \\leq r \\leq %lu$)\\hfil}", Buffer, GroupRange->index_min, GroupRange->index_max);

   if (strcmp (algebra_string (Buffer, &(GroupRange->group)), "{}"))
      {
      algebra_string (Buffer, &(GroupRange->group));
      Match = strstr (Buffer, NoIndexStrg);
      Match[0] = 'r';
      for (i = 1; i < strlen (NoIndexStrg); i++) Match[i] = ' ';
      printf ("\\hbox to3.5cm{---\\hfil$%s$", Buffer);
      }
   }

if (GroupRange->group.field == COMPLEX)
   printf ("\\hfil---}\\hskip2cm{real dimension %ld}}\n\\smallskip\n", (unsigned long)(2 * GroupDimension (&(GroupRange->group))));
else
   printf ("\\hfil---}\\hskip2cm{real dimension %ld}}\n\\smallskip\n", (unsigned long)GroupDimension (&(GroupRange->group)));
}


/***************************************************************************
 *                                                                         *
 *  Function name : FillGroupRange ()                                      *
 *                                                                         *
 *  Description : Initializes a struct GroupRange with the actual values   *
 *                of a TableHeader structure.                              *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : FillGroupRange (*GroupRange, *TableHeader)                  *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct GroupRange *) GroupRange:                                    *
 *       This structure is filled with the correponding tags of struct     *
 *       TableHeader.                                                      *
 *    (struct TableHeader *) TableHeader:                                  *
 *       Contains info about the algebra.                                  *
 *                                                                         *
 *  Return value : None                                                    *
 *                                                                         *
 ***************************************************************************/


void  FillGroupRange (struct GroupRange *GroupRange, struct TableHeader *TableHeader)

{
GroupRange->group.type  = TableHeader->group.type;
GroupRange->group.rank  = TableHeader->group.rank;
GroupRange->group.field = TableHeader->group.field;
GroupRange->index_min   = TableHeader->group.index;
GroupRange->index_max   = TableHeader->group.index;
GroupRange->dim_min     = TableHeader->dim_min;
GroupRange->dim_max     = TableHeader->dim_max;
}


/***************************************************************************
 *                                                                         *
 *  Function name : linearizer ()                                          *
 *                                                                         *
 *  Description : Determines the linearizer of a simple real Lie group.    *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : linearizer (*linearizerstring, *Group)                      *
 *                                                                         *
 *  Parameters :                                                           *
 *    (char) *linearizerstring:                                            *
 *       A pointer to a string which will be filled with the linearizer.   *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *                                                                         *
 *  Return value :                                                         *
 *    (char *)                                                             *
 *       Same as *linearizerstring.                                        *
 *                                                                         *
 ***************************************************************************/

char *linearizer (char *linearizerstring, struct LiegroupClass *Group)

{
long  a, b, c, u, v;

strcpy (linearizerstring, "1");     /* default is a linear group. */

switch (Group->type)
   {
   case 'A':
      switch (Group->field)
         {
         case R:
            if (Group->rank == 1) strcpy (linearizerstring, "\\langle z^2 \\rangle");
            else
               if ((Group->rank % 2) == 0) strcpy (linearizerstring, "Z");
               else
                  if ((Group->rank % 4) == 1) strcpy (linearizerstring, "\\langle z^2 \\rangle");
                  else                        strcpy (linearizerstring, "\\langle zz' \\rangle");
            break;
         case C:
            if (Group->index > 0)
               {
               a = gcd_cf (Group->rank + 1, Group->index, &u, &v);
               b = (Group->rank + 1) / a;
               c = Group->index / a;
               gcd_cf (b, c, &u, &v);
               v = (a - v) % a;
               if ((v == 0) || (a == 1)) sprintf (linearizerstring, "\\langle z^{%ld} \\rangle", b);
               if (v == 1)               sprintf (linearizerstring, "\\langle z{z'}^{%ld} \\rangle", b);
               if (v > 1)                sprintf (linearizerstring, "\\langle z^{%ld}z'^{%ld} \\rangle", v, b);
               }
            break;
         }
      break;

   case 'B':
      switch (Group->field)
         {
         case R:
            if (Group->index >= 2) strcpy (linearizerstring, "\\langle zz' \\rangle");
            break;
         }
      break;

   case 'C':
      switch (Group->field)
         {
         case R:
            if ((Group->rank % 2) == 0) strcpy (linearizerstring, "\\langle z' \\rangle");
            else                        strcpy (linearizerstring, "\\langle z^2 \\rangle");
            break;
         }
      break;

   case 'D':
      switch (Group->field)
         {
         case R:
            if (Group->index == 2)
               if ((Group->rank % 2) == 0) strcpy (linearizerstring, "\\langle {z'}^2 \\rangle");
               else                        strcpy (linearizerstring, "\\langle z{z'}^2 \\rangle");
            if (Group->index > 2)
               if ((Group->index % 2) == 0)
                  {
                  if ((Group->index % 4) == 0)
                     if ((Group->rank % 2) == 0)
                        strcpy (linearizerstring, "\\langle zz' \\rangle");
                     else
                        strcpy (linearizerstring, "\\langle z{z'}^{2} \\rangle");
                  else
                     if ((Group->rank % 2) == 0)
                        strcpy (linearizerstring, "\\langle z'^2 \\rangle");
                     else
                        strcpy (linearizerstring, "\\langle zz'^2 \\rangle");
                  }
               else
                  strcpy (linearizerstring, "\\langle zz' \\rangle");
            break;
         case H:
            if ((Group->rank % 2) == 0) strcpy (linearizerstring, "\\langle {z'}^2 \\rangle");
            else                        strcpy (linearizerstring, "\\langle z^4 \\rangle");
            break;
         }
      break;

   case 'E':
      switch (Group->rank)
         {
         case 6:
            if (Group->field == 1)
               {
               if (Group->index == -14) strcpy (linearizerstring, "\\langle z^3 \\rangle");
               if (Group->index ==   2) strcpy (linearizerstring, "\\langle z^2 \\rangle");
               if (Group->index ==   6) strcpy (linearizerstring, "Z");
               }
            break;
         case 7:
            if (Group->field == 1)
               {
               if (Group->index == -25) strcpy (linearizerstring, "\\langle z^2 \\rangle");
               if (Group->index ==  -5) strcpy (linearizerstring, "\\langle zz' \\rangle");
               if (Group->index ==   7) strcpy (linearizerstring, "\\langle z^2 \\rangle");
               }
            break;
         case 8:
            if ((Group->field == 1) && (Group->index != 248)) strcpy (linearizerstring, "Z");
            break;
         }
      break;

   case 'F':
      if ((Group->field == 1) && (Group->index == 4)) strcpy (linearizerstring, "Z");
      break;

   case 'G':
      if ((Group->field == 1) && (Group->index == 2)) strcpy (linearizerstring, "Z");
      break;
   }

return (linearizerstring);
}



/***************************************************************************
 *                                                                         *
 *  Function name : rep_string ()                                          *
 *                                                                         *
 *  Description : Creates a string for a highest weight.                   *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : rep_string (*repstring, *Weight, Star)                      *
 *                                                                         *
 *  Parameters :                                                           *
 *    (char) *repstring:                                                   *
 *       A pointer to a string which will be filled with the highest       *
 *       weight string.                                                    *
 *    (struct Vector) *Weight:                                             *
 *       Pointer to a struct Vector which contains the highest weight.     *
 *    (BOOL) Star:                                                         *
 *       If TRUE, \lambda_k^* is printed instead of \lambda_k.             *
 *                                                                         *
 *  Return value :                                                         *
 *    (char *)                                                             *
 *       Same as *repstring.                                               *
 *                                                                         *
 ***************************************************************************/

char *rep_string (char *repstring, struct Vector *Weight, BOOL Star)

{
int   i;
char  buffer[STRLENGTH];

repstring[0] = 0;

for (i = 0; i < Weight->length; i++)
   {
   switch (Weight->elements[i])
      {
      case 0:
         break;
      case 1:
         if (repstring[0] != 0) strcat (repstring, "+");
         sprintf (buffer, "\\lambda_{%ld}", i + 1);
         strcat (repstring, buffer);
         if (Star) strcat (repstring, "^*");
         break;
      default:
         if (repstring[0] != 0) strcat (repstring, "+");
         sprintf (buffer, "{%ld}\\lambda_{%ld}", Weight->elements[i], i + 1);
         strcat (repstring, buffer);
         if (Star) strcat (repstring, "^*");
         break;
      }
   }

if (repstring[0] == 0) strcpy (repstring, "0");

return (repstring);
}


/***************************************************************************
 *                                                                         *
 *  Function name : algebraclass_string ()                                 *
 *                                                                         *
 *  Description : Creates a string for an algebra class.                   *
 *                Sets the global flags RealFormE8, RealFormG2, RealFormF4 *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : algebraclass_string (*algebrastring, *Group)                *
 *                                                                         *
 *  Parameters :                                                           *
 *    (char) *algebrastring:                                               *
 *       A pointer to a string which will be filled with the algebra string*
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *                                                                         *
 *  Return value :                                                         *
 *    (char *)                                                             *
 *       Same as *algebrastring.                                           *
 *                                                                         *
 ***************************************************************************/

char *algebraclass_string (char *algebrastring, struct LiegroupClass *Group)

{
char  buffer[STRLENGTH];

sprintf (algebrastring, "%c$_{%ld", Group->type, Group->rank);
if (Group->field != 0)
   {
   switch (Group->type)
      {
      case 'A':
         switch (Group->field)
            {
            case R:
            case H:
               sprintf (buffer, "}^{%s}$", field_symbol[Group->field]);
               break;
            case C:
               sprintf (buffer, "}^{\\CC,%ld}$", Group->index);
               break;
            }
         break;

      case 'B':
         sprintf (buffer, "}^{\\RR,%ld}$", Group->index);
         break;

      case 'C':
         switch (Group->field)
            {
            case R:
               sprintf (buffer, "}^{\\RR}$");
               break;
            case H:
               sprintf (buffer, "}^{\\HH,%ld}$", Group->index);
               break;
            }
         break;

      case 'D':
         switch (Group->field)
            {
            case R:
               sprintf (buffer, "}^{\\RR,%ld}$", Group->index);
               break;
            case H:
               sprintf (buffer, "}^{\\HH}$");
               break;
            }
         break;

      case 'E':
         if (Group->rank == 8)
            {
            sprintf (buffer, "(r)}^{\\RR}\\ \\ {\\scriptstyle (r=-248,-24,8)}$");
            RealFormE8 = TRUE;
            }
         else
            sprintf (buffer, "(%ld)}^{\\RR}$", Group->index);
         break;

      case 'F':
         sprintf (buffer, "(r)}^{\\RR}\\ \\ {\\scriptstyle (r=-52,-20,4)}$");
         RealFormF4 = TRUE;
         break;

      case 'G':
         sprintf (buffer, "(r)}^{\\RR}\\ \\ {\\scriptstyle (r=-14,2)}$");
         RealFormG2 = TRUE;
         break;
      }

   strcat (algebrastring, buffer);
   }
else
   strcat (algebrastring, "}$");


return (algebrastring);
}



/***************************************************************************
 *                                                                         *
 *  Function name : algebra_string ()                                      *
 *                                                                         *
 *  Description : Creates a string for an algebra.                         *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : algebra_string (*algebrastring, *Group)                     *
 *                                                                         *
 *  Parameters :                                                           *
 *    (char) *algebrastring:                                               *
 *       A pointer to a string which will be filled with the algebra string*
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *                                                                         *
 *  Return value :                                                         *
 *    (char *)                                                             *
 *       Same as *algebrastring.                                           *
 *                                                                         *
 ***************************************************************************/

char *algebra_string (char *algebrastring, struct LiegroupClass *Group)

{
char  buffer[STRLENGTH];

strcpy (algebrastring, "{}");

switch (Group->type)
   {
   case 'A':
      switch (Group->field)
         {
         case COMPLEX:
            sprintf (algebrastring, "\\sla {%ld} {\\CC}", Group->rank + 1);
            break;
         case R:
            sprintf (algebrastring, "\\sla {%ld} {\\RR}", Group->rank + 1);
            break;
         case C:
            sprintf (algebrastring, "\\sua {%ld} {\\CC}", Group->rank + 1);
            if (Group->index)
               {
               sprintf (buffer, "(%ld)", Group->index);
               strcat (algebrastring, buffer);
               }
            break;
         case H:
            sprintf (algebrastring, "\\sla {%ld} {\\HH}", 1 + (Group->rank - 1) / 2);
            break;
         }
      break;

   case 'B':
      switch (Group->field)
         {
         case COMPLEX:
            sprintf (algebrastring, "\\oa {%ld} {\\CC}", 2 * Group->rank + 1);
            break;
         case R:
            sprintf (algebrastring, "\\oa {%ld} {\\RR}", 2 * Group->rank + 1);
            if (Group->index)
               {
               sprintf (buffer, "(%ld)", Group->index);
               strcat (algebrastring, buffer);
               }
            break;
         }
      break;

   case 'C':
      switch (Group->field)
         {
         case COMPLEX:
            sprintf (algebrastring, "\\spa {%ld} {\\CC}", 2 * Group->rank);
            break;
         case R:
            sprintf (algebrastring, "\\spa {%ld} {\\RR}", 2 * Group->rank);
            break;
         case H:
            sprintf (algebrastring, "\\sua {%ld} {\\HH}", Group->rank);
            if (Group->index)
               {
               sprintf (buffer, "(%ld)", Group->index);
               strcat (algebrastring, buffer);
               }
            break;
         }
      break;

   case 'D':
      switch (Group->field)
         {
         case COMPLEX:
            sprintf (algebrastring, "\\oa {%ld} {\\CC}", 2 * Group->rank);
            break;
         case R:
            sprintf (algebrastring, "\\oa {%ld} {\\RR}", 2 * Group->rank);
            if (Group->index)
               {
               sprintf (buffer, "(%ld)", Group->index);
               strcat (algebrastring, buffer);
               }
            break;
         case H:
            sprintf (algebrastring, "\\saua {%ld} {\\HH}", Group->rank);
            break;
         }
      break;

   case 'E':
   case 'F':
   case 'G':
      break;
   }


return (algebrastring);
}



/***************************************************************************
 *                                                                         *
 *  Function name : group_string ()                                        *
 *                                                                         *
 *  Description : Creates a string for a group.                            *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : group_string (*groupstring, *Group)                         *
 *                                                                         *
 *  Parameters :                                                           *
 *    (char) *groupstring:                                                 *
 *       A pointer to a string which will be filled with the group string. *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *    (char) *kernel:                                                      *
 *       The kernel string of the representation.                          *
 *                                                                         *
 *  Return value :                                                         *
 *    (char *)                                                             *
 *       Same as *groupstring.                                             *
 *                                                                         *
 ***************************************************************************/

char *group_string (char *groupstring, struct LiegroupClass *Group, char *kernel)

{
char  buffer[STRLENGTH];
char *omit = "\\omit";
long  a, b, c, u, v;

strcpy (groupstring, omit);

switch (Group->type)
   {
   case 'A':
      switch (Group->field)
         {
         case COMPLEX:
            if (!strcmp (kernel, "1"))
               sprintf (groupstring, "\\SL{%ld}{\\CC}", Group->rank + 1);
            if (!strcmp (kernel, "Z"))
               sprintf (groupstring, "\\PSL{%ld}{\\CC}", Group->rank + 1);
            break;

         case R:
            if (Group->rank % 2)
               {
               if (!strcmp (kernel, "Z"))
                  sprintf (groupstring, "\\PSL{%ld}{\\RR}", Group->rank + 1);
               else
                  sprintf (groupstring, "\\SL{%ld}{\\RR}", Group->rank + 1);
               }
            else
               sprintf (groupstring, "\\SL{%ld}{\\RR}", Group->rank + 1);
            break;

         case C:
            if (!strcmp (kernel, "1"))
               sprintf (groupstring, "\\SU{%ld}{\\CC}", Group->rank + 1);
            if (!strcmp (kernel, "Z"))
               sprintf (groupstring, "\\PSU{%ld}{\\CC}", Group->rank + 1);
            if (Group->index > 0)
               {
               a = gcd_cf (Group->rank + 1, Group->index, &u, &v);
               b = (Group->rank + 1) / a;
               c = Group->index / a;
               gcd_cf (b, c, &u, &v);
               v = (a - v) % a;
               if ((v == 0) || (a == 1))
                  {
                  sprintf (buffer, "{z}^{%ld}", b);
                  if (!strcmp (kernel, buffer)) sprintf (groupstring, "\\SU{%ld}{\\CC}", Group->rank + 1);
                  }
               if (v == 1)
                  {
                  sprintf (buffer, "z{z'}^{%ld}", b);
                  if (!strcmp (kernel, buffer)) sprintf (groupstring, "\\SU{%ld}{\\CC}", Group->rank + 1);
                  }
               if (v > 1)
                  {
                  sprintf (buffer, "{z}^{%ld}{z'}^{%ld}", v, b);
                  if (!strcmp (kernel, buffer)) sprintf (groupstring, "\\SU{%ld}{\\CC}", Group->rank + 1);
                  sprintf (buffer, "{z}^{%ld}{z'}^{%ld}", a - v, b);
                  if (!strcmp (kernel, buffer)) sprintf (groupstring, "\\SU{%ld}{\\CC}", Group->rank + 1);
                  }
               }

            if ((Group->index) && (strcmp (groupstring, omit)))
               {
               sprintf (buffer, "(%ld)", Group->index);
               strcat (groupstring, buffer);
               }
            break;

         case H:
            if (!strcmp (kernel, "1"))
               sprintf (groupstring, "\\SL{%ld}{\\HH}", Group->rank / 2 + 1);
            else
               sprintf (groupstring, "\\PSL{%ld}{\\HH}", Group->rank / 2 + 1);
            break;
         }
      break;

   case 'B':
      switch (Group->field)
         {
         case COMPLEX:
            if (!strcmp (kernel, "1"))
               sprintf (groupstring, "\\Spin{%ld}{\\CC}", 2 * Group->rank + 1);
            else
               sprintf (groupstring, "\\SO{%ld}{\\CC}", 2 * Group->rank + 1);
            break;

         case R:
            if (Group->index <= 1)
               {
               if ((!strcmp (kernel, "1")) || (!strcmp (kernel, "zz'")))
                  sprintf (groupstring, "\\Spin{%ld}{\\RR}", 2 * Group->rank + 1);
               else /* kernel = Z */
                  sprintf (groupstring, "\\SO{%ld}{\\RR}", 2 * Group->rank + 1);
               }
            else
               {
               if (!strcmp (kernel, "zz'"))
                  sprintf (groupstring, "\\Spin{%ld}{\\RR}", 2 * Group->rank + 1);
               if (!strcmp (kernel, "Z"))
                  sprintf (groupstring, "\\SO{%ld}{\\RR}", 2 * Group->rank + 1);
               }

            if ((Group->index) && (strcmp (groupstring, omit)))
               {
               sprintf (buffer, "(%ld)", Group->index);
               strcat (groupstring, buffer);
               }
            break;
         }
      break;

   case 'C':
      switch (Group->field)
         {
         case COMPLEX:
            if(!strcmp(kernel, "1"))
               sprintf (groupstring, "\\Sp{%ld}{\\CC}", 2 * Group->rank);
            else /* kernel = Z */
               sprintf (groupstring, "\\PSp{%ld}{\\CC}", 2 * Group->rank);
            break;

         case R:
            if(!strcmp(kernel, "Z"))
               sprintf (groupstring, "\\PSp{%ld}{\\RR}", 2 * Group->rank);
            else /* in all cases this is Sp! */
               sprintf (groupstring, "\\Sp{%ld}{\\RR}", 2 * Group->rank);
            break;

         case H:
            if (!strcmp(kernel, "1"))
               sprintf (groupstring, "\\SU{%ld}{\\HH}", Group->rank);
            else /* kernel = Z */
               sprintf (groupstring, "\\PSU{%ld}{\\HH}", Group->rank);
            if ((Group->index) && (strcmp (groupstring, omit)))
               {
               sprintf (buffer, "(%ld)", Group->index);
               strcat (groupstring, buffer);
               }
            break;
         }
      break;

   case 'D':
      switch (Group->field)
         {
         case COMPLEX:
            if (!strcmp(kernel, "1"))
               sprintf (groupstring, "\\Spin{%ld}{\\CC}", 2 * Group->rank);
            else
               {
               if (!strcmp(kernel, "Z"))
                  sprintf (groupstring, "\\PSO{%ld}{\\CC}", 2 * Group->rank);
               else
                  {
                  if ((Group->rank % 2 == 0) &&
                     ((!strcmp(kernel, "zz'")) || (!strcmp(kernel, "z")) || (!strcmp(kernel, "z'"))))
                     sprintf (groupstring, "\\SO{%ld}{\\CC}", 2 * Group->rank);
                  if ((Group->rank % 2 == 1) && (!strcmp(kernel, "{z}^{2}")))
                     sprintf (groupstring, "\\SO{%ld}{\\CC}", 2 * Group->rank);
                  }
               }
            break;

         case R:
            if (Group->index % 2 == 1)
               {
               if (!strcmp(kernel, "Z"))
                  sprintf(groupstring, "\\SO{%ld}{\\RR}", 2 * Group->rank);
               if ((!strcmp(kernel, "1")) || (!strcmp(kernel, "zz'")))
                  sprintf(groupstring, "\\Spin{%ld}{\\RR}", 2 * Group->rank);
               }
            else
               {
               if (!strcmp(kernel, "Z"))
                  if ((Group->rank % 2) == 1)
                     sprintf(groupstring, "\\SO{%ld}{\\RR}", 2 * Group->rank);
                  else
                     sprintf(groupstring, "\\PSO{%ld}{\\RR}", 2 * Group->rank);
               if (Group->rank == 4)
               	{
               	if ((!strcmp(kernel, "zz'")) || (!strcmp(kernel, "z")) || (!strcmp(kernel, "z'")) ||
                  	(!strcmp(kernel, "zz',z''")) || (!strcmp(kernel, "zz',z'z''")) || (!strcmp(kernel, "z,z'")) ||
                  	(!strcmp(kernel, "{z}^{2}")) || (!strcmp(kernel, "z,{z'}^{2}")) || (!strcmp(kernel, "{z}^{2},z'")))
                  	sprintf(groupstring, "\\SO{%ld}{\\RR}", 2 * Group->rank);
						}
					else
						{
 	               if ((!strcmp(kernel, "zz'")) ||
   	               (!strcmp(kernel, "zz',z''")) || (!strcmp(kernel, "zz',z'z''")) || (!strcmp(kernel, "z,z'")) ||
                     (!strcmp(kernel, "{z}^{2}")) || (!strcmp(kernel, "z,{z'}^{2}")) || (!strcmp(kernel, "{z}^{2},z'")))
                     sprintf(groupstring, "\\SO{%ld}{\\RR}", 2 * Group->rank);
                  }
               if ((!strcmp(kernel, "1")) || (!strcmp(kernel, "z{z'}^{2}")) || (!strcmp(kernel, "{z}^{2}z'")))
                  sprintf(groupstring, "\\Spin{%ld}{\\RR}", 2 * Group->rank);
               }

            if ((Group->index) && (strcmp (groupstring, omit)))
               {
               if ((((Group->rank % 2) == 1) || (Group->index % 2 == 1)) && (!strcmp(kernel, "Z")))
                  sprintf (buffer, "(%ld)$^\\circ$", Group->index);
               else
                  sprintf (buffer, "(%ld)", Group->index);
               strcat (groupstring, buffer);
               }
            break;

         case H:
            if (((Group->rank % 2 == 0) && (!strcmp(kernel, "{z'}^{2}"))) || ((Group->rank % 2 != 0) && (!strcmp(kernel, "{z'}^{4}"))))
               sprintf (groupstring, "\\SaU{%ld}{\\HH}", Group->rank);
            if (!strcmp(kernel, "Z"))
               sprintf (groupstring, "\\PaU{%ld}{\\HH}", Group->rank);
            break;
         }
      break;
   }

/* exceptional isomorphisms */

if ((Group->type == 'A') && (Group->rank == 3) && (Group->field == COMPLEX) && (Group->index == 0) &&
   (!strcmp(kernel, "{z}^{2}")))
   sprintf (groupstring, "\\SO{6}{\\CC}");
if ((Group->type == 'A') && (Group->rank == 3) && (Group->field == C) && (Group->index == 0) &&
   (!strcmp(kernel, "{z}^{2}")))
   sprintf (groupstring, "\\SO{6}{\\RR}");
if ((Group->type == 'A') && (Group->rank == 3) && (Group->field == C) && (Group->index == 1) &&
   (!strcmp(kernel, "{z}^{2}")))
   sprintf (groupstring, "\\SaU{3}{\\HH}");
if ((Group->type == 'A') && (Group->rank == 3) && (Group->field == C) && (Group->index == 2) &&
   (!strcmp(kernel, "z,{z'}^{2}")))
   sprintf (groupstring, "\\SO{6}{\\RR}(2)");
if ((Group->type == 'B') && (Group->rank == 2) && (Group->field == R) && (Group->index == 2) &&
   (!strcmp(kernel, "zz'")))
   sprintf (groupstring, "\\Sp{4}{\\RR}");
if ((Group->type == 'D') && (Group->rank == 4) && (Group->field == R) && (Group->index == 2) &&
   ((!strcmp(kernel, "{z}^{2},z'")) || (!strcmp(kernel, "{z}^{2},z'"))))
   sprintf (groupstring, "\\SaU{4}{\\HH}");

return (groupstring);
}

