/***************************************************************************
 *                                                                         *
 *  Filename : WeightConj.c                                                *
 *                                                                         *
 *  Description : Routines for determining the conjugated weight of a real *
 *                simple Lie algebra and the weight of a contragradient    *
 *                representation.                                          *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *                         Modification History                            *
 *                                                                         *
 *  Date      Author       Comments                                        *
 * ----------------------------------------------------------------------  *
 *  25.4.93    R.Boedi     Created.                                        *
 *   5.8.93    R.Boedi     Changed memory allocation for vectors and       *
 *                         matrices, see file Memory.c                     *
 *  19.8.93    M.Joswig    Remember: arrays start with a 0 index!          *
 *                         Changed the permutation vector once.            *
 *  23.9.93    R.Boedi     Added function ContraGradient().                *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 * Copyright (C) 1993 Richard Boedi, Michael Joswig.  All rights reserved. *
 *                                                                         *
 ***************************************************************************/

/*----------------------------- INCLUDES ----------------------------------*/

#include "IrrReps.h"
#include "Protos.h"

/*------------------------------- CODE ------------------------------------*/

/***************************************************************************
 *                                                                         *
 *  Function Name : WeightConj ()                                          *
 *                                                                         *
 *  Description : Determines the conjugated weight of a real simple Lie    *
 *                algebra.                                                 *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : WeightConj (*Group)                                         *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *    (struct Vector) *Weight:                                             *
 *       The weight which has to be conjugated.                            *
 *                                                                         *
 *  Return value :                                                         *
 *    (struct Vector *)                                                    *
 *       The conjugated weight vector.                                     *
 *       Returns a zero pointer if the lengths of these vectors are odd    *
 *       values in the complex case, or if no temporal memory was          *
 *       available.                                                        *
 *                                                                         *
 ***************************************************************************/

struct Vector *WeightConj (struct LiegroupClass *Group, struct Vector *Weight)

{
int            i;
struct Vector *Permutation, *ConjWeight;

Permutation = CreateVector (Weight->length);
ConjWeight  = CreateVector (Weight->length);

if (Permutation && ConjWeight && ((Group->field != COMPLEX) || (Weight->length % 2 == 0)))
   {
   for (i = 0; i < Permutation->length; i++)    /* Default is the identity permutation */
      Permutation->elements[i] = i;

   switch (Group->type)
      {
      case 'A':
         switch (Group->field)
            {
            case COMPLEX:
               for (i = 0; i < (Permutation->length / 2); i++)
                  {
                  Permutation->elements[i] = i + Permutation->length / 2;
                  Permutation->elements[i + Permutation->length / 2] = i;
                  }
               break;
            case R:
               break;
            case C:
               for (i = 0; i < Permutation->length; i++)
                  Permutation->elements[i] = Permutation->length - 1 - i; /* MIC */
               break;
            case H:
               break;
            }
         break;

      case 'B':
         switch (Group->field)
            {
            case COMPLEX:
               for (i = 0; i < (Permutation->length / 2); i++)
                  {
                  Permutation->elements[i] = i + Permutation->length / 2;
                  Permutation->elements[i + Permutation->length / 2] = i;
                  }
               break;
            case R:
               break;
            case C:
               break;
            case H:
               break;
            }
         break;

      case 'C':
         switch (Group->field)
            {
            case COMPLEX:
               for (i = 0; i < (Permutation->length / 2); i++)
                  {
                  Permutation->elements[i] = i + Permutation->length / 2;
                  Permutation->elements[i + Permutation->length / 2] = i;
                  }
               break;
            case R:
               break;
            case C:
               break;
            case H:
               break;
            }
         break;

      case 'D':
         switch (Group->field)
            {
            case COMPLEX:
               for (i = 0; i < (Permutation->length / 2); i++)
                  {
                  Permutation->elements[i] = i + Permutation->length / 2;
                  Permutation->elements[i + Permutation->length / 2] = i;
                  }
               break;
            case R:
               if ((Group->rank - Group->index) % 2 != 0)
                  {
                  Permutation->elements[Permutation->length - 2] = Permutation->length - 1;
                  Permutation->elements[Permutation->length - 1] = Permutation->length - 2;
                  }
               break;
            case C:
               break;
            case H:
               if (Group->rank % 2 != 0)
                  {
                  Permutation->elements[Permutation->length - 2] = Permutation->length - 1;
                  Permutation->elements[Permutation->length - 1] = Permutation->length - 2;
                  }
               break;
            }
         break;

      case 'E':
         switch (Group->field)
            {
            case COMPLEX:
               for (i = 0; i < (Permutation->length / 2); i++)
                  {
                  Permutation->elements[i] = i + Permutation->length / 2;
                  Permutation->elements[i + Permutation->length / 2] = i;
                  }
                  break;
            case R:
               switch (Group->rank)
                  {
                  case 6:
                     if ((Group->index == 2) || (Group->index == -14) || (Group->index == -78))
                        {
                        Permutation->elements[0] = 5;
                        Permutation->elements[2] = 4;
                        Permutation->elements[4] = 2;
                        Permutation->elements[5] = 0;
                        }
                     break;
                  case 7:
                     break;
                  case 8:
                     break;
                  }
               break;
            case C:
               break;
            case H:
               break;
            }
         break;

      case 'F':
         if (Group->rank == 4)
            switch (Group->field)
               {
               case COMPLEX:
               for (i = 0; i < (Permutation->length / 2); i++)
                  {
                  Permutation->elements[i] = i + Permutation->length / 2;
                  Permutation->elements[i + Permutation->length / 2] = i;
                  }
                  break;
               case R:
                  break;
               case C:
                  break;
               case H:
                  break;
               }
         break;

      case 'G':
         if (Group->rank == 2)
            switch (Group->field)
               {
               case COMPLEX:
               for (i = 0; i < (Permutation->length / 2); i++)
                  {
                  Permutation->elements[i] = i + Permutation->length / 2;
                  Permutation->elements[i + Permutation->length / 2] = i;
                  }
                  break;
               case R:
                  break;
               case C:
                  break;
               case H:
                  break;
               }
         break;

      }
   PermuteVector (ConjWeight, Weight, Permutation);
   }

FreeVector (Permutation);

return (ConjWeight);
}


/***************************************************************************
 *                                                                         *
 *  Function Name : ContraGradient ()                                      *
 *                                                                         *
 *  Description : Determines the weight of a contragradient representation.*
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : ContraGradient (*Group, *Weight)                            *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *    (struct Vector) *Weight:                                             *
 *       The highest weight of the original representation.                *
 *                                                                         *
 *  Return value :                                                         *
 *    (struct Vector *)                                                    *
 *       The highest weight vector of the contragradient representation.   *
 *       Returns a zero pointer if no temporal memory was available.       *
 *                                                                         *
 ***************************************************************************/

struct Vector    *ContraGradient (struct LiegroupClass *Group, struct Vector *Weight)

{
int            i;
struct Vector *ContraGradientWeight;


ContraGradientWeight = CreateVector (Weight->length);

if (ContraGradientWeight)
   {
   for (i = 0; i < Weight->length; i++)    /* Default is Weight = ContraGradientWeight */
      ContraGradientWeight->elements[i] = Weight->elements[i];

   switch (Group->type)
      {
      case 'A':
         for (i = 0; i < Weight->length; i++)
            ContraGradientWeight->elements[i] = Weight->elements[Weight->length - i - 1];
      break;

      case 'D':
         if ((Group->rank % 2) != 0)
            {
            ContraGradientWeight->elements[Weight->length - 2] = Weight->elements[Weight->length - 1];
            ContraGradientWeight->elements[Weight->length - 1] = Weight->elements[Weight->length - 2];
            }
      break;

      case 'E':
         if (Group->rank == 6)
            {
            ContraGradientWeight->elements[0] = Weight->elements[5];
            ContraGradientWeight->elements[2] = Weight->elements[4];
            ContraGradientWeight->elements[4] = Weight->elements[2];
            ContraGradientWeight->elements[5] = Weight->elements[0];
            }
      break;
      }
   }

return (ContraGradientWeight);
}



/***************************************************************************
 *                                                                         *
 *  Function name : PermuteVector ()                                       *
 *                                                                         *
 *  Description : Reads the input vector and stores this vector in the     *
 *                output vector due to the permutation vector.             *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : PermuteVector (*OutVector, *InVector, Permutation)          *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct Vector) *OutVector:                                          *
 *       Pointer to the permutated vector.                                 *
 *    (struct Vector) *InVector:                                           *
 *       Pointer to the input vector.                                      *
 *    (struct Vector) *Permutation:                                        *
 *       Pointer to the permutation.                                       *
 *                                                                         *
 *  Return value :                                                         *
 *    (struct Vector)                                                      *
 *       Same pointer as *OutVector.                                       *
 *                                                                         *
 ***************************************************************************/

struct Vector  *PermuteVector (struct Vector *OutVector, struct Vector *InVector, struct Vector *Permutation)

{
int   i;

for (i = 0; i < OutVector->length; i++)
   OutVector->elements[i] = InVector->elements[Permutation->elements[i]];

return (OutVector);
}

/*--------------------------- END OF FILE ---------------------------------*/

