/***************************************************************************
 *                                                                         *
 *  Filename : Weyl.c                                                      *
 *                                                                         *
 *  Description : Routine for evaluating the Weyl dimension formula.       *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *                         Modification History                            *
 *                                                                         *
 *  Date      Author       Comments                                        *
 * ----------------------------------------------------------------------  *
 * 10.4.93    R.Boedi      Created.                                        *
 * 22.5.93    R.Boedi      Adapted code to new struct Matrix definition.   *
 *  5.8.93    R.Boedi      Changed memory allocation for vectors and       *
 *                         matrices, see file Memory.c                     *
 * 11.8.93    R.Boedi      Added handling for a NULL weight vector.        *
 * 23.8.93    R.Boedi      Now using a static matrix for the positive      *
 *                         roots for speed-up.                             *
 * 29.3.94    R.Boedi      Changed error return codes in DimIrrRep() to    *
 *                         distinguish between overflow errors and no      *
 *                         memory errors.                                  *
 * 30.3.94    MJ/RB        Changed return value of DimIrrRep() to enum     *
 *                         ErrorType.                                      *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 * Copyright (C) 1993 Michael Joswig, Richard Boedi.  All rights reserved. *
 *                                                                         *
 ***************************************************************************/

/*----------------------------- INCLUDES ----------------------------------*/

#include "IrrReps.h"
#include "Protos.h"

/*------------------------------- CODE ------------------------------------*/

/***************************************************************************
 *                                                                         *
 *  Function name : DimIrrRep ()                                           *
 *                                                                         *
 *  Description : Calculates the dimension of an irreducible representation*
 *                which is given by its highest weight.                    *
 *                The algorithm is based on the dimension formula of Weyl. *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : DimIrrRep (*Group, *HWeight, *Dimension)                    *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct LiegroupClass) *Group:                                       *
 *       Pointer to a LiegroupClass structure which contains the data of   *
 *       the group.                                                        *
 *    (struct Vector) *HWeight:                                            *
 *       Pointer to a Vector structure which contains the heighest weight  *
 *       vector of the irreducible representation.                         *
 *    (unsigned long) *Dimension:                                          *
 *       The dimension of the irreducible representation.                  *
 *                                                                         *
 *  Return value :                                                         *
 *    (enum ErrorType)                                                     *
 *       NO_ERROR, OVERFLOW_ERROR, MEM_ERROR.                              *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 * Note: since we are not using big integers for evaluating the Weyl       *
 *       formula, we use the DivideInteger() routine in order to be able   *
 *       to compute dimensions up to MAX_ULONG.                            *
 *       This is possible, since the variables denom_factor and nom_factor *
 *       only exceed the value MAX_ULONG if the final dimension exceeds    *
 *       MAX_ULONG. For a proof look at the table below.                   *
 *       To check for overflow it must be assured that the maximum double  *
 *       value is at least MAX_ULONG * MAX_ULONG.                          *
 *       Let m be the maximal coefficient of the vector HWeight. Then      *
 *                                                                         *

algebra   pos. root of        maximum       maximum     min. dimension of
           max. norm        denom_factor   nom_factor   the representation
=============================================================================
  An      (1,1,...,1)           n           m+n         m+n
  Bn      (1,2,...,2)           2n-1        2m+2n-1     2m+2n-1 (if n>2 or m>1),
  Cn      (2,...,2,1)           2n-1        2m+2n-1     2n (if m=1), 2m+2n (if m>1)
  Dn      (1,2,...,2,1,1)       2n-3        2m+2n-3     2m+2n-2
  E6      (1,2,2,3,2,1)         11          3m+11       3m+24
  E7      (2,2,3,4,3,2,1)       17          4m+17       4m+52
  E8      (2,3,4,6,5,4,3,2)     29          6m+29       6m+242
  F4      (2,3,4,2)             11          4m+11       4m+22
  G2      (3,2)                 5           3m+5        3m+21 (if m>1)

 *                                                                         *
 ***************************************************************************/

enum ErrorType    DimIrrRep (struct LiegroupClass *Group, struct Vector *HWeight, unsigned long *Dimension)

{
static char             stype = 0;
static unsigned long    srank = 0;
static struct Matrix   *SPRoots = NULL;
enum ErrorType          ReturnCode = NO_ERROR;

unsigned long  denom_factor, nom_factor;
unsigned long  n, i, j, temp;
unsigned long  rank = Group->rank;
struct Vector *Norms, *denom_vector, *nom_vector;


if (!VectorNull (HWeight))
   {
   n              = NoOfPositiveRoots (Group);
   nom_vector     = CreateVector (n);
   denom_vector   = CreateVector (n);
   Norms          = PositiveRootNorms (Group);

   if ((stype != Group->type) || (srank != Group->rank))
      {
      FreeMatrix (SPRoots);
      SPRoots = PositiveRoots (Group);
      stype = Group->type;
      srank = Group->rank;
      }

   if (nom_vector && denom_vector && Norms && SPRoots)
      {
      for (i = 0; i < n; i++)
         {
         for (denom_factor = 0, nom_factor = 0, j = 0; j < rank; j++)
            {
            temp          = SPRoots->prow[i][j] * Norms->elements[j];
            nom_factor   += temp * (1 + HWeight->elements[j]);
            denom_factor += temp;
            }
         nom_vector->elements[i]   = nom_factor;
         denom_vector->elements[i] = denom_factor;
         }

      *Dimension = DivideInteger (nom_vector, denom_vector);
      if (*Dimension == 0) ReturnCode = OVERFLOW_ERROR;
      }
   else
      ReturnCode = MEM_ERROR;

   FreeVector (Norms);
   FreeVector (denom_vector);
   FreeVector (nom_vector);
   }
else
   *Dimension = 1;

return (ReturnCode);
}



/***************************************************************************
 *                                                                         *
 *  Function name : DivideInteger ()                                       *
 *                                                                         *
 *  Description : Two integers are given as a vector of factors.           *
 *                The first integer must be a multiple of the second one.  *
 *                The quotient of these two integers is computed by        *
 *                iterated reduction of common factors.                    *
 *                Note that the contents of both vectors are modified!     *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : DivideInteger (*Nom, *Denom)                                *
 *                                                                         *
 *  Parameters :                                                           *
 *    (struct Vector) *Nom, *Denom:                                        *
 *       Vectors containing the factors of a product representation.       *
 *                                                                         *
 *  Return value :                                                         *
 *    (unsigned long)                                                      *
 *       The quotient of the first and the second integer.                 *
 *       If an overflow error is detected, then 0 is returned.             *
 *                                                                         *
 ***************************************************************************/

unsigned long  DivideInteger (struct Vector *Nom, struct Vector *Denom)

{
double         quotient = 0;
unsigned long  g;
int            i, j;
BOOL           Overflow;

for (i = 0; i < Denom->length; i++)
   for (j = 0; j < Nom->length; j++)
      {
      if (Denom->elements[i] > 1)
         {
         if (Nom->elements[j] > 1)
            {
            g = gcd (Nom->elements[j], Denom->elements[i]);
            if (g > 1)
               {
               Denom->elements[i]   /= g;
               Nom->elements[j]     /= g;
               }
            }
         }
      else
         break;
      }

for (i = 0, Overflow = FALSE; i < Denom->length; i++)
   if (Denom->elements[i] != 1)
      {
      Overflow = TRUE;
      break;
      }

if (!Overflow)
   for (i = 0, quotient = 1; i < Denom->length; i++)
      {
      quotient *= (double)Nom->elements[i];
      if (quotient > (double)ULONG_MAX)
         {
         quotient = 0;
         break;
         }
      }

return ((unsigned long)quotient);
}

/*--------------------------- END OF FILE ---------------------------------*/

