/***************************************************************************
 *                                                                         *
 *  Filename : gGen.c                                                      *
 *                                                                         *
 *  Description : This is something vbery peculiar. The average user of    *
 *                our Lie group representation package will not ever need  *
 *                it.                                                      *
 *                It just serves as a front end to Generate within some    *
 *                particular system of distributed computing.              *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *                         Modification History                            *
 *                                                                         *
 *  Date      Author       Comments                                        *
 * ----------------------------------------------------------------------  *
 * 21.9.93    M.Joswig     Created from Generate.c                         *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  Synopsis : gGen -dmin <dmin> -dmax <dmax> [-t {<Algebra>}] [-p <par>]  *
 *                                                                         *
 *  Parameters :                                                           *
 *       The same as for Generate; besides:                                *
 *       -p <par> where <par> denotes a positive natural number to         *
 *       determine how many parameters are wished for each call of         *
 *       Generate. The default is set to 5.                                *
 *                                                                         *
 *  Output :                                                               *
 *       gGen is meant to generate a list of calls to Generate (i.e. the   *
 *       necessary paramters) which can be distributed to several machines *
 *       via network.                                                      *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 * Copyright (C) 1993 Richard Boedi, Michael Joswig. All rights reserved.  *
 *                                                                         *
 ***************************************************************************/

/*----------------------------- INCLUDES ----------------------------------*/

#include "IrrReps.h"
#include "Protos.h"

/*------------------------- LOCAL PROTOTYPES ------------------------------*/

LGList MakeToDo (LGList, double);
LGList ExpandWild (struct LiegroupClass *, double);


/*------------------------------- CODE ------------------------------------*/

main (int argc, char *argv[])

{
unsigned long           dim_min = 0, dim_max = ULONG_MAX, i = 1, p = 5, cnt = 0;
long                    RepsExist, TempIn;
double                  dim_max_sq;
enum InputType          TheInput = INPUT_OK;
struct LiegroupClass    Group;
LGList                  GroupList = NULL, ToDo, LastNode, NodeToCheck;
BOOL                    WeAlreadyHadAp = FALSE;
char							dq = '"';

while ((i < argc) && (TheInput == INPUT_OK))
   {
	if (strcmp(argv[i],"-dmin") == 0)
      {
		if (((i + 1) < argc) && (dim_min == 0))
         {
			TempIn = atol(argv[i+1]);
			if ((TempIn > 0) && (TempIn <= dim_max)) dim_min = TempIn;
			else                                     TheInput = RANGE_ERROR;
			i += 2;
			}
		else TheInput = SYNTAX_ERROR;
		continue;
		}

	if (strcmp(argv[i],"-dmax") == 0)
      {
		if (((i + 1) < argc) && (dim_max == ULONG_MAX))
         {
			TempIn = atol(argv[i+1]);
			if ((TempIn > 0) && (TempIn >= dim_min)) dim_max = TempIn;
			else                                     TheInput = RANGE_ERROR;
			i += 2;
			}
		else TheInput = SYNTAX_ERROR;
		continue;
		}

	if (strcmp(argv[i],"-p") == 0)
      {
		if (!WeAlreadyHadAp)
         {
			TempIn = atol(argv[i+1]);
			if (TempIn > 0)  p = TempIn;
			else             TheInput = RANGE_ERROR;
			i += 2;
         WeAlreadyHadAp = TRUE;
			}
		else TheInput = SYNTAX_ERROR;
		continue;
		}

	if (strcmp(argv[i],"-t") == 0)
      {
		i++;
		while ((i < argc) && (argv[i][0] != '-') && (TheInput == INPUT_OK))
         {
			if ((TheInput = StringToLGClass (&Group, argv[i])) == INPUT_OK) {
				if (CheckType(&Group) != DONT_EXIST) {
					if (!(GroupList = LGConstruct (Group.type, Group.rank, Group.field, Group.index, GroupList)))
				   	TheInput = MEM_ERROR;
					}
				else TheInput = TYPE_ERROR;
				}
			i++;
			}
		continue;
		}

	TheInput = SYNTAX_ERROR; /* If you get here, then you skipped all the "continues". */
	}

switch (TheInput) {

	case INPUT_OK:
		/* If there are wildcards for the rank, then this is the maximum dimension to look for. */
		dim_max_sq = (double)dim_max * (double)dim_max;

		/* Expand the wildcards and delete double entries. */
		ToDo = GroupList = MakeToDo(GroupList, dim_max_sq);

		/* Scan the list of types to process. */
		while (ToDo != NULL) {
			if (cnt%p==0) printf("-dmin %lu -dmax %lu -t ",dim_min,dim_max);
         printf ("%c(%c,%lu,%d,%ld)%c ", dq, ToDo->LGC.type, ToDo->LGC.rank,
												    (int)ToDo->LGC.field,ToDo->LGC.index,dq);
			cnt++;
			if (cnt%p==0) printf("\n");
			ToDo = ToDo->NextNode;
			}
		break;

	case SYNTAX_ERROR:
		fprintf (stderr,"USAGE : %s -dmin <dmin> -dmax <dmax> [-t {<Algebra>}]\n", argv[0]);
		fprintf (stderr,":     dmin, dmax : determine the interval in which the dimensions of the\n");
		fprintf (stderr,":               irreducible representations are contained.\n");
		fprintf (stderr,":     Algebra    : specifies the types of algebras that should be processed.\n");
		fprintf (stderr,":                  Algebra is a vector (t,r,f,i) with\n");
		fprintf (stderr,":        t       : one of types A, B, C, D, E, F, G\n");
		fprintf (stderr,":        r       : the rank of the Liegroup (rank > 0)\n");
		fprintf (stderr,":        f       : the canonical field of the group (see Tits)\n");
		fprintf (stderr,":                    0 :   complex type\n");
		fprintf (stderr,":                    1 :   real numbers\n");
		fprintf (stderr,":                    2 :   complex numbers\n");
		fprintf (stderr,":                    4 :   quaternions\n");
		fprintf (stderr,":        i       : the index of the Liegroup\n");
		fprintf (stderr,":                  No blanks in the vector are allowed.\n");
		fprintf (stderr,":                  If you don't want to specify a component, simply write\n");
		fprintf (stderr,":                  an asterisk, e.g.\n");
		fprintf (stderr,":                  %s -dmin 15 -dmax 100 -t (A,*,1,1) (*,3,*,*)\n", argv[0]);
		fprintf (stderr,":                  will list all representations of dimension 15 up to 100\n");
		fprintf (stderr,":                  of the types A_{n,1}^R (where n is arbitrary) and of\n");
		fprintf (stderr,":                  all groups of rank 3.\n\n");
		break;

	case MEM_ERROR:
		fprintf(stderr,"Execution of %s terminated due to memory overflow.\n",argv[0]);
		break;

	case TYPE_ERROR:
		fprintf(stderr,"%s: not a valid type.\n",argv[0]);
		break;

	case RANGE_ERROR:
		fprintf(stderr,"%s: numerical parameters out of range.\n",argv[0]);

	default:
		fprintf(stderr,"%s: mysterious error code (%d).\n",argv[0],(int)TheInput);

	}

FreeLGList (GroupList);

return ((int)TheInput);
}




LGList   MakeToDo (LGList OldList, double dim_max_sq)

{

LGList						NewList = NULL, Temporary, SomeMore;
struct LiegroupClass		Group;

/* We destroy the old list and make a new one. At the same time we reverse the order. */
while (OldList != NULL) {
	Group = OldList->LGC;
	if ((Group.type != WildCard) && (Group.rank != 0)
				&& (Group.field != UNSPECIFIED) && (Group.index != NO_INDEX)) {
		/* No wildcards, so just bend some pointers. */
		Temporary = OldList;
		OldList = OldList->NextNode;
		/* Is it a new one? */
		if (!LGinList(&Group,NewList)) {
			Temporary->NextNode = NewList;
			NewList = Temporary;
			}
		else {
			free(Temporary);
			}
		}
	else {
		/* First we generate a second list. */
		SomeMore = ExpandWild(&Group,dim_max_sq);
		/* Process this list. */
		while (SomeMore != NULL) {
			Temporary = SomeMore;
			SomeMore = SomeMore->NextNode;
			/* Is it a new one? */
			if (!LGinList(&(Temporary->LGC),NewList)){
				Temporary->NextNode = NewList;
				NewList = Temporary;
				}
			else {
				free(Temporary);
				}
			}
		Temporary = OldList;
		OldList = OldList->NextNode;
		free(Temporary);
		}
	}

return (NewList);

}



LGList   ExpandWild (struct LiegroupClass *Group, double dim_max_sq)

{

LGList 						TheList = NULL;
struct LiegroupClass		ActualGroup;

/* select first possible type */
ActualGroup.type = 'A';
while (TRUE) {
	if (Group->type != WildCard) ActualGroup.type = Group->type;
	
	/* select first possible rank */
	switch (ActualGroup.type) {
		case 'A': ActualGroup.rank = 1; break;
		case 'B': ActualGroup.rank = 2; break;
		case 'C': ActualGroup.rank = 3; break;
		case 'D': ActualGroup.rank = 4; break;
		case 'E': ActualGroup.rank = 6; break;
		case 'F': ActualGroup.rank = 4; break;
		case 'G': ActualGroup.rank = 2; break;
		}
	while (ActualGroup.rank != 0) {
		if (Group->rank != 0) ActualGroup.rank = Group->rank;

		ActualGroup.field = COMPLEX;
		while (ActualGroup.field != UNSPECIFIED) {
			if (Group->field != UNSPECIFIED) ActualGroup.field = Group->field;
			
			ActualGroup.index = 0;
			while (ActualGroup.index != NO_INDEX) {
				if (Group->index != NO_INDEX) ActualGroup.index = Group->index;

				/* Now we have all we need. */
				if (CheckType (&ActualGroup) != DONT_EXIST)
					if (!(TheList = LGConstruct(ActualGroup.type,ActualGroup.rank,ActualGroup.field,
														 ActualGroup.index, TheList)))
				   	return(NULL); /* this is too cheap */
				
				/* Perhaps we have to switch to the next index. */
				if (Group->index == NO_INDEX)
					switch (ActualGroup.type) {
						case 'A':
						case 'B':
						case 'C': /* We prefer the naive guess in these cases. */
						case 'D': if (ActualGroup.index < ActualGroup.rank) ActualGroup.index++;
									 else ActualGroup.index = NO_INDEX;
									 break;
						case 'E': switch (ActualGroup.rank) {
										case 6: switch (ActualGroup.index) {
													case   0: ActualGroup.index = -78; break;
													case -78: ActualGroup.index = -26; break;
													case -26: ActualGroup.index = -14; break;
													case -14: ActualGroup.index =   2; break;
													case   2: ActualGroup.index =   6; break;
													default : ActualGroup.index = NO_INDEX;
												  	}
													break;
										case 7: switch (ActualGroup.index) {
													case    0: ActualGroup.index = -133; break;
													case -133: ActualGroup.index =  -25; break;
													case  -25: ActualGroup.index =   -5; break;
													case   -5: ActualGroup.index =    7; break;
													default  : ActualGroup.index = NO_INDEX;
												  	}
													break;
										case 8: switch (ActualGroup.index) {
													case    0: ActualGroup.index = -248; break;
													case -248: ActualGroup.index =  -24; break;
													case  -24: ActualGroup.index =    8; break;
													default  : ActualGroup.index = NO_INDEX;
												  	}
													break;
										default: ActualGroup.index = NO_INDEX;
									 	}
									  break;
						case 'F': switch (ActualGroup.index) {
										case   0: ActualGroup.index = -52; break;
										case -52: ActualGroup.index = -20; break;
										case -20: ActualGroup.index =   4; break;
										default : ActualGroup.index = NO_INDEX;
										}
									 break;
						case 'G': switch (ActualGroup.index) {
										case   0: ActualGroup.index = -14; break;
										case -14: ActualGroup.index =   2; break;
										default : ActualGroup.index = NO_INDEX;
										}
									 break;
						default:  ActualGroup.index = NO_INDEX;
						}
				else break;
				}
			
			/* Perhaps we have to switch to the next field. */
			if (Group->field == UNSPECIFIED)
				switch (ActualGroup.field) {
					case COMPLEX:	ActualGroup.field = R; break;
					case R:			ActualGroup.field = C; break;
					case C:			ActualGroup.field = H; break;
					default:			ActualGroup.field = UNSPECIFIED;
					}
			else break; /* leave the field loop */
			}

		/* Perhaps we have to switch to the next rank. */
		if ((Group->rank == 0)) {
			switch (ActualGroup.type) {
				case 'A':
				case 'B':
				case 'C':
				case 'D': if (GroupDimension(&ActualGroup) < dim_max_sq) ActualGroup.rank++;
							 else ActualGroup.rank = 0;
							 break;
				case 'E': if (ActualGroup.rank < 8) ActualGroup.rank++;
							 else ActualGroup.rank = 0;
							 break;
				default: ActualGroup.rank = 0;
				}
			}
		else break; /* leave the rank loop */
		}
	
	/* Perhaps we have to switch to the next type. */
	if ((Group->type == WildCard) && (ActualGroup.type < 'G')) ActualGroup.type++;
	else break; /* leave the type loop */
	}
	
return (TheList);
}


/*--------------------------- END OF FILE ---------------------------------*/
